#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

 LeARN_AddSoftware.pl - Add a software providing result in gff format, modify Template_LeARN.cfg

=head1 SYNOPSIS


 ./bin/LeARN_AddSoftware.pl --name=wonderful_prg --cmd=/usr/loca/bin/my_wonderfull_program --param "-X 3 -d /usr/local/db/my_db --version=0.01" 


=head1 DESCRIPTION

 	
 	Mandatory :
	--name	: Name of the program, which be available in web site
	--cmd	: path to program
	Optionnal :
	--help  : this message	
	--param : [''] string with parameters of the  programs.
	--version : [0.0] string coresponding to the version of the programs 
	--cfg 	: [$LEARN_DIR/cfg/Template_LeARN.cfg] template cfg to update

=cut

BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../lib");
}

use strict;
use warnings;
use Learn;
use ParamParser;
use Logger;
use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);

our $LOGGER_LEVEL_EXEC = 'DE';

MAIN:
{
	# Tout les parametres sont dans le cfg
	my $o_param_cli = New ParamParser('GETOPTLONG',"cfg=s", "name=s","cmd=s", "param=s", "version=s");
	$o_param_cli->SetUsage(my $usage=sub { &Usage(); } );

	$o_param_cli->AssertFileExists('cmd');
	$o_param_cli->AssertDefined('name');	
	$o_param_cli->SetUnlessDefined('param',"");
	$o_param_cli->SetUnlessDefined('version',"0.0");	
	#variable d'environement
	$o_param_cli->Update('ENV', "A");
	$o_param_cli->AssertDirExists('LEARN_DIR');
	my $learn_dir = $o_param_cli->Get('LEARN_DIR');

	#definition du cfg et chargement
	$o_param_cli->SetUnlessDefined('cfg',"$learn_dir/cfg/Template_Learn.cfg");
	$o_param_cli->AssertFileExists('cfg');
	my $new_cfg = $o_param_cli->Get('cfg');
	my $old_cfg = $o_param_cli->Get('cfg').".$$.old";

	# on fait une sauvegarde du fichier
	my $cmd = "cp -a $new_cfg $old_cfg";
	system $cmd;
	$o_param_cli->Update($old_cfg,'A');
	
	my $o_param_new = New ParamParser($old_cfg);
	my $type_program = &FindNextType(\$o_param_new);
	&PrintAndSetPriority(\$o_param_new,$type_program);
	my $prog_name=$o_param_cli->Get('name');
        $prog_name=~s/ /_/;

	open (OLD,  "$old_cfg");
	open (NEW,  ">$new_cfg");
	my $line;
	while ($line = <OLD>)
	{
		if ($line =~ /^#end ref name type/)
		{
			print NEW "$prog_name=$type_program\n";
		}
		if ($line =~ /^PROG_TO_LOAD/)
		{
			chomp ($line);
			$line .= " ".$type_program."\n";
		}
		if ($line =~ /^cli_PROG_TO_SCAN/)
		{
			$line ="cli_PROG_TO_SCAN=".$o_param_new->Get('cli_PROG_TO_SCAN');
		}
		if ($line =~ /^#end def scan program/)
		{
			print NEW $type_program."_name=".$prog_name."\n";
			print NEW $type_program."_cmd=".$o_param_cli->Get('cmd')."\n";
			print NEW $type_program."_param=".$o_param_cli->Get('param')."\n";
			print NEW $type_program."_version=".$o_param_cli->Get('version')."\n";
			print NEW $type_program."_type=".$type_program."\n\n";
		}

		print NEW $line;
	}

	print "Program added, you must run LeARN_Install.pl to propage updates\n";
}


=head2 function FindNextType

 Title        : FindNextType
 Prerequisite : none
 Function     : Find next name for scan program
 Args         : $msg : message to print before read online
 Return		  : prefix for key
=cut

sub FindNextType
{
	my ($ro_param_new)=@_;
	my @a_scan_keys=$$ro_param_new->GetKeys("SCAN_.*_name");
	for (my $i =0; $i<= $#a_scan_keys; $i++ )
	{
		$a_scan_keys[$i] =~ s/SCAN_//;
		$a_scan_keys[$i] =~s/_name//;
	}
	my @a_numbers = sort (@a_scan_keys);
	my $max = $a_numbers[$#a_numbers];
	$max++;
	return ("SCAN_$max");
	
		
}

=head2 procedure PrintAndSetPriority

 Title        : PrintAndSetPriority
 Prerequisite : none
 Function     : Print list of program in ordre of priority and get the number corresponding to the priority of the new program , recompute the string of priority
 Args         : none
 Return		  : prefix for key
=cut

sub PrintAndSetPriority
{
	my ($ro_param_new, $key)=@_;
	my @a_order_scan_program = split (' ',$$ro_param_new->Get('cli_PROG_TO_SCAN'));

	my $ok =0;
	my $order;
	while (!$ok )
	{
		print "#### Programs priority order ####\n";
		my $i=0;
		for ($i=0; $i<= $#a_order_scan_program; $i++)
		{
			my $num_to_print=$i+1;
			my $name=$$ro_param_new->Get($a_order_scan_program[$i]."_name");
			print "[$i] > $name > ";
			
		}
		print " [$i]\n";
		print "Enter the priority number for your program ? \n";		
		$order= <STDIN>;
		if (($order=~/\d+/) && $order <= ($#a_order_scan_program+1) )
		{
			$ok=1;
		}
		else
		{
			print "Not a valid number !\n";
		}
	}
	my $programs="";
	for (my $i=0; $i<= ($#a_order_scan_program+1); $i++)
	{
		
		if ($order == $i)
		{
			$programs.=$key." ";
		}
		$programs.=$a_order_scan_program[$i]." " if defined($a_order_scan_program[$i]);
		
	}
	$programs.="\n";
	$$ro_param_new->Set('cli_PROG_TO_SCAN', $programs);
}



=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
Add a software in the Template_Learn.cfg
[Mandatory]
	--name		string 		Name of the program (without spaces)
	--cmd		filename 	Program path

[Optionnal]
	--help  		 		This message	
	--param  	string		['']  Parameters of the  programs.
	--version	string		[0.0] String coresponding to the version of the programs 
	--cfg 	 	filename	[LEARN_DIR/cfg/Template_Learn.cfg] Template to be updated
END
}
