#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

 LeARN_AddUser.pl - Add an user, in a file with etcpasswd format for LeARN website

=head1 SYNOPSIS

Interactif mode :
 ./bin/LeARN_AddUser.pl 

Command line mode :
 ./bin/LeARN_AddUser.pl -login noirot -pass cece -privilege 1 -email celine.noirot@toulouse.inra.fr
 

=head1 DESCRIPTION

 	--help  : this message
	--cfg	: path configuration file (same as cgi)
	--login : login of user	
	--email : email of user
	--privilege :  [0] Number corresponding to the privileges defines LearnCgi.cfg : cgi_privilege_Number 

=cut

BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../lib");
}

use strict;
use warnings;
use Learn;
use ParamParser;
use Logger;
use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);

our $LOGGER_LEVEL_EXEC = 'DE';

MAIN:
{
	# Tout les parametres sont dans le cfg
	my $o_param = New ParamParser('GETOPTLONG',"cfg=s", "login=s","email=s", "privilege=s", "passwd=s");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	$o_param->SetUnlessDefined('cfg',"$dir/../cfg/LearnCgi.cfg");
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');

	# la premiere fois le fichier n'existe pas $o_param->AssertFileExists('cgi_file_etcpassword');
	my $etcpwd=$o_param->Get('cgi_file_etcpassword');
	$o_param->SelectNameSpace("cgi_privilege_");
	my @a_key_privilege=$o_param->GetKeys('\d+');
	my @a_privilege;
	foreach my $key (@a_key_privilege)
	{
		$a_privilege[$key]=$o_param->Get($key);
 	}
	$o_param->SelectNameSpace("");
	
	print "********* NEW USER ********* \n";
	print "x to exit \n";
	my %h_users;
	system "touch $etcpwd " if ( ! -e $etcpwd);
	open (PWD, $etcpwd) or print ">$etcpwd< does not exist ";
	while (my $line =<PWD> )
	{
		chomp($line);
		#cnoirot:b4ea23a623e058f392f1fe4:1575:1:celine.noirot@inra.toulouse.fr:/www/LeARN_dev/web/tmp:/bin/tcsh
		next if ($line eq "");
		my @a_user=split (/:/,$line );
		$h_users{$a_user[0]}=$line;
	}
	close (PWD);
	my $login="";
	my $ok=0;
	
	# Login
	$login=$o_param->Get('login');
	while (!$ok )
	{
		if ( &ToBeRead ($login) )	
		{
			$login=ReadLine("Enter login :");		
		}
		else
		{
			$login=$o_param->Get('login');
		}
		$ok=1;
		if ( defined $h_users{$login} )
		{
			print "User already define \n";
			$login=undef;
			$ok=0;
		}
	}

	# password
	my ($mdp1, $mdp2);
	if ($o_param->IsDefined('passwd'))
	{
		$mdp1=$o_param->Get('passwd');
	}
	else
	{
		$ok=0;
		while (!$ok)
		{
			$mdp1=ReadLine("Enter password    :");
			$mdp2=ReadLine("Re-enter password :");
			if ( ($mdp1 eq  $mdp2) && !($mdp1 eq "") )
			{
				$ok=1;
			}
			else
			{
				print "Error on password\n";
			}
	
		}
	}
	
	# privilege
	$ok=0;
	my $priv=$o_param->Get('privilege');
	while (!$ok )	
	{
		if ( &ToBeRead ($priv))
		{
			print "\nAvailable privilege level : \n";
			foreach my $i (	0 .. $#a_privilege )
			{
				print " $i :\t $a_privilege[$i]\n";
			}
			$priv=ReadLine("Enter privilege level:");
		}

		if (  (defined ($priv) ) && ($priv>=0) && ($priv <= $#a_privilege) )
		{
			$ok=1;
		}
		else
		{
			print "Error on privilege level\n";
			$priv=undef;
		}
		
	}

	# email
	$ok=0;
	my $email=$o_param->Get('email');
	while (!$ok)
	{
		if (&ToBeRead ($email))
		{
			$email=ReadLine("Enter email :");
		}
		if ($email =~  /.+@.+/)
		{
			$ok=1;
		}
		else
		{
			print "Error on email\n";
			$email=undef;
		}
		
	}
	
	open (PWD, ">>$etcpwd");
	my $wks=$o_param->Get('dir_learn')."/".$o_param->Get('cgi_dir_wks')."/$login";
	print PWD "$login:".md5_hex($mdp1).":none:$priv:$email:$wks:none\n";
	close PWD;
	print "User $login added in $etcpwd\n";
}


=head2 function ReadLine

 Title        : ReadLine
 Prerequisite : none
 Function     : Read line on stdin
 Args         : $msg : message to print before read online
 Return		  : string read.
=cut


sub ReadLine
{
    my ($msg)=@_;
	print "$msg \n";
	my $var=<STDIN> ;
	chomp($var);
	if ( "\U$var" eq 'X' )
	{
		print "Abort by user\n";
		exit;
	}
	return ($var);
} 


=head2 function ToBeRead

 Title        : ToBeRead
 Prerequisite : none
 Function     : Test if parameter is initialise or not
 Args         : $var : variable to be test
 Return		  : 0 if variable is inialise
 				1 if variable must be initialise
=cut

sub ToBeRead
{
    my ($var)=@_;
	if ((!defined ($var)) || ($var eq ""))
	{ 
		return 1;
	}
	else
	{
		return 0;
	}
} 


=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
Add an user for web site

[Mandatory]
        none , interactive mode
[Optionnal]
 	--help                     this message
	--cfg	       filename    path configuration file (same as cgi)
	--login        string      login of user	
	--email        string      email of user 
	--privilege    Number      [0] Number corresponding to the privileges defines LearnCgi.cfg : cgi_privilege_Number 

END
}
