#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

LeARN_Install.pl - Create from Template config file, several adequate configuration files for LeARN application and LeARN web site , and download Rfam databe in release_template.

=head1 SYNOPSIS

 

=head1 DESCRIPTION


=cut

BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../lib");
}

use strict;
use warnings;
use ParamParser;
use Logger;
use Template_Var;

our $LOGGER_LEVEL_EXEC = 'DE';
#our $o_param ;
MAIN:
{
	# Tout les parametres sont dans le cfg
	my $o_param = New ParamParser('GETOPTLONG',"learn_dir=s", "learn_url=s", "document_root=s","help");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );

	$o_param->AssertDirExists('learn_dir');
	my $learn_dir=$o_param->Get('learn_dir');
	
	# clean param learn_dir
	chop($learn_dir) if ($learn_dir =~ /\/$/);
	my $learn_url=$o_param->Get('learn_url');
	if ($learn_url !~ /^http:\/\//)
	{
                &Usage();
		die "Error on server name";
	}
	# clean url 
	chop($learn_url) if ($learn_url =~ /\/$/);
	my ($value_web_dir) = $learn_url =~ /http:\/\/[^\/]+(.+)/;

	#http://cat.toulouse.inra.fr/~cnoirot/cgi-bin/LeARN/cgi-bin/learn.cgi
	#/home/cnoirot/public_html/cgi-bin/LeARN
	#/home/cnoirot/public_html
	# I want to be able to rebuild : /~cnoirot/cgi-bin/LeARN
	# try to find the index for sub string 	
        #$o_param->AssertDirExists('document_root');
	#my $document_root=$o_param->Get('document_root');
	
	
	my $value_substr=rindex($learn_dir,$value_web_dir)+1; 
	
	# LEs masques :
	my $mask_learn_dir="MASK_LEARN_DIR";  # /www/db/LeARN
	my $mask_url="MASK_URL"; 	# http://symbiose.toulouse.inra.fr/db/LeARN
	my $mask_learn_web="MASK_LEARN_WEB"; # /db/LeARN
	my $mask_substr="MASK_SUBSTR"; 	# length ("/www/db") + 1

	# LeARN_Release	
	my $learn_cfg_pipeline=$learn_dir."/cfg/Template_Learn.cfg";
	$o_param->Update($learn_cfg_pipeline,'A');

	my $nb_errors = 0;

	foreach my $prg_key ("R_FOLD", "SCAN_2", "ISCAN")
	{
		my $prg=$o_param->Get($prg_key.'_cmd');
		$prg=~s/$mask_learn_dir/$learn_dir/g;
		my @a_prg = split(";",$prg);
		$prg=$a_prg[$#a_prg] if ($#a_prg > -1 );
		$nb_errors += &TestPrg ( $prg, $learn_cfg_pipeline,$o_param->Get($prg_key.'_name'), $prg_key.'_cmd');
	}

	foreach my $prg_key ("R_FOLD_ALT","R_PLOT_EVAL" , "RFL_FOLD", "RFL_ALIGN", "RFL_BUILD", "RFL_SEARCH", "R_CONV_B2CT", "R_CONV_CT2B", "SCAN_1", "SCAN_3")
	{
		my $prg=$o_param->Get($prg_key.'_cmd');
		$prg=~s/$mask_learn_dir/$learn_dir/g;
		$nb_errors += &TestPrg ( $prg, $learn_cfg_pipeline, $o_param->Get($prg_key.'_name'), $prg_key.'_cmd');
	}

	my $prg_xsl=$o_param->Get('xsltproc_prg');
	$prg_xsl=~s/$mask_learn_dir/$learn_dir/g;
	$nb_errors += &TestPrg ( $prg_xsl, $learn_cfg_pipeline,'xsltproc', 'xsltproc_prg');		

	# LeARN_Cgi
	my $learn_cfg_cgi=$learn_dir."/cfg/Template_LearnCgi.cfg";
	$o_param->Update($learn_cfg_cgi,'A');
	my $prg_xsl_cgi=$o_param->Get('cgi_prg_xsl');
	$prg_xsl_cgi=~s/$mask_learn_dir/$learn_dir/g;
	my @a_prg_xsl=split (' ', $prg_xsl_cgi);
	$prg_xsl_cgi= $a_prg_xsl[0];
	$nb_errors += &TestPrg ( $prg_xsl_cgi, $learn_cfg_cgi,'xsltproc', 'cgi_prg_xsl');

	
	# LeARN variables 
	my $var_file=$learn_dir."/lib/Template_Var.pm";
	$nb_errors += &TestPrg ( $Template_Var::CONVERT_CMD, $var_file,'convert', 'CONVERT_CMD');		
	
	my $cmd_reformat=$Template_Var::CMD_REFORMAT;
	$cmd_reformat=~s/$mask_learn_dir/$learn_dir/g;
	$nb_errors += &TestPrg ( $cmd_reformat, $var_file,'sreformat', 'CMD_REFORMAT');

	die("You must correct all errors and then run again the current program\n") if ( $nb_errors );

	# LeARN config files :
	my $xml_file=$learn_dir."/web/xml/Template_learn.xml";
	my $learn_cfg_rnaploteval=$learn_dir."/cfg/Template_RNAPlotEval.cfg";
	my $learn_cfg_rnalfold=$learn_dir."/cfg/Template_RNALocalFold.cfg";
	
	foreach my $file ( $learn_cfg_cgi, $var_file,$xml_file, $learn_cfg_rnaploteval, $learn_cfg_rnalfold )
	{
		my $file_out= $file;
		$file_out=~s/Template_//;
		print "Writing : $file_out\n";
		open (IN,$file);
		open (OUT,">$file_out");
		while ( my $line = <IN>)
		{
			$line=~s/Template_//;
			$line=~s/$mask_learn_dir/$learn_dir/g;
			$line=~s/$mask_url/$learn_url/g;
			$line=~s/$mask_learn_web/$value_web_dir/g;
			$line=~s/$mask_substr/$value_substr/g;
			print OUT $line;
		}
		close (IN);
		close (OUT);
	}

	# A partir d'un template : Template_Learn on creer 2 cfg : Learn.cfg et ForCgi.cfg

	my $file=$learn_cfg_pipeline;

	my $file_out1=$learn_cfg_pipeline;
	$file_out1=~s/Template_//;

	my $file_out2=$learn_dir."/cfg/ForCgi.cfg";
	open (IN,$file);
	open (OUT1,">$file_out1");
	open (OUT2,">$file_out2");
	print "Writing : $file_out1\n";
	print "Writing : $file_out2\n";
	while ( my $line = <IN>)
	{
		$line=~s/Template_//;
		$line=~s/$mask_learn_dir/$learn_dir/g;
		$line=~s/$mask_url/$learn_url/g;
		$line=~s/$mask_learn_web/$value_web_dir/g;
		$line=~s/$mask_substr/$value_substr/g;
		if ($line=~/^cgi_/)
		{
			$line=~s/^cgi_//;
			print OUT2 $line;
		}
		elsif ($line=~/^cli_/)
		{
			$line=~s/^cli_//;
			print OUT1 $line;
		}
		else
		{
			print OUT1 $line;
			print OUT2 $line;
		}
		
	}
	close (IN);
	close (OUT1);
	close (OUT2);
	
	system("chmod 777 $learn_dir/web/tmp $learn_dir/web/workspace");
	print "Get and install the Rfam database now ? [Y/N]\n";
	my $response =<STDIN>;
	my $cfg_file=$learn_cfg_pipeline;
	$cfg_file=~s/Template_//;
	my $o_param_learn= New ParamParser($cfg_file);
 
	if ($response =~ /Y/i)
	{
		# first time in template release
		my $rfam_dir=$o_param_learn->Get('path_old_release')."/".$o_param_learn->Get('path_db_rfam');
		mkdir($rfam_dir);
		chdir($rfam_dir);
		my $ftp_site=$o_param_learn->Get('rfam_ftp_site');
		my $rfam_thr=$o_param_learn->Get('ftp_file_rfam_thr');
		my $rfam_fasta=$o_param_learn->Get('ftp_file_rfam_fasta');
		my $rfam_models=$o_param_learn->Get('ftp_file_rfam_models');
				
		my $cmd="wget $ftp_site/$rfam_thr";
		print "Index for rfam_scan.pl : ".$cmd."\n";
		system $cmd;
		$cmd="wget $ftp_site/$rfam_fasta";
		print  "Sequences : ". $cmd."\n";
		system $cmd;
		$cmd="wget $ftp_site/$rfam_models";
		print "Models : ". $cmd."\n";
		system $cmd;

		$cmd="gzip -df $rfam_thr; gzip -df $rfam_fasta; gzip -cfd $rfam_models | tar xvf -";
		print "Decompress : ". $cmd."\n";
		system $cmd;

		$cmd="rm -f $rfam_thr $rfam_fasta  $rfam_models";
		print "Clean : ". $cmd."\n";
		system $cmd;
	
		$cmd=$o_param_learn->Get('FORMAT_cmd')." Rfam.fasta ".$o_param_learn->Get('FORMAT_param');
		print "Format db  : ". $cmd."\n";
		system $cmd;

	}
	
	# On charge le cfg du cgi pour recuperer le path du fichier d'authentification au format etcpasswd
	$file=$learn_cfg_cgi;
	$file=~s/Template_//;
	my $o_param_cgi= New ParamParser($file);
	
	my $auth=$o_param_cgi->Get('cgi_file_etcpassword');

	if (! -e $auth)
	{
		my $cmd = $learn_dir ."/bin/LeARN_AddUser.pl --login demo1 --passwd demo1 --priv 0 --email demo1\@test ";
		system $cmd;
		$cmd = $learn_dir ."/bin/LeARN_AddUser.pl --login demo2 --passwd demo2 --priv 1 --email demo2\@test ";
		system $cmd;
	}
	
	print "Now you must configure your web server with lines like that:\n";
	

	print "The LeARN instance is now configured\n";
	print "Now you must configure your web server with lines like that:\n";
	printf "<Directory \"%s\">\n",$o_param->Get('learn_dir');
	print "\tOptions +ExecCGI  -Indexes +FollowSymLinks\n";
	print "\tOrder deny,allow\n";
	print "\tAllow from all\n";
	print "</Directory>\n";
	printf "And then play with LeARN at:\n%s/cgi-bin/learn.cgi\n",$o_param->Get('learn_url');

}

sub TestPrg
{
my ($file, $template, $name, $variable_name )=@_;
	

	$file =~ s/^\s*//;
	$file =~ s/\s*$//;
	if ( ! -e $file )
	{
		print ("Prg: $file doesn't exist, ");
		print "please specify a valid path for program $name in template file : $template, ";
		print "key '$variable_name' \n";
		return 1;
	}
	return 0;

}
=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
Init the template file, with path.
[Mandatory]
	--learn_dir	directory   Path to learn intall
                                    eg : /www/db/LeARN
	--learn_url	url         eg : http://symbiose.toulouse.inra.fr/db/LeARN

[Optionnal]
	--help  	            this message
	--document_root directory   apache document root /www still not use.
END
}
