#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


BEGIN
{
       my ($dirprg,$nameprg) = $0 =~ /(.+)\/(.+)/;
       push(@INC,"$dirprg/../lib");
}

use strict;
use warnings;
use Learn;
use ParamParser;
use Logger;
our $LOGGER_LEVEL_EXEC = 'DE';
use Var;
#use vars qw($Var::E_XML );

MAIN:
{
	# Tout les parametres sont dans le cfg
	my $rh_param = New ParamParser('GETOPTLONG',"cfg=s", "createtree", "copy","db_opt=s", "input_fasta=s", "log=s", "families=s");
	$rh_param->AssertFileExists('cfg');
	$rh_param->Update($rh_param->Get('cfg'),'A');
	$rh_param->SetUnlessDefined('LOGGER_LEVEL_EXEC',$LOGGER_LEVEL_EXEC);
	# cr� le logger
	my $logger = New Logger ($rh_param->Get('log'),$LOGGER_LEVEL_EXEC,'NOCOMPRESSION');
	$logger->Trace("Begin Learn :\n",$logger->CalcStamp,"DE");
	
	# cr� un learn
	my $application= New Learn($rh_param, \$logger);
	$rh_param->Set('createtree',1);
	$rh_param->Set('db_opt',"in");
	$application->Initialize($rh_param->Get('createtree'), $rh_param->Get('db_opt')); # crée l'arborescence

	$application->CreatePrograms(); # blastn / rfam_scan / trnaScan-SE / rnafold / mfold
	my @lst_of_family=split(";",$rh_param->Get('families'));
	my $cmd;	
	$application->LoadReleaseFromListOfFamily("",@lst_of_family);

	if ($rh_param->IsDefined('copy') && $rh_param->Get('copy') == 1 )
	{
		my $current_dbs=$application->{'__current_release'}."/".$rh_param->Get('path_db');
		my $old_dbs=$application->{'__old_release'}."/".$rh_param->Get('path_db');
		$cmd = "cp -a -R $old_dbs/* $current_dbs";
		system $cmd ;
	}

	my $min_acc_fam=&MinElt(@lst_of_family);
	${$application->{'__h_family'}}{$min_acc_fam}->SetProgStructure(\${$application->GetProgram("RFL_FOLD","actual")});
	foreach my $acc (@lst_of_family)
	{
		next if ($acc eq $min_acc_fam);
		${$application->{'__h_family'}}{$min_acc_fam}->MergeWithFamily(\${$application->{'__h_family'}}{$acc}, $application->{'__current_release'}."/".$rh_param->Get('path_db_rfam'));
		delete (${$application->{'__h_family'}}{$acc});

		my $rnaml_file = $application->{'__current_release'}."/".$rh_param->Get('path_db_learn_family_xml')."/$acc$Var::E_XML";
		$cmd = "rm -rf $rnaml_file";
		system $cmd ;
	}
	${$application->{'__h_family'}}{$min_acc_fam}->SetContextDir($application->{'__current_release'}."/".$rh_param->Get('path_db_rfl'));
	${$application->{'__h_family'}}{$min_acc_fam}->BuildCm($application->{'__current_release'}."/".$rh_param->Get('path_db_rfam'), $application->{'__current_release'}."/".$rh_param->Get('path_db_rfl_intern')."/$min_acc_fam.cm");
	${$application->{'__h_family'}}{$min_acc_fam}->ImproveSS();
	${$application->{'__h_family'}}{$min_acc_fam}->SaveRnaml($application->{'__current_release'}."/".$rh_param->Get('path_db_learn_family_xml'),$application->{'__current_release'}."/".$rh_param->Get('path_db_learn_family_tmp'));
	$application->SaveRnasToRnaml();


	my $rnaml_file = $application->{'__current_release'}."/".$rh_param->Get('path_db_learn_family_xml')."/$min_acc_fam$Var::E_XML";
	$cmd="chmod -R 755 ". $application->{'__current_release'};
	system $cmd;	
	$logger->Trace("End Learn :\n",$logger->CalcStamp,"DE");

}


sub MinElt
{
	my (@a_fam)=(@_);
	my $min_elt=$a_fam[0];
	foreach my $acc (@a_fam) 
	{
		if ($acc lt $min_elt ) 
		{
			$min_elt=$acc;
		}
	}
	return ($min_elt);

}

=item Usage

	$rh_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:testlearn
$0	
[Mandatory]
        --cfg          filename  Learn.cfg file adapted to web tree
        --input_fasta  filename  fasta file 
        --families     string     list of family to merge
[Optionnal]
        --help     	         this message
        --createtree             if flag is set, tree directory will be create
        --db_opt       string    specify the type of database copy (Rfam and Blast)
                                 [cp|ln|none],this option is available if creatretree is set, 
        --log          filename  log file 
	
END
}
