#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


# cette version charge en memoire ARN et famille et charge un nouvel organisme
BEGIN
{
       my ($dirprg,$nameprg) = $0 =~ /(.+)\/(.+)/;
       push(@INC,"$dirprg/../lib");
}

use strict;
use warnings;
use Program;
use Logger;
use GFF;
use ParamParser;
use SecondaryStructure;
our $LOGGER_LEVEL_EXEC = 'DE';
our $CMD_MV="mv";
our $CMD_RM="rm -rf";
my $RNA_MIN_LG;
my ($LOGGER, $LOGGER_STAMP);

MAIN:
{
	# Tout les parametres sont dans le cfg
	my $o_param = New ParamParser('GETOPTLONG',"cfg=s", "input=s", "output=s", "format=s", "start=s", "end=s", "pRNAlfold=s");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );

	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');

	$LOGGER = New Logger ($o_param->Get('log'),$LOGGER_LEVEL_EXEC,'NOCOMPRESSION');
	$LOGGER_STAMP= $LOGGER->CalcStamp();


	my $name = $o_param->Get("R_FOLDL_name");
	my $cmd = $o_param->Get("R_FOLDL_cmd");
	my $param = $o_param->Get("R_FOLDL_pRNAlfold");
	my $version = $o_param->Get("R_FOLDL_version");
	my $type = $o_param->Get("R_FOLDL_type");

	my $rnalfold= New Program($name, $cmd, $param, $type, $version, \$LOGGER, $LOGGER_STAMP);
	$o_param->AssertFileExists('input');
	my $in=$o_param->Get('input');
	my $out=$o_param->Get('output');
	$rnalfold->SetInOut( $in, $out);
	$rnalfold->Run();
	my @a_struc;
	# Pour tout les structure locale on vas les stock�dans un tableau
	open (STRUC , $out);
	while (my $line=<STRUC>)
	{
		next if ($line =~ /[\>AUGCT]/i);
		chomp($line);
		# a revoir car espace entre paranthese de l'energy
		# .(((((((((.((....)).))))))))). (-10.10)  361
		my @a_line=split(/ /, $line, 2);
		my ($tmp_free,$begin) = $a_line[1]=~/\((.+)\)\s+(\d+)/;# recupère l'energie libre
		next if (not defined ($begin));
		push(@a_struc, [($tmp_free,$a_line[0],$begin)]);
		#print "PUSH\t Free $tmp_free\t str : ".$a_line[0]."\t beg : $begin\n";
	}
	close (STRUC);
	#trie selon l'energie libre.

	(@a_struc)=sort { $$a[0] <=> $$b[0] }  @a_struc;
	# recup�e les bornes de l'ancrange
	my $start_ancr = $o_param->Get("start");
	my $end_ancr = $o_param->Get("end");
	# on s'arrete �la premiere structure qui chevauche l'ancrage
	my $find=0;
	my ($start, $end);
	my $i=-1;
	#print "ANCR : $start_ancr - $end_ancr\n";
	#$#a_struc--; # car la derniere ligne du fichier contiens que l'info enegie libre de la sequence
	foreach $i (0..$#a_struc)
	{
		
		$start=$a_struc[$i][2];
		$end=$start+length($a_struc[$i][1]);
		if ( $start_ancr >= $start and $end_ancr  <= $end ) #mon ancrage est-il contenu ou chevauch�?
		{
			$find=$i;
			last;
		}
	}
	if ($find != -1)
	{
		my $under_struc=$a_struc[$find][1];
		my $seqio = Bio::SeqIO -> new( -file => $in, '-format' => 'Fasta' ) ;
		my $seq = $seqio->next_seq();
		my $sequence=uc($seq->seq());
		my $length=length($sequence);
		my $structure=sprintf ("%s%s%s",'.' x $start, $under_struc ,'.' x ($length-$end));
		print "LG : $start + ".length($under_struc)." + $length-$end = ".($length-$end).", seq : ".$length."\n";
		open (OUT, ">".$out);
			print OUT ">".$seq->id()."\n";
			print OUT $seq->seq()."\n";
			print OUT $structure;
			print OUT " (".$a_struc[$find][0].")\n";
		close OUT;
		if ($o_param->Get('format') eq 'mfold')
		{
			my $secondary_str = New SecondaryStructure();
			
			$secondary_str->SetStructure($structure);
			$secondary_str->SetSSFile($o_param->Get('ss'));
			$secondary_str->ConvertRnafoldToMfoldSSfile($in, $out);
		}
	}
	
}

=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
Run a folding program on sequence, and search the best local structure.
[Mandatory]
	--input 	filename        fasta file
	--format 	string          [mfold|rnafold] format
[Optionnal]
        --help     : this message
	--output	filename        output file 
        --start 	integer         start of anchor 
	--end		integer         end of anchor (the structure must contains or overlap anchor)
	--pRNAlfold     quoted string   parameters of RNALfold 
                                        (see: http://www.tbi.univie.ac.at/RNA/RNALfold.html)
END
}




