#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


BEGIN
{
       my ($dirprg,$nameprg) = $0 =~ /(.+)\/(.+)/;
       push(@INC,"$dirprg/../lib");
}

use strict;
use warnings;
use ParamParser;
use Program;
use Logger;
MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',"help","cfg=s", "input=s","output=s");
        $o_param->SetUsage(my $usage=sub { &Usage(); } );
	$o_param->AssertFileExists('cfg');
	$o_param->AssertFileExists('input');
	$o_param->Update($o_param->Get('cfg'),'A');

	my $logger = New Logger ($o_param->Get('log'),"DE",'NOCOMPRESSION');
	$logger->Trace("Begin Learn :\n",$logger->CalcStamp,"DE");
	$o_param->SelectNameSpace("R_PLOT");
	my $rnaplot= New Program(
					$o_param->Get("_name"), 
					$o_param->Get("_cmd"),
					$o_param->Get("_param"),
					$o_param->Get("_version"),
					$o_param->Get("_type"),
					\$logger,
					$logger->CalcStamp() );

	$o_param->SelectNameSpace();
	# cr� un image dans le repertoire courrant
	$rnaplot->SetInOut($o_param->Get('input'), "");
	$rnaplot->Run();
	$o_param->SelectNameSpace("R_EVAL");
	my $rnaeval= New Program(
					$o_param->Get("_name"), 
					$o_param->Get("_cmd"),
					$o_param->Get("_param"),
					$o_param->Get("_version"),
					$o_param->Get("_type"),
					\$logger,
					$logger->CalcStamp() );
	$o_param->SelectNameSpace();
	$rnaeval->SetInOut($o_param->Get('input'),$o_param->Get('output'));
	$rnaeval->Run();
}

=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
Run a plot asturcture according to a parenthesis structure and eval the free energie.
[Mandatory]
	--input 	filename        fasta file
        --output	filename        output file 
[Optionnal]
        --help          this message
	--cfg	        filename        configuration file 
END
}

