#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


BEGIN
{
       my ($dirprg,$nameprg) = $0 =~ /(.+)\/(.+)/;
       push(@INC,"$dirprg/../lib");
}

use strict;
use warnings;
use Learn;
use ParamParser;
use Logger;
our $LOGGER_LEVEL_EXEC = 'DE';

MAIN:
{
	# Tout les parametres sont dans le cfg
	my $rh_param = New ParamParser('GETOPTLONG',"cfg=s", "createtree","db_opt=s", "input_fasta=s", "log=s", "dir_rfam=s");
	$rh_param->SetUsage(my $usage=sub { &Usage(); } );
	$rh_param->AssertFileExists('cfg');
	$rh_param->Update($rh_param->Get('cfg'),'A');
	$rh_param->AssertFileExists('input_fasta');
	my $fasta=$rh_param->Get('input_fasta');
	$rh_param->Set('createtree',1);
	$rh_param->Set('db_opt','ln');

	$rh_param->SetUnlessDefined('LOGGER_LEVEL_EXEC',$LOGGER_LEVEL_EXEC);
	my $logger = New Logger ($rh_param->Get('log'),$LOGGER_LEVEL_EXEC,'NOCOMPRESSION');
	$logger->Trace("Begin Learn :\n",$logger->CalcStamp,"DE");

	my $application= New Learn($rh_param, \$logger);
	$application->Initialize($rh_param->Get('createtree'), $rh_param->Get('db_opt')); # crée l'arborescence
	$application->CreatePrograms(); # blastn / rfam_scan / trnaScan-SE / rnafold / mfold
	
	my $rfl_intern_rfam =$rh_param->Get('dir_rfam') ;
	my $pattern = $rh_param->Get('ISCAN_pattern_export');
	my $cmd_rfam=$Learn::H_PROGRAM_ACTUAL{'ISCAN'}->GetPrg();
	$cmd_rfam=~s/$pattern/$rfl_intern_rfam/;
	$Learn::H_PROGRAM_ACTUAL{'ISCAN'}->SetPrg($cmd_rfam);
	$Learn::H_PROGRAM_ACTUAL{'ISCAN'}->SetParam($Learn::H_PROGRAM_ACTUAL{'ISCAN'}->GetParam() ." -d $rfl_intern_rfam");
	$Learn::H_PROGRAM_ACTUAL{"SCAN_1"}=undef;
	$Learn::H_PROGRAM_ACTUAL{"SCAN_3"}=undef;
	$Learn::H_PROGRAM_ACTUAL{"SCAN_2"}=$Learn::H_PROGRAM_ACTUAL{"ISCAN"};
	$application->LoadSequences($fasta);
	$application->AnalyseSequences("LoadRna"); # => appele loadRnaL
	$application->PrintSequences();
	$application->ComputeAllRnaStructure();
	$application->PrintRnas();
	$application->SaveRnasToRnaml();
	$logger->Trace("End Learn :\n",$logger->CalcStamp,"DE");
}

=item Usage

	$rh_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:$0	
This program search for input sequence if it match with a family of LeARN db
[Mandatory]
	--input_fasta filename      fasta input file of one species
        --dir_rfam    directory     workspace or public learn

[Optionnal]
	--help                      this message
	--cfg         filename      [LEARN_DIR/cfg/ForCgi.cfg] full path of the configuration file
        --createtree                if flag is set, tree directory will be create
        --db_opt      string        specify the type of database copy (Rfam and Blast)
                                    [cp|ln|none],this option is available if creatretree is set, 
        --log         filename      log file 
END
}
