#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.



BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../lib");
}

use strict;
use warnings;
use ParamParser;
use Logger;

our $LOGGER_LEVEL_EXEC = 'DE';
#our $o_param ;
MAIN:
{
	# Tout les parametres sont dans le cfg
	my $o_param = New ParamParser('GETOPTLONG',"root=s" ,"release_dir=s", "new");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );

        $o_param->SetUnlessDefined('root', $ENV{LEARN_DIR});
	$o_param->AssertDirExists('root');
	
	my $learn_dir=$o_param->Get('root');
        $o_param->AssertDefined('release_dir');
	my $release_dir=$o_param->Get('release_dir');
	#if ( !-e "$learn_dir/$release_dir" )
#	{
#		die "Release does not exist : >$learn_dir/$release_dir< \n";
#	}

	my $file="$learn_dir/cfg/LearnCgi.cfg";

	if ( !-e "$file" )
	{
		&Usage();
		die "You must run install program 'InstallLeArn to create cfg file: >$file<\n";
	}

	my $file_out= $file;
	$file_out=~s/\.cfg/.new.cfg/;
	#print "in : $file; out: $file_out \n";
	open (IN,$file);
	open (OUT,">$file_out");
	while ( my $line = <IN>)
	{
		if ( $line =~ /cgi_dir_release=/)
		{
			$line = "cgi_dir_release=$release_dir\n";
		}
		print OUT $line;
	}
	close (IN);
	close (OUT);
	
	$o_param->Update($file_out,'A');
	if ( ! $o_param->IsDefined('new') )
	{
		system "cp $file_out $file ; rm $file_out";
		print "See release at : ". $o_param->Get('cgi_url')."/cgi-bin/learn.cgi\n";
	}
	else
	{		
		print "See tempory release at : ". $o_param->Get('cgi_url')."/cgi-bin/learn.cgi?__wb_cfg=$file_out\n";
	}
}

=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
Init the template file, with path.
[Mandatory]
	--root     	directory   Path to learn intall
                                    eg : /www/db/LeARN
	--release_dir	directory   relative to learn root dir eg: data/rel_template
        --new		            Do not erase current cfg
                                    initialize a web site for testing a new release
[Optionnal]
	--help  	            this message

END
}
