#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


BEGIN
{
       my ($dirprg,$nameprg) = $0 =~ /(.+)\/(.+)/;
       unshift (@INC,"$dirprg/../lib");
}

use Getopt::Long;
use ParamParser;
use Logger;
use strict;
use warnings;
use Program ;


MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',"cfg=s", "input=s", "output=s","program_type=s","repository_dir=s","overwrite_repository","path_new_release=s" );
	$o_param->SetUsage(my $usage=sub { &Usage(); } );
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');
        $o_param->AssertDefined('program_type');
        my $program_type=$o_param->Get('program_type');
	$o_param->AssertFileExists('input');
	my $seq_file=$o_param->Get('input');
        my $out_file=$o_param->Get('output');
        $o_param->AssertDirExists('repository_dir');
        my $repository_dir=$o_param->Get('repository_dir');
        my $overwrite_repository=$o_param->Get('overwrite_repository');
	my $analyse = New Program($o_param->Get($program_type.'_name'), $o_param->Get($program_type.'_cmd'), $o_param->Get($program_type.'_param'), $o_param->Get($program_type.'_type'), $o_param->Get($program_type.'_version'), undef , undef );
        # rfam
        if ($program_type eq 'SCAN_2')
        {
            my $rfam_db= $o_param->Get('path_new_release')."/".$o_param->Get('path_db_rfam');
	   $analyse->SetParam($analyse->GetParam()." -d $rfam_db");
        }
        # blast 
        if ($program_type eq 'SCAN_3')
        {
            my $blast_db=$o_param->Get('path_new_release')."/".$o_param->Get('path_db_blast')."/".$o_param->Get('file_name_fasta_db');
            $analyse->SetParam($analyse->GetParam()." -d $blast_db");
        }
        #rfam_scan_intern
	if ($program_type eq 'ISCAN')
	{

		my $rfl_intern_rfam = $o_param->Get('path_new_release')."/".$o_param->Get('path_db_rfl_intern');
		my $pattern = $o_param->Get('ISCAN_pattern_export');
		my $cmd_rfam=$analyse->GetPrg();
		$cmd_rfam=~s/$pattern/$rfl_intern_rfam/;
		$analyse->SetPrg($cmd_rfam);
		$analyse->SetParam($analyse->GetParam() ." -d $rfl_intern_rfam");
	}
        $analyse->SetRepository($repository_dir,$overwrite_repository);
  	$analyse->SetInOut($seq_file,$out_file);
        #print $analyse->GetCmd()."\n";
 	$analyse->Run();
}
=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:
$0	
	--help  : 	this message
	--cfg	: 	path configuration file
	--input :       input fasta file
        --output:       output file
        --program_type: ie SCAN_1
        --repository_dir: directory 
        --overwrite_repository : flag

END
}