#!/usr/bin/perl -I../lib
#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.


=head1 NAME

 get_SnoScan.pl - Launch snoscan and reformat output in gff format

=head1 SYNOPSIS


 ./bin/get_SnoScan.pl --input=file --output=file.gff 


=head1 DESCRIPTION

 	
 	Mandatory :
	--input		filename	sequence file
	--output	filename 	output file 
	Optionnal :
	--help  				this message	
	--cfg		filename 	[$LEARN_DIR/cfg/get_snoscan.cfg]
	
=cut

BEGIN
{
	my ($dir,$file) = $0 =~ /(.+)\/(.+)/;
	unshift (@INC,"$dir/../lib");
}

use strict;
use warnings;
use Learn;
use ParamParser;
use Logger;
use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);

our $LOGGER_LEVEL_EXEC = 'DE';

MAIN:
{
	# Tout les parametres sont dans le cfg
	my $o_param = New ParamParser('GETOPTLONG',"cfg=s", "input=s","output=s");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );

	$o_param->AssertFileExists('input');
	my $input=$o_param->Get('input');

	$o_param->SetUnlessDefined('output',$input.".gff" );
	my $output=$o_param->Get('output');

	my $output_tmp=$output.".tmp";
	
	#variable d'environement
	$o_param->Update('ENV', "A");
	$o_param->AssertDirExists('LEARN_DIR');
	my $learn_dir = $o_param->Get('LEARN_DIR');

	#definition du cfg et chargement
	$o_param->SetUnlessDefined('cfg',"$learn_dir/cfg/get_snoscan.cfg");
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');
	$o_param->AssertFileExists('cmd','rna_file');
	
	my $cmd= 	$o_param->Get('cmd')." ".$o_param->Get('param')." -o $output_tmp " . $o_param->Get('rna_file') ." $input";

        #print "$cmd\n";
	system ($cmd);
	&Reformat($output_tmp,$output); # TODO
        system ("grep -v \"ID=-\" $output | sort -u  -k 4,4 -n > $output_tmp; mv $output_tmp $output");
	
}

=head2 procedure Reformat

 Title        : Reformat
 Usage        : Reformat($output_tmp,$output);
 Function     : Reformat standars output of snoscan in gff format
 Returns      : None
 Args         : $output_tmp : snoscan ouput file
 				$output		: gff file
 
=cut 

sub Reformat
{
	my ($snofile,$gfffile) = @_;
	open (INFILE, $snofile);
	open (GFF, ">$gfffile");
	
	while (my $line = <INFILE>) 
	{
		my  ($seq_name,$sno_scores,$sno_start,$sno_end,$sno_match,$sno_methnuc,$sno_meth,$sno_meth_id  );
    	if ($line =~ /^\>\>\s(\S+)\s+(\S+)\s+\((\d+)\-(\d+)\)\s+Cmpl:\s(\S+)\-([ACGU]m)(\d+)\s\((\S+)\)/) 
    	{
    		#>> snR60  16.46  (11-113)  Cmpl: ySc-18S-Am636 (-)  10/1 bp  Gs-DpBox: 34 (24)  Len: 103
    		#snR60,16.46,11,113,ySc-18S,Am,636,-
    		#OU
    		#>> snR60  29.97  (11-113)  Cmpl: ySc-25S-Am815 (snR60)  11/1 bp  Gs-DpBox: 33 (23)  Len: 103
    		#snR60,29.97,11,113,ySc-25S,Am,815,snR60
    		
    		
 			($seq_name,$sno_scores,$sno_start,$sno_end,$sno_match,$sno_methnuc,$sno_meth,$sno_meth_id  )=($1,$2,$3,$4,$5,$6,$7,$8);
                        my ($beg,$end,$brin) = ( $sno_start < $sno_end ) ? ( $sno_start,$sno_end,'+' ) : ( $sno_end,$sno_start,'-' );
 			# <seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
 			print GFF "$seq_name\tsnoscan\tsnorna\t$beg\t$end\t$sno_scores\t$brin\t.\tAtt \"MATCH=$sno_match;METH_NUC=$sno_methnuc;METH=$sno_meth;ID=$sno_meth_id;\"\n";
    	}
	}
	close GFF;
	close INFILE;
	
}

=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:$0	

[Mandatory]
	--input		filename	sequence file
	--output	filename 	output file 

[Optionnal]
	--help  			this message	
	--cfg		filename 	[LEARN_DIR/cfg/get_snoscan.cfg]
END
}
