#!/usr/bin/perl

BEGIN
{
       my ($dirprg,$nameprg) = $0 =~ /(.+)\/(.+)/;
       unshift (@INC,"$dirprg/../../lib");
}

use strict;
use warnings;
use SecondaryStructure ;
use Range ;
use Sequence;
use Program;
use Rna;
use RnaFamily;
use Annotate;
use ParamParser;
use Logger;
MAIN:
{
	my $o_param = New ParamParser('GETOPTLONG',"cfg=s","path_old_release=s","path_new_release=s", "seq=s");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');
	$o_param->SetUnlessDefined('LOGGER_LEVEL_EXEC',"DE");
	
	$o_param->AssertFileExists('seq');
	my $seq_file=$o_param->Get('seq');
	
	my $logger = New Logger ($o_param->Get('log'),"DE",'NOCOMPRESSION');
	$logger->Trace("Begin Learn :\n",$logger->CalcStamp,"DE");
	
	my $application= New Learn($o_param, \$logger);

	$application->Initialize(1,"ln"); # crée l'arborescence
	$application->CreatePrograms();
	
    	my $rnafold= Learn::GetRefProgram("R_FOLD", "actual");
	my $convert= Learn::GetRefProgram("R_CONV_CT2B", "actual");

	my $seq= New Sequence("$seq_file");
	$seq->Initialize("Seq1","SeqTest","1",$o_param->Get('specie'),$application->{'__today'});
	$seq->SetSequence();
	my $seq2= New Sequence($seq_file);
	$seq2->Initialize("seq2","SeqTest","2",$o_param->Get('specie'),$application->{'__today'});
	$seq2->SetSequence();

	print "Creation d'un arn\n";	
	my $rna= New Rna();
	$rna->Initialize("RnaTest", "Test", "Petit ARN", $application->{'__today'}, "Automatic",$o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_rna'),$rnafold, $convert);
	$rna->SetPrincipalRange(1,36,"-",\$seq,\$$rnafold,$application->{'__today'} );
	
	$rna->ComputeStructure();

	my $rna2= New Rna();
	$rna2->Initialize("RnaTest2", "Test", "Petit ARN", $application->{'__today'}, "Automatic",$o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_rna'),$rnafold, $convert);
	$rna2->SetPrincipalRange(50,85,"+",\$seq,\$$rnafold,$application->{'__today'} );
	
	$rna2->ComputeStructure();
	

	
	$rna->SaveRnaml($o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_rna_xml'),$o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_rna_tmp'));
	
	$rna2->SaveRnaml($o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_rna_xml'),$o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_rna_tmp'));
	$application->AddRna(\$rna);
	$application->AddRna(\$rna2);
	
	${$application->{'__h_family'}}{'RFL00001'}= New RnaFamily();
	${$application->{'__h_family'}}{'RFL00001'}->Initialize("RFL00001", "Test", "implique dans ...", $application->{'__today'}, "Rfam_RF00001", "Annotated", $o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_family'), $o_param->Get('path_new_release')."/".$o_param->Get('path_db_rfl'),  Learn::GetRefProgram("RFL_FOLD", "actual"), Learn::GetRefProgram("RFL_ALIGN", "actual"), Learn::GetRefProgram("RFL_BUILD", "actual"));

	${$application->{'__h_family'}}{'RFL00001'}->AddRna(\${$application->{'__h_rna'}}{'RnaTest'});
	${$application->{'__h_family'}}{'RFL00001'}->AddRna(\${$application->{'__h_rna'}}{'RnaTest2'});
	${$application->{'__h_rna'}}{'RnaTest2'}->SetFamilyAcc('RFL00001');
	${$application->{'__h_rna'}}{'RnaTest'}->SetFamilyAcc('RFL00001');
	#$family->DumpFasta();
	#$family->DumpClustal();	

	#$family->ComputeStructure( );
	#$family->DumpStockholm();

	#$family->DumpMss();

	${$application->{'__h_family'}}{'RFL00001'}->BuildCm();
	${$application->{'__h_family'}}{'RFL00001'}->SaveRnaml($o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_family_xml'),$o_param->Get('path_new_release')."/".$o_param->Get('path_db_learn_family_tmp'));
	${$application->{'__h_family'}}{'RFL00001'}->Print();
	$application->DumpRflDb();
	my $res = $application->RnaBelongsFamily($rna2->GetId());	
	if (defined($res) ) 
	{
		print "Rna belongs !!! \n";
	}
	else
	{
		print "Rna NOT belong !!! \n";
	}

	
}
=item Usage

	$o_param->Usage();
	Print the usage of the program

=cut

sub Usage
{
print STDERR<<END
Usage:$0	
	--help  : 	this message
	--cfg	: 	path configuration file
	--path_old_release : 	path of old release
	--path_new_release : 	path of new release (to create)
	--seq   : 	path of a fasta sequence file

END
}








