<?xml version="1.0" encoding="iso-8859-1"?>
<!-- Affichage d'une famille a partir d'un fichier Rnaml -->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="html"/>
<xsl:param name="url"/>
<xsl:param name="link_rfam"/>
<xsl:param name="fnt_rna"/>
<xsl:param name="index_car_substr_img"/>
<xsl:template match="/">
	<xsl:apply-templates/>
</xsl:template>

	<!-- Demarage du premier tag : rnaml et traitement des noeuds suivants -->
	<xsl:template match="rnaml">
		<!-- Information de la famille-->
		<xsl:apply-templates select="molecule-class"/><br/>
		<!--  Analyses utilisé pour detection des arn (affichage en dernier) -->
		<h2 class="learn">Analyses : </h2>
		<div class="div_learn">
		<table>
		<xsl:apply-templates select="analysis"/>
		</table>
		</div>
	</xsl:template>

	<xsl:template match="molecule-class" >
	
		<h1 class="learn" >
			<!-- Affichage de l'accession de la famille en titre-->
			<xsl:value-of select="./numbering-system/@id"/>
		</h1>
		<h2 class="learn">Description: </h2>
		<div class="div_learn">
		<table>
		<tr>
			<td><label class="text1">Id:</label></td><td><xsl:value-of select="./identity"/></td>
			<td><label class="text2">Length:</label></td><td><xsl:value-of select="./numbering-system/numbering-range/length"/></td>
			
		</tr>
		<tr>
			<xsl:apply-templates select="/rnaml/database-entry" mode="lien_dbxref"/>
			<td><label class="text2">Number of RNA:</label></td>
			<td><xsl:value-of select="./alignment/@comment"/></td>
		</tr>
		<tr>
			<td><label class="text1">Status:</label></td>
			<td><xsl:value-of select="./identity/@comment"/></td>
			<xsl:apply-templates select="/rnaml/revision" />
		</tr>
		</table>		
		<label class="text1">Comment:</label>
		<xsl:value-of select="@comment"/>
		</div>
	
	<xsl:apply-templates select="consensus-molecule/molecule/structure"/>
	
	<xsl:apply-templates select="alignment"/>
	</xsl:template>

	<xsl:template match="/rnaml/database-entry" mode="lien_dbxref">
		<td><label class="text1">Dbxref:</label></td>
		<!--  on teste que la database entry correspond bien à le lien dbxref qui est dans /rnaml/molecule/@bd...-->
		<td>
		<xsl:if test="@id=/rnaml/molecule-class/@database-ids">
		<xsl:choose>
			<!--  On fait un lien sur rfam scan si dbxref=~Rfam -->
			<xsl:when test="./database='Rfam'">
				<xsl:element name='a'>
					<xsl:attribute name='href'>
						<xsl:value-of select="$link_rfam"/>
						<xsl:value-of select="./entry"/>
					</xsl:attribute>
					<xsl:attribute name='target'>
						<xsl:text>blank</xsl:text>
					</xsl:attribute>
					<xsl:value-of select="@id"/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="@id"/>
			</xsl:otherwise>
		</xsl:choose>		
		</xsl:if>
		</td>
	</xsl:template>
	
	<xsl:template match="length">
		<p class="node" ><label class="text1">Length:</label><xsl:value-of select="."/></p>
	</xsl:template>
	
	<xsl:template match="structure">
		<!-- Affichage la structure secondaire-->
		<h2 class="learn">Structure :</h2>
		<div class="div_learn">
		<table>
		<tr valign="top">
			<td><br/><label class="text1">Structure computed by: </label>	
			<xsl:value-of select="@analysis-ids"/><br/>
			<label class="text1">Free-energie:</label>
			<xsl:value-of select="./model/model-info/free-energy"/><br/><br/>
			<xsl:if test="contains(./model/str-annotation/@comment,'PARENTHESIS=')">
				<label class="text1">Parenthesis structure: </label><br/>
				<pre>
				<xsl:call-template name="cut">
					<xsl:with-param name="string" select="substring-before(substring-after(./model/str-annotation/@comment, '='), ' ')"/>
				</xsl:call-template>
				</pre>
			</xsl:if>
			</td>
			<td>
				<xsl:apply-templates select="./model/secondary-structure-display" />
			</td>
		
		
		</tr>
		
		</table>
		</div>
		
		
	</xsl:template>

	
	<xsl:template match="secondary-structure-display">
	<!-- pour afficher l'image, on crée un element IMG eavec les attributs correspondants-->
		<p class="img">
			<xsl:element name='img'>
				<xsl:attribute name='src'>
					<xsl:value-of select="substring(@comment,$index_car_substr_img,string-length(@comment))"/>
				</xsl:attribute>
				<xsl:attribute name='class'>ss</xsl:attribute> 		
			</xsl:element>
		</p>
	</xsl:template>

	<xsl:template match="alignment">
		
		<h2 class="learn">Alignment:</h2>
		<label class="text1">Computed by:</label><xsl:value-of select="@analysis-ids"/>
		<pre>
		<table>
			<tr>
				<td><xsl:text>Consensus:</xsl:text>    </td>
				<td><xsl:value-of select="normalize-space(/rnaml/molecule-class/consensus-molecule/molecule/sequence)"/></td>
			</tr>
			<tr>
				<td>Structure:	</td>
				<td><xsl:value-of select="substring-before(substring-after(/rnaml/molecule-class/consensus-molecule/molecule/structure/model/str-annotation/@comment, '='), ' ')"/>
				</td>
			</tr>
		<!-- boucle por afficher chaque sequences correctement-->
		<xsl:for-each select="ali-sequence">
			<tr>
				<td>	
					<xsl:choose>
						<xsl:when test="$url='none'">
							<xsl:value-of select="./molecule-id/@ref"/>
						</xsl:when>
						<xsl:otherwise>
						<xsl:element name='a'>
							<xsl:attribute name='href'>
								<xsl:value-of select="$url"/>
								<xsl:text>&amp;__wb_function=</xsl:text>
								<xsl:value-of select="$fnt_rna"/>
								<xsl:text>&amp;RnaDisplay=</xsl:text>
								<xsl:value-of select="./molecule-id/@ref"/>
							</xsl:attribute>
							<xsl:value-of select="./molecule-id/@ref"/>
						</xsl:element>	
						</xsl:otherwise>
					</xsl:choose>			
				</td>
				<td><xsl:value-of select="seq-data"/></td>
			</tr>
		</xsl:for-each>
		</table>
		</pre>
	</xsl:template>
	<!-- Derniere mise a jour de la famille-->
	<xsl:template match="revision">
		<td><label class="text2">Last updated:</label></td>
		<td><xsl:apply-templates select="date" /></td>
	</xsl:template>

	<!-- Affiche une analyse: program, version, et date d'execussion-->
	<xsl:template match="analysis">
		<tr>
			<td><label class="text1">Program:</label><b><xsl:value-of select="./program/prog-name"/> v<xsl:value-of select="./program/prog-version"/></b></td>
			<td><label class="text1">Running date:</label><xsl:apply-templates select="date" /></td>
		</tr>
	</xsl:template>

	<xsl:template match="date">
		<xsl:value-of select="./day"/>/<xsl:value-of select="./month"/>/<xsl:value-of select="./year"/>
	</xsl:template>
	<xsl:template name="cut">
		<!-- template de saucisonnage. Navigue sur le modele et non le fichier source, la chaîne à découper étant passée en paramètre -->
		<xsl:param name="string"/>
		<!-- determine la rondelle a decouper -->
		<xsl:choose>
			<xsl:when test="string-length($string) > 50">
				<xsl:value-of select="substring($string, 1,50)"/><br/>
				<xsl:variable name="rest">
					<xsl:value-of select="substring($string, 51, string-length($string))"/>
				</xsl:variable>
				<!-- relancer le processus de saucisonnage sur les éléments suivants du modèle -->
				<xsl:call-template name="cut">
			 		<xsl:with-param name="string" select="$rest"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				 <xsl:value-of select="$string"/><br/>
			</xsl:otherwise>
		</xsl:choose>
		<!-- détermine le reste du saucisson -->
		
	</xsl:template>

</xsl:stylesheet>
