#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: March 20, 2006
#

package Annotate;

=pod

=head1 NAME

Annotate- It's an object which contain annotation data on an rna and the annotator's login.

=head1 SYNOPSIS

	my $annot = New Annotate("rna_id", start, end, "legend");	
	
example: 
	
=head1 DESCRIPTION


=cut

use strict;
use warnings;

BEGIN
{
    our $VERSION = "0.2";
}

=head2 function New

 Title        : New
 Usage        : New Annotate;
 Prerequisite : none
 Function     : Create an object Annotate
 Returns      : An object Annotate
 Args         : none
 Globals      : none

=cut 

sub New
{
	my ($class) = @_;
	
	my $self = {
		__r_rna		=> '@',
		__start		=> 0,
                __end		=> 0,
		__title		=> "",
		__legend	=> "",
		__login		=> "",
		__time_stamp	=> ""
                };

    	bless($self, $class);
	return ($self);

}


=head2 procedure Initialize

 Title        : Initialize
 Usage        : $annot->Initialize( $r_rna, $start, $end, $legend,$login);
 Prerequisite : end must be <= length of rna
 Function     : Initialize the Annotate object
 Returns      : none
 Args         : $r_rna : ref on the object rna 
 		$start
		$end (< length of rna)
		$legend
		$login 
 Globals      : none

=cut 

sub Initialize
{
	my ($self, $start, $end,$title, $legend, $login, $time_stamp) = @_;
	$self->SetStart($start);
	$self->SetEnd($end);
	$self->SetTitle($title);
	$self->SetLegend($legend);
	$self->SetLogin($login);
	$self->SetTimeStamp($time_stamp);
}
=head2 fuction Print

 Title        : Print
 Usage        : $seq->Print;
 Prerequisite : none
 Function     : Display the range
 Returns      : none
 Args         : none
 Globals      : none

=cut 

sub Print
{
	my ($self) = @_;
	print "\tstart : ".$self->GetStart()."\n";
	print "\tEnd : ".$self->GetEnd()."\n";	
	print "\tLegend : ".$self->GetLegend()."\n";
	print "\tLogin : ".$self->GetLogin()."\n";
	print "\tTime : ".$self->GetTimeStamp()."\n";
}


#########GETTEUR ET ACCESSEUR ########"


=item GetRna

	Get Reference of rna
	
=cut

sub GetRna
{
	my ($self) = @_;
	return ($self->{'__r_rna'});
}

=item SetRna

	Set reference on rna
	
=cut

sub SetRna
{
	my ($self, $rna) = @_;
	$self->{'__r_rna'}=$rna;
}


=item GetStart

	Get start value
	
=cut

sub GetStart
{
	my ($self) = @_;

	return $self->{'__start'};
}

=item SetStart

	Set start value
	
=cut

sub SetStart
{
	my ($self, $start) = @_;

	$self->{'__start'}=$start;
}

=item GetEnd

	Get end value
	
=cut

sub GetEnd
{
	my ($self) = @_;

	return $self->{'__end'};
}

=item SetEnd

	Set end value
	
=cut

sub SetEnd
{
	my ($self, $end) = @_;

	$self->{'__end'}=$end;
}


=item GetLegend

	Get legend value
	
=cut

sub GetLegend
{
	my ($self) = @_;

	return $self->{'__legend'};
}

=item SetLegend

	Set legend value
	
=cut

sub SetLegend
{
	my ($self, $legend) = @_;

	$self->{'__legend'}=$legend;
}

=item GetLogin

	Get login
	
=cut

sub GetLogin
{
	my ($self) = @_;

	return $self->{'__login'};
}

=item SetLogin

	Set login
	
=cut

sub SetLogin
{
	my ($self, $login) = @_;

	$self->{'__login'}=$login;
}


=item GetTitle

	Get title
	
=cut

sub GetTitle
{
	my ($self) = @_;

	return $self->{'__title'};
}

=item SetTitle

	Set title
	
=cut

sub SetTitle
{
	my ($self, $title) = @_;

	$self->{'__title'}=$title;
}

=item GetTimeStamp

	Get time stamp (used as an identifiant)
	
=cut

sub GetTimeStamp
{
	my ($self) = @_;

	return $self->{'__time_stamp'};
}

=item SetTimeStamp

	set time stamp 
	
=cut

sub SetTimeStamp
{
	my ($self, $time_stamp) = @_;

	$self->{'__time_stamp'}=$time_stamp;
}


1;
