#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
# 
# 	Sebastien.Carerre@toulouse.inra.fr
# 	Celine.noirot@toulouse.inra.fr
#	Created: March 08, 2006
#

package Auth;

=pod

=head1 NAME

Auth - An object to manage authentification using cookies
	
=head1 SYNOPSIS
	
use CGI;
use ParamParser;

my $RH_PARAM = New ParamParser('CGIPM');


 # all parameters are optionnal, not privilege to use function IsPrivilege
 # if you want to retrieve the default cookie specify none
 # if you want to retrieve a specifique cookie specify cookie_name
 # if you want to authentified a user specified login, password, and etcpassword file
 # add cookie name to create a specifiq cookie name.
 my $file = "/tmp/etcpasswd";
 my @a_privilege=("guest","poweruser","admin");
 $O_AUTH=New Auth({login		=>	$RH_PARAM->Get('__login'), 
		password	=>	$RH_PARAM->Get('__passwd'),
		file		=>	$file,
		cookie		=>	$RH_PARAM->Get('__wb_cookie'),
		cookie_name	=>	"appli",
		cookie_expire	=>	"+1h",
		list_privilege=> \@a_privilege });

 print header(-cookie=>$O_AUTH->GetCookie());

 if ($O_AUTH->IsPrivilege('guest') )
 {
	print "You are guest<br>";
 }
 elsif ($O_AUTH->IsPrivilege('admin') )
 {
	print "You are admin<br>";
 }

=head1 DESCRIPTION

The privilege array parameter must correspond to the level in etcpassword file corresponding to a privilege.
 eg etcpassword file : 
 cnoirot:b4ea23a368b20bc1623e058f392f1fe4:1575:1:celine.noirot@inra.toulouse.fr:/www/LeARN_dev/web/tmp:/bin/tcsh
 cnoirot = login
 b4ea23a368b20bc1623e058f392f1fe4 = password_in_MD5
 1575 = id(not use)
 1 = privilege_number (corresponds to the index in an ordered array of privilege labels)
 celine.noirot@inra.toulouse.fr = email
 /www/LeARN_dev/web/tmp = path_workspace
 /bin/tcsh = shell (not use)

=cut

use Data::Dumper;
use Digest::MD5 qw(md5 md5_hex md5_base64);
use strict;
use CGI;

my $COOKIE_NAME="auth.default";

=head2 function New

 Title		  : New
 Usage		  : $o_auth= New Auth(%hash);
 Prerequisite : none
 Function	  : Create a new objet Auth
 Returns	  : An object Auth
 Args		  : %hash :which contain at least cookie_name
 				and for an authentification a file, login and password are requested.

=cut

sub New
{
    my ($class, $h_build_param) = (@_);
	
    my $self = {
				__file=>"",
				__login=>"",
				__password=>"",
				__workspace=>"",
				__email=>"",
				__privilege=>0,
				__cookie=>"",
				__cookie_name=>"",
				__cookie_expires=>'+24h',
				__a_privilege_label=>[]
				};
	bless($self, $class);

	# si le nom est specifier on prend en comte ce nom.
	$self->{'__cookie_name'} = $$h_build_param{'cookie_name'} if (defined($$h_build_param{'cookie_name'}) && $$h_build_param{'cookie_name'} ne "") ;
	$self->{'__cookie_expires'} = $$h_build_param{'cookie_expires'} if (defined($$h_build_param{'cookie_expires'}) && $$h_build_param{'cookie_expires'} ne "") ;
	
	# liste des privilege 
	@{$self->{'__a_privilege_label'}}=@{$$h_build_param{'list_privilege'}} if (defined($$h_build_param{'list_privilege'}));
	
	$self->{'__file'}=$$h_build_param{'file'} ;
	if ( defined ($$h_build_param{'login'} ) && defined ($$h_build_param{'password'}) 
	&& $$h_build_param{'login'} ne ""  && $$h_build_param{'password'} ne "" )
	{	
		
		$self->{'__login'}=$$h_build_param{'login'} ;
		$self->{'__password'}=$$h_build_param{'password'};
		$self->CheckAuth(); # si l'utilisateur est connu + bon pwd => Cree un cookie
	}# si un cookie est pass en parametre on va chercher les info associes 
	elsif (defined ($$h_build_param{'cookie_name'}))
	{
		$self->InitializeFromCookie();
	}
	elsif (defined($$h_build_param{'cookie'}) && $$h_build_param{'cookie'} ne "" )
	{	
		
		my @a_cookie=split(/=/,$$h_build_param{'cookie'});
		# un cookie existe deja ?
		if ( $self->ExistCookie($a_cookie[0]) )
		{ 
			$self->InitializeFromCookie($a_cookie[0]);
		}
	}
	
	return ($self);
}


=head2 function Print

 Title		  : Print use it for debug
 Usage		  : $o_auth->Print();
 Prerequisite : none
 
=cut

sub Print
{
	my ($self)=(@_);
	print "file etc passwd : ". $self->{'__file'}."<br>";
	print "login : ". $self->{'__login'}."<br>";
	print "passwd : ". $self->{'__password'}."<br>";
	print "workspace : ". $self->{'__workspace'}."<br>";
	print "email : ". $self->{'__email'}."<br>";
	print "privilege : ". $self->{'__privilege'}."<br>";
	print "cookie value : ". $self->{'__cookie'}."<br>";
	print "cookie name : ". $self->{'__cookie_name'}."<br>";
}



=head2 function ExistCookie

 Title		  : ExistCookie
 Usage		  : $obj_auth->ExistCookie($)
 Prerequisite : none
 Function	  : Check if the cookie name "__cookie_name" exists
 Returns	  : 1 if cookie exists
 				0 if not exists
 Args		  : $name : name of the cookie

=cut

sub ExistCookie
{
	my ($self)=(@_);
	my $o_cgi = new CGI();
	my $cookie_value = &UrlDecode($o_cgi->cookie($self->{'__cookie_name'}));
	if (defined ($cookie_value) && ($cookie_value ne "" ))
	{
		return (1);
	}
	else
	{
		return (0);
	}
}



=head2 function InitializeFromCookie

 Title		  : InitializeFromCookie
 Usage		  : $obj_auth->InitializeFromCookie($name)
 Prerequisite : cookie nemes $name must exist
 Function	  : For an existing cookie , initialize object auth
 Returns	  : none
 Args		  : $name : name of the existing cookie

=cut

sub InitializeFromCookie
{
	my $self=shift;
	
	my $o_cgi = new CGI();
	my $cookie_value = &UrlDecode($o_cgi->cookie($self->{'__cookie_name'}));
	#cnoirot:1575:1:celine.noirot@inra.toulouse.fr:/www/LeARN_dev/web/tmp:/bin/tcsh
	my $c_name=$self->{'__cookie_name'};
	#print STDERR "COokie value : $cookie_value\n";
	if ($cookie_value =~ /$c_name/ )
	{
		$cookie_value =~ s/$c_name=//;
	}
	my @a_cook_user=split (/:/,$cookie_value );

	open (USER, $self->{'__file'});
	my $find=0;
	my @a_user;
	while (my $line =<USER>)
	{	
		chomp($line);
		#cnoirot:b4ea23a368b20bc1623e058f392f1fe4:1575:1:celine.noirot@inra.toulouse.fr:/www/LeARN_dev/web/tmp:/bin/tcsh
		next if ($line eq "");
		@a_user=split (/:/,$line );
		#print STDERR "cook user = ".$a_cook_user[0] ." User: ".$a_user[0]." <\n>";
		if ($a_cook_user[0] eq $a_user[0])
		{
			$find=1;
			last;
		}
	}
	close(USER);
	if ($find==1)
	{
		$self->{'__login'}=$a_user[0];
		$self->{'__privilege'}=$a_user[3];
		$self->{'__workspace'}=$a_user[5];
		$self->{'__email'}=$a_user[4];
		my $value=$a_user[0].":".$a_user[2].":".$a_user[3].":".$a_user[4].":".$a_user[5].":".$a_user[6];# on vire le password !!!splice(@{$find},1,1)
		
		$self->{'__cookie'}=$o_cgi->cookie(-name => $self->{'__cookie_name'}, -value => $value, -expires => $self->{'__cookie_expires'});
		
	}
	else
	{
		$self->{'__privilege'}=$a_user[0];
	}
}




=head2 function CheckAuth

 Title		  : CheckAuth
 Usage		  : $obj_auth->CheckAuth
 Prerequisite : Attributes file / login / password must be set
 Function	  : Parse file of identification and check if login and password a correct.
 				If their are correct put initialize object Auth with identification data and create a cookie
 Returns	  : 1 : if authetification is ok
 				0 : if password or login are false

=cut

sub CheckAuth
{
	my $self=shift;
	my $find=undef;
	my $value="";
	my $o_cgi = new CGI();	
	if ($self->{'__cookie_name'} eq "" || not defined($self->{'__cookie_name'}))
	{
		$self->{'__cookie_name'}=$COOKIE_NAME;
	}
	if (defined ($self->{'__file'}) && (-e $self->{'__file'}) && (-s $self->{'__file'} ))
	{
		open (USER, $self->{'__file'});
		while (my $line =<USER>)
		{	
			chomp($line);
			#cnoirot:b4ea23a368b20bc1623e058f392f1fe4:1575:1:celine.noirot@inra.toulouse.fr:/www/LeARN_dev/web/tmp:/bin/tcsh
			next if ($line eq "");
			my @a_user=split (/:/,$line );
			#print STDERR "me5: " .md5_hex($self->{'__password'}). " : ".$a_user[1]."<br>";
			if ($a_user[0] eq $self->{'__login'}  && $a_user[1] eq md5_hex($self->{'__password'})) #md5_hex
			{
				$find=\@a_user;
				$self->{'__privilege'}=$a_user[3];
				$self->{'__workspace'}=$a_user[5];
				$self->{'__email'}=$a_user[4];
				$value=$a_user[0].":".$a_user[2].":".$a_user[3].":".$a_user[4].":".$a_user[5].":".$a_user[6];# on vire le password !!!splice(@{$find},1,1)
				$self->{'__cookie'}=$o_cgi->cookie(-name => $self->{'__cookie_name'}, -value => $value, -expires => $self->{'__cookie_expires'});
				last;
			}
		}
	}
	if (defined($find))
	{
		return (1);
	}
	else
	{
		return (0);
	}
}

=head2 function IsPrivilege

 Title		  : IsPrivilege
 Usage		  : $obj_auth->IsPrivilege(privilege_name)
 Prerequisite : @A_PRIV is defined
 Function	  : Check if self have the same privilege as the param
 Returns	  : 1 : if privilege are the same
 				0 : if privilege are different

=cut


sub IsPrivilege
{
	my ($self, $name_priv)=@_;
	my $res=0;
	#print STDERR "IsPrivilege : $name_priv | $self->{'__privilege'} | ".$self->{'__a_privilege_label'}[$self->{'__privilege'}]."\n";
	if ($name_priv eq $self->{'__a_privilege_label'}[$self->{'__privilege'}] ) 
	{
		$res=1;
	}
	return $res;
}

=item GetFile

	$o_auth->GetFile();
	Return file attribut

=cut

sub GetFile
{
	my $self=shift;
	return $self->{'__file'};
}

=item SetFile

	$o_auth->SetFile($file);
	Set file attribut

=cut

sub SetFile
{
	my ($self, $value)=@_;
	$self->{'__file'}=$value;
}

=item GetLogin

	$o_auth->GetLogin();
	Return login attribut

=cut

sub GetLogin
{
	my $self=shift;
	return $self->{'__login'};
}


=item SetLogin

	$o_auth->SetLogin($login);
	Set login attribut

=cut

sub SetLogin
{
	my ($self, $value)=@_;
	$self->{'__login'}=$value;
}


=item GetPassword

	$o_auth->GetPassword();
	Return password attribut

=cut

sub GetPassword
{
	my $self=shift;
	return $self->{'__password'};
}

=item SetPassword

	$o_auth->SetPassword($pwd);
	Set password attribut

=cut

sub SetPassword
{
	my ($self, $value)=@_;
	$self->{'__password'}=$value;
}


=item GetWorkspace

	$o_auth->GetWorkspace();
	Return workspace attribut

=cut

sub GetWorkspace
{
	my $self=shift;
	return $self->{'__workspace'};
}

=item SetWorkspace

	$o_auth->SetWorkspace($wks);
	Set workspace attribut

=cut

sub SetWorkspace
{
	my ($self, $value)=@_;
	$self->{'__workspace'}=$value;
}


=item GetEmail

	$o_auth->GetEmail();
	Return email attribut

=cut

sub GetEmail
{
	my $self=shift;
	return $self->{'__email'};
}

=item SetEmail

	$o_auth->SetEmail($email);
	Set email attribut

=cut

sub SetEmail
{
	my ($self, $value)=@_;
	$self->{'__email'}=$value;
}

=item GetPrivilege

	$o_auth->GetPrivilege();
	Return privilege attribut

=cut

sub GetPrivilege
{
	my $self=shift;
	return $self->{'__privilege'};
}

=item SetPrivilege

	$o_auth->SetPrivilege($priv);
	Set privilege attribut

=cut

sub SetPrivilege
{
	my ($self, $value)=@_;
	$self->{'__privilege'}=$value;
}

=item GetCookie

	$o_auth->GetCookie();
	Return cookie attribut

=cut

sub GetCookie
{
	my $self=shift;
	return $self->{'__cookie'};
}

=item SetCookie

	$o_auth->SetCookie($cookie);
	Set cookie attribut

=cut

sub SetCookie
{
	my ($self, $value)=@_;
	$self->{'__cookie'}=$value;
}


=item GetCookieName

	$o_auth->GetCookieName();
	Return cookie name attribut

=cut

sub GetCookieName
{
	my $self=shift;
	return $self->{'__cookie_name'};
}

=item SetCookieName

	$o_auth->SetCookieName($cookie);
	Set cookie name attribut

=cut

sub SetCookieName
{
	my ($self, $value)=@_;
	$self->{'__cookie_name'}=$value;
}


=item UrlDecode

	$o_auth->UrlDecode();
	Decode url value

=cut

sub UrlDecode
{
    my $string = shift;

    $string =~ tr/+/ /;
    $string =~ s/%([\da-fA-F]{2})/chr (hex ($1))/eg;

    return $string;
}

				
1;
