#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
	#
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: October 24, 2005
#

# TODO: buildfamily : ameliore + dbxref a voir en fonction de l'arn et du prg de dectect de larn
package Learn;

=pod

=head1 NAME

Rna- It's an object Learn

=head1 SYNOPSIS

	my $prog = New Learn();	
	
example: 
	my $o_param = New ParamParser('GETOPTLONG',"cfg=s", "createtree", "input_fasta=s");
	$o_param->SetUsage(my $usage=sub { &Usage(); } );
	$o_param->AssertFileExists('cfg');
	$o_param->Update($o_param->Get('cfg'),'A');
	$o_param->AssertFileExists('input_fasta');
	my $fasta=$o_param->Get('input_fasta');

	my $application= New Learn($o_param);
	

	$application->Initialize($o_param->Get('createtree')); # cree l'arborescence
	
	#$application->LoadRelease($o_param->Get('old_release'));


	
	$application->CreatePrograms(); # blastn / rfam_scan / trnaScan-SE / rnafold / mfold
	$application->PrintPrograms();	

	$application->LoadSequences($fasta);
	$application->PrintSequences();
	
	$application->AnalyseSequences("LoadRna"); # => appele loadRnaL
	
	#$application->DeleteAnalyseRedondancy();
	$application->ComputeAllRnaStructure();

	#$application->ComputeAllFamilyStructure();
	$application->PrintRnas();	
	$application->SaveRnasToRnaml();
	$application->BuildFamily();
	$application->SaveFamiliesToRnaml();
	
=head1 DESCRIPTION


=cut

use strict;
use warnings;

use Bio::Seq;
use Bio::SeqIO;
use Rna;
use RnaFamily;
use Program;
use Sequence;
use Overlap;
use Bio::Tools::GFF;
use Bio::SeqFeature::Generic;
use GFF;

use Var ;
#use vars qw( $E_XML );

eval "use Time::HiRes";
#$Var::E_XML=".xml";
BEGIN
{
    our $VERSION = "0.2";
}

our $R_LOGGER;
our $LOGGER_STAMP;
our %H_PROGRAM_ACTUAL;# programme utilise pour l'execution
our %H_PROGRAM_PSEUDO;# programme decrit ms ne pouvant etre execute (gerer les version de prog n'etant plus utilises)
our %H_PROG_TYPE; # nome de programme (sans version) => type de programme

my @ANALYSIS_PRG=(); # par defaut
my $STATUS_DEFAULT="automatic";



=head2 function New

 Title        : New
 Usage        : New Learn;
 Prerequisite : none
 Function     : Create an object Learn
 Returns      : An object Learn
 Args         : none
 Globals      : none

=cut 

sub New
{
	my ($class, $o_param, $logger) = @_;# lors de l'appel pas besoin de dereferencer !
	my $machine=`uname -n`;
	chop($machine);
    	my $self = {
		__o_param		=> $o_param,
		__h_rna 		=> {},
		# pour gerer les instances multiple fod est nomme nomMachine.PID.filename
		__fod			=>$o_param->Get('path_new_release')."/".$o_param->Get('path_source_sequence')."/".$machine.$$.$o_param->Get('file_name_rna_index'),
		__h_sequence		=> {},
		__h_family 		=> {},
		__a_loaded_rna		=> {}, # liste des arn deja loaded (old released) qui ne doivent pas etre pris en compte dans le BuildFamily => sinon duplication des donnees dans rfl db !
		__current_release	=> $o_param->Get('path_new_release'),
		__old_release		=> $o_param->Get('path_old_release'),
		__today			=> "",
		__last_family_id	=> 1,
		__intern_fasta_db	=> $o_param->Get('ISCAN_rfam_db'),
		__intern_fasta_db_for_blast	=>$o_param->Get('ISCAN_rfl_db'),
		__intern_thr		=> $o_param->Get('ISCAN_thr'), 
		__overlap		=> New Overlap ($o_param->Get('overlap_file') ),
		__bac_upgraded		=> []
             };
	bless($self, $class);
	$R_LOGGER=$logger;
	$LOGGER_STAMP=$$R_LOGGER->CalcStamp();
	
    	return ($self);
}



=head2 function SetLogger

 Title        : SetLogger
 Usage        : Learn::SetLogger($log) ;
 Prerequisite : none
 Function     : Initalize global variable : the Logger of Learn
 Returns      : none
 Args         : none
 Globals      : none

=cut 
sub SetLogger
{
	my ($class)=shift;
	$R_LOGGER=shift;
	$LOGGER_STAMP=$$R_LOGGER->CalcStamp();
}


=head2 procedure Initialize

 Title        : Initialize
 Usage        : $MyLearn->Initialize();
 Prerequisite : none
 Function     : Initialize the MyLearn object
 Returns      : none
 Args         :
 Globals      : none

=cut

sub Initialize
{
	my ($self, $createtree, $mode , $overlap, $bac_upgade_file) = @_;

	print "==== LeARN pipeline ====\n";
	
	# Get the a ll the values for current time
   	my ($sec, $min, $hour, $mday, $mon, $year) = gmtime();
   	$year = (1900 + $year);    # We'll have to modify this before 2100
    	$mon++;
   	my $time = sprintf("%04i%02i%02i", $year, $mon, $mday);
	$self->{'__today'}=$time;
	
	my $step= $$self{'__o_param'}->Get('step_rna_id');

	$Sequence::STEP_RNA_ID=$step if (defined ($step));
	$self->CreateTreeDir($mode) if ($createtree);

	my $prgs = $$self{'__o_param'}->Get('PROG_TO_SCAN');
	@ANALYSIS_PRG=[] if (defined ($prgs));
	my @lst_prg= split(' ', $prgs );
	foreach my $pgr ( @lst_prg )
	{
		push (@ANALYSIS_PRG, $pgr);
	}

	if (defined($overlap) && $overlap ne "")
	{
		$self->{'__overlap'}->Init();
	} 
	if (defined($bac_upgade_file) && $bac_upgade_file ne "")
	{
		if (-e $bac_upgade_file )
		{
			open (BACS, $bac_upgade_file);
			while (my $line = <BACS>)
			{
				chomp ($line);
				my @versions=split (' ',$line);
				push (@{$self->{'__bac_upgraded'}},[($versions[0],$versions[1] )] );
			}
		}
	}
}

=head2 procedure CopyRflDb

 Title        : CopyRflDb
 Usage        : $MyLearn->CopyRflDb();
 Prerequisite : rfl db exist in old repository
 Function     : Copy rfl db from old release to new release
 		Sequences analysed in previous release

=cut

sub CopyRflDb
{
	my ($self)=@_;
	print "CreateTreeDir ... Copy previous rfl db \n";
	my $rfl_db=$$self{'__o_param'}->Get('path_old_release')."/".$$self{'__o_param'}->Get('path_db_rfl');
	
	$$R_LOGGER->Trace("Learn\tCreateTreeDir\tCopie de la banque Rfl \n",$LOGGER_STAMP,"D");
	my $cmd= "cp -a $rfl_db ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db');
	&SystemLaunchLog($cmd);
	
	$cmd= "rm -rf ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_source_sequence')." ; cp -a -r ".$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_source_sequence')." ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_source');
	&SystemLaunchLog($cmd);
}

=head2 procedure CreateTreeDir

 Title        : CreateTreeDir
 Usage        : $MyLearn->CreateTreeDir();

 Function     : Create tree directory of release
 Args         : $mode : [ ln|cp|none] mode of copy of db blast and rfam
 Globals      : none

=cut

sub CreateTreeDir
{
	my($self, $mode)=@_;
	print "Create the directory tree of the release\n";

	$$R_LOGGER->Trace("Learn\tCreateTreeDir\t".$self->{'__current_release'}."\n",$LOGGER_STAMP,"DE");
	mkdir ($self->{'__current_release'});
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db'));
	#system "rm -rf ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn');
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn'));
	#system "rm -rf ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl');
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_xml'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_ss'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_tmp'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_img'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_bug'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_ss'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_tmp'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_img'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_bug'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_detection'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_detection_gff'));	
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_source'));
	mkdir ($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_source_sequence'));

	print "Copy previous Rfam and blast databases\n";

	# traite les sources des banques a copier par defaut utilise l'ancienne release sinon utilise des repertoires specifiques

	my $blast_db=$$self{'__o_param'}->Get('abs_dir_blast');
	if (! defined($blast_db) || ! -d $blast_db)
	{
		$blast_db=$$self{'__o_param'}->Get('path_old_release')."/".$$self{'__o_param'}->Get('path_db_blast');
	}

	my $rfam_db=$$self{'__o_param'}->Get('abs_dir_rfam');
	if (! defined($rfam_db) || ! -d $rfam_db)
	{
		$rfam_db=$$self{'__o_param'}->Get('path_old_release')."/".$$self{'__o_param'}->Get('path_db_rfam');
	}

	$$R_LOGGER->Trace("Learn\tOld blast dir : $blast_db : old rfam dir: $rfam_db\n",$LOGGER_STAMP,"DE");
	my $cmd;
	# Copie de la banque blast
	if ( defined($mode) && $mode eq "cp" ) 
	{	#BLAST
		print "\tCopy blast db...\n";
		$$R_LOGGER->Trace("Learn\tCreateTreeDir\tCopie banque blast \n",$LOGGER_STAMP,"D");

		$cmd= "cp -a $blast_db ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db');
		&SystemLaunchLog($cmd);

		#RFAM
		print "\tCopy Rfam db...\n";
		$$R_LOGGER->Trace("Learn\tCreateTreeDir\tCopie banque rfam \n", $LOGGER_STAMP,"DE");
		
		$cmd= "cp -a $rfam_db ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db');
		&SystemLaunchLog($cmd);

	}
	elsif ( defined($mode) && $mode eq "ln" ) 
	{
		$cmd = "rm -f ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam');
		&SystemLaunchLog($cmd);
		$cmd= "ln -sf $rfam_db ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam');
		&SystemLaunchLog($cmd);
		
		$cmd = "rm -f ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_blast');
		&SystemLaunchLog($cmd);
		$cmd= "ln -sf $blast_db ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_blast');
		&SystemLaunchLog($cmd);
	}
	elsif (!defined($mode) || $mode eq "none" ) 
	{
		print "\tBlast db not copied ... \n";
		$$R_LOGGER->Trace("Learn\tCreateTreeDir\tblast db not copied : $blast_db -> ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db')."\n",$LOGGER_STAMP,"DE");
		
		print "\tRfam  db not copied ... \n";
		$$R_LOGGER->Trace("Learn\tCreateTreeDir\trfam db not copied: $rfam_db -> ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db')."\n",$LOGGER_STAMP,"DE");
	}

}


=head2 procedure CreateProgram

 Title        : CreateProgram
 Usage        : $MyLearn->CreateProgram();
 Function     : Create objects programs in the hash of program
 
=cut 
sub CreatePrograms
{
	my ($self) = @_;
	print "Create analysis programs\n";
	$$R_LOGGER->Trace("Learn\tCreatePrograms\n",$LOGGER_STAMP,"DE");
# RAJOUTER VERSION DU PROGRAMME
	
	my $prgs = $$self{'__o_param'}->Get('PROG_TO_LOAD');
	my @lst_ns_prg= split(' ', $prgs );
	foreach my $ns_pgr ( @lst_ns_prg )
	{
		next if ($ns_pgr eq "");
		$$self{'__o_param'}->SelectNameSpace($ns_pgr);
		my $name = $$self{'__o_param'}->Get("_name");
                next if ($name eq "" );
		my $cmd = $$self{'__o_param'}->Get("_cmd");
		my $param = $$self{'__o_param'}->Get("_param");
		my $version = $$self{'__o_param'}->Get("_version");
		my $type = $$self{'__o_param'}->Get("_type");
		$H_PROGRAM_ACTUAL{$type}= New Program($name, $cmd, $param, $type, $version,$R_LOGGER, $LOGGER_STAMP);
		
		$$self{'__o_param'}->SelectNameSpace();
		my $lst_type=$$self{'__o_param'}->Get($name);
		chomp($lst_type);
		$H_PROG_TYPE{$name}=$lst_type;
		
	}
	#des traitement supplementaire sont necessaire pour certain prg.
	if (defined ($H_PROGRAM_ACTUAL{'SCAN_2'}))
	{
		my $rfam_db= $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam');
		$H_PROGRAM_ACTUAL{'SCAN_2'}->SetParam($H_PROGRAM_ACTUAL{'SCAN_2'}->GetParam()." -d $rfam_db");
	}
	#des traitement supplementaire sont necessaire pour certain prg.
	if (defined ($H_PROGRAM_ACTUAL{'SCAN_3'}))
	{
		my $blast_db= $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_blast')."/".$$self{'__o_param'}->Get('file_name_fasta_db');
		$H_PROGRAM_ACTUAL{'SCAN_3'}->SetParam($H_PROGRAM_ACTUAL{'SCAN_3'}->GetParam()." -d $blast_db");
	}
	# internal rfam_scan used for the detection on our family
	# on modifie les chemin d'acces aux donnees par une substitution
	if (defined ($H_PROGRAM_ACTUAL{'ISCAN'}))
	{

		my $rfl_intern_rfam = $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern');
		my $pattern = $$self{'__o_param'}->Get('ISCAN_pattern_export');
		my $cmd_rfam=$H_PROGRAM_ACTUAL{'ISCAN'}->GetPrg();
		$cmd_rfam=~s/$pattern/$rfl_intern_rfam/;
		$H_PROGRAM_ACTUAL{'ISCAN'}->SetPrg($cmd_rfam);
		$H_PROGRAM_ACTUAL{'ISCAN'}->SetParam($H_PROGRAM_ACTUAL{'ISCAN'}->GetParam() ." -d $rfl_intern_rfam");
	}
	
	if (defined ($H_PROGRAM_ACTUAL{'ANCHOR_SCAN'}))
	{
	
		my $mir_db = $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam')."/".$$self{'__o_param'}->Get('path_db_rfam_mir')."/".$$self{'__o_param'}->Get('file_name_mir_db');
		$H_PROGRAM_ACTUAL{'ANCHOR_SCAN'}->SetParam($H_PROGRAM_ACTUAL{'ANCHOR_SCAN'}->GetParam() ." --mir_db $mir_db");
	}
}

=head2 procedure SetScanProgramRepository

 Title        : SetScanProgramRepository
 Usage        : $MyLearn->SetScanProgramRepository($repository_dir, $overwrite_repository);
 Prerequisite : Scan program must have been created
 Function     : set the use of the repository for scan programs
 Args         : $repository_dir       : directory which contain results of analyses
              : $overwrite_repository : to erase old result.
 Globals      : none

=cut 

sub SetScanProgramRepository
{
	my ($self,$repository_dir, $overwrite_repository ) = @_;
        my @prgs =split(' ', $$self{'__o_param'}->Get('PROG_TO_SCAN'));
        foreach my $key ( @prgs )
        {
            #print $key."\n";
            if (defined ($self->GetProgram("$key")))
            {
                $H_PROGRAM_ACTUAL{$key}->SetRepository($repository_dir,$overwrite_repository);
            }
        }
}

=head2 procedure LoadSequences

 Title        : LoadSequences
 Usage        : $MyLearn->LoadSequences($file);
 Prerequisite : none
 Function     : from multifasta file, create simple fasta file and instanciate Sequence object in hr_sequence,
 Args         : $file : multifasta file to analyse.
 Globals      : none

=cut 
sub LoadSequences
{
	my ($self, $file_multfasta) = @_;
	print "Split multifasta file \n";
	$$R_LOGGER->Trace("Learn\tLoadSequences\tDecoupage du fichier multifasta\n",$LOGGER_STAMP,"DE");


	my ($seq_obj);

	# $dir_out : release/source/sequence
	my $dir_out = $$self{'__o_param'}->Get('path_new_release')."/".$$self{'__o_param'}->Get('path_source_sequence');
	
	# fichier d'index : ecrasement de l'ancien pour l'increment a voir + tard.
	open (IDX, ">".$self->{'__fod'}) or die("Erreur decriture >".$self->{'__fod'}."\n");

	# read fasta file
	my $seq = Bio::Seq->new();

	# pas besoin de tester l'existance, deja fait dans prg principal
	my $in = Bio::SeqIO->new( -file => $file_multfasta, '-format' => 'Fasta' );

	while( $seq = $in->next_seq())
	{	
                my $seqid=$seq->id();
                #>gi|109715764|dbj|AP009200.1| Lotus japonicus genomic DNA, chromosome 2, clone:LjT17O14, TM0610a, complete sequence
                if ($seqid =~ /\S\S\|\d+\|\S+\|\S\S.+\|/)
                {
                    ($seqid)= $seqid =~ /\S\S\|\d+\|\S+\|(.+)\|/;
                    $seqid=~s/;//;
                }
                # Non gere : >MtBR:mtgsp_003e12_19-Nov-2003_C9 | 3541 | MtBR:mtgsp_003e12_19-Nov-2003_C9;|
                mkdir ("$dir_out/".$seqid);
		my $out = Bio::SeqIO->new( -file => ">$dir_out/".$seqid."/".$seqid, '-format' => 'Fasta' );
		$out->write_seq($seq);
		$out->DESTROY();
		print IDX "$dir_out/".$seqid."\n";

		# Creation d'un objet sequence
		$self->AddSequence("$dir_out/".$seqid."/".$seqid, $seqid, $$self{'__o_param'}->Get('species'));
	}
	close (IDX);	

}



=head2 procedure AddSequence

 Title        : AddSequence
 Usage        : $MyLearn->AddSequence();
 Function     : Create an object sequence in the hash of sequence
 Returns      : none
 Args         : $seq_file: path of sequence data file
 		$id : identifier of the sequence
 		$species : organis of the sequence
 Globals      : none

=cut 

sub AddSequence
{
	my ($self, $seq_file, $id, $species) = @_;
	$$R_LOGGER->Trace("Learn\tAddSequence\t$id \n",$LOGGER_STAMP,"D");
	${$self->{'__h_sequence'}}{$id}=New Sequence();
	${$self->{'__h_sequence'}}{$id}->SetSeqFile($seq_file) if (defined ($seq_file));	
	${$self->{'__h_sequence'}}{$id}->SetId($id);
	my ($acc,$vers) = split('.',$id); # ex AC137828.24
	${$self->{'__h_sequence'}}{$id}->SetAcc($acc);
	${$self->{'__h_sequence'}}{$id}->SetVersion($vers) if (defined($vers));
	${$self->{'__h_sequence'}}{$id}->SetOrganism($species);
	my $old_bac=$self->GetOldAccBac($id);
	if ($old_bac ne "" && defined(${$self->{'__h_sequence'}}{$old_bac}))
	{
		${$self->{'__h_sequence'}}{$id}->SetOldBacAcc($old_bac) ;
		${$self->{'__h_sequence'}}{$id}->SetNextRnaId(${$self->{'__h_sequence'}}{$old_bac}->GetNextRnaId());
	}
}


=head2 procedure AnalyseSequences

 Title        : AnalyseSequences
 Usage        : $MyLearn->AnalyseSequences("LoadRna");
 Prerequisite : none
 Function     : Execute RunProgram on all sequences and if parameter what=LoadRna then load rna object
 Args         : $what ["LoadRna"|""] if what=LoadRna then load rna object 
 Globals      : none

=cut 
sub AnalyseSequences
{	
	
	my ($self, $what) = @_;
	print "Sequence analyses \n";
	$$R_LOGGER->Trace("Learn\tAnalyseSequences\tMode : $what\n",$LOGGER_STAMP,"DE");
	#parcourt du fichier d'index des rna
	open (FOD, $self->{'__fod'}) or die ("\tImpossible d'ouvrir le FOD: ".$self->{'__fod'}."\n");

	while (my $in_path=<FOD>) # line contien le chemin du repertoire d'une sequence
	{	
		chomp($in_path);
		$$R_LOGGER->Trace("\tLearn\tAnalyseSequences work on directory: $in_path \n",$LOGGER_STAMP,"DE");
		my ($path,$seq_name)= $in_path=~/(.+)\/(.+)/;  # recupere le nom du dernier repertoire car = nom du fichier fasta
		#calcul du MD5 de la sequence d'entre si on est en mode repository

		# create a directory with file name in gff directory
		my $out_path=$$self{'__o_param'}->Get('path_new_release')."/".$$self{'__o_param'}->Get('path_detection_gff')."/$seq_name/";
		mkdir ($out_path);
		my $prog;
		# pour chaque  pgr on lance l'execution sur le fichier
		foreach $prog (@ANALYSIS_PRG)# 
		{
			next if (not defined($H_PROGRAM_ACTUAL{$prog}));
			my $name=$H_PROGRAM_ACTUAL{$prog}->GetName();
			$H_PROGRAM_ACTUAL{$prog}->SetInOut("$in_path/$seq_name", "$out_path/$seq_name.$name");
			
			$H_PROGRAM_ACTUAL{$prog}->Run('gff');
		}
		#pour une sequence mere on va charger les ARN detecte si demande et sequence non analyse
		if (($what eq "LoadRna") && (${$self->{'__h_sequence'}}{$seq_name}->GetAnalyzed() == 0))
		{
			$self->LoadRna($seq_name);
			${$self->{'__h_sequence'}}{$seq_name}->SetAnalyzed("1");
			
		}
	}
	close FOD;
}	


=head2 procedure CreateFilesForAnalyses

 Title        : CreateFilesForAnalyses
 Usage        : $MyLearn->CreateFilesForAnalyses($cfg,$repository_dir,$overwrite_repository);
 Prerequisite : none
 Function     : Create files with command line to launch analysis on sequences
 Args         : 
 Globals      : none

=cut 
sub CreateFilesForAnalyses
{	
	
	my ($self,$cfg,$repository_dir,$overwrite_repository) = @_;
	print "Sequences analysis\n";
	$$R_LOGGER->Trace("Learn\tCreateFilesForAnalyses\t\n",$LOGGER_STAMP,"DE");
	#parcourt du fichier d'index des rna
	open (FOD, $self->{'__fod'}) or die ("\tImpossible d'ouvrir le FOD: ".$self->{'__fod'}."\n");
        my $file_prefix=$self->{'__fod'};
        
        # debut de commande id pour tous :
        my $prg_run_analyse=$$self{'__o_param'}->Get('run_analyse');
        my $command="$prg_run_analyse --cfg $cfg --repository_dir $repository_dir --path_new_release ".$$self{'__o_param'}->Get('path_new_release');
        $command.=" --overwrite_repository" if ($overwrite_repository);
        

	while (my $in_path=<FOD>) # line contien le chemin du repertoire d'une sequence
	{	
		chomp($in_path);
		$$R_LOGGER->Trace("\tLearn\tAnalyseSequences work on directory: $in_path \n",$LOGGER_STAMP,"DE");
		my ($path,$seq_name)= $in_path=~/(.+)\/(.+)/;  # recupere le nom du dernier repertoire car = nom du fichier fasta
		
		# create a directory with file name in gff directory
		my $out_path=$$self{'__o_param'}->Get('path_new_release')."/".$$self{'__o_param'}->Get('path_detection_gff')."/$seq_name/";
		mkdir ($out_path);
		my $prog;
		# pour chaque  pgr on lance l'execution sur le fichier
		foreach $prog (@ANALYSIS_PRG) 
		{
                    next if (not defined($H_PROGRAM_ACTUAL{$prog}));
                    my $name=$H_PROGRAM_ACTUAL{$prog}->GetName();
                    open (CMD, ">>$file_prefix.$prog.cmd");
                    print CMD "$command --input $in_path/$seq_name --output $out_path/$seq_name.$name --program_type $prog \n";
                    close (CMD);
		}
	}
	close FOD;
        print "Command files available in :$file_prefix.*.cmd\n";
}	
	
=head2 procedure LoadRna

 Title        : LoadRna
 Usage        : $MyLearn->LoadRna($seq_name);
 Function     : For all result (gff file) create Range / RNA / Family
 Args         : $seq_name : id of the sequence
 Globals      : none

=cut 

sub LoadRna
{	
	my ($self, $seq_name) = @_;
	#print "...Creation des objets rna\n";
	$$R_LOGGER->Trace("Learn\tLoadRna\tOn sequence : $seq_name\n",$LOGGER_STAMP,"DE");
	my ($tmp_rna,$rna_id,$path,$prog, $cmd);
	my $length_threshold_blast=$$self{'__o_param'}->Get('SCAN_3_threshold_length');

	#$seq_name contien le nom de la sequence mere, on supprime la version car va etre utilise pour 
	# l'id des ARN 
	my ($id_seq,$version)= $seq_name=~/([^\.]+)\.*(.*)/;
#	my $old_bac=$self->IsUpgraded($seq_name);

	#$seq_name est utilise pour le hash des sequences
	#$id_seq utilise pour les id des arn

	# recupere la reference de la sequence corespondante
	my $r_sequence=\${$self->{'__h_sequence'}}{$seq_name};
	$$r_sequence->SetSequence();# charge la chaine sequence en memoire

	#Remplie le tableau des correspondance de range avec cette sequence 
	my $nb_of_overlap = $self->{'__overlap'}->ComputeSeqOverlaped($$r_sequence->GetId()); 

	#parcourt de tout les fichiers gff de la sequence
	foreach $prog (@ANALYSIS_PRG)
	{	
		next if (not defined($H_PROGRAM_ACTUAL{$prog}));
		my $gff_file=$H_PROGRAM_ACTUAL{$prog}->GetFileOut();
		next if ( (!defined( $gff_file)) || (! -e $gff_file));
	
		$$R_LOGGER->Trace("Learn\tLoadRna\t$gff_file\t NB OVERLAP : $nb_of_overlap\n",$LOGGER_STAMP,"D");
	
		# DATE ! recupere la date d'execution via le fichier gff !
		# recup les stat du fichier d'origine (sans l'extention gff!)
		my $prg_file=$gff_file;
		$prg_file =~ s/.gff$//;
		my @etat=stat($prg_file); 
		# indice 9 => date
		my ($sec, $min, $hour, $day, $mon, $year)=gmtime($etat[9]);
		# format gmtime : $sec, $min, $hour, $mday, $mon, $year
		$mon++; #(increment du mois)
		$year+=1900; # et l'annee
		my $date_detect= sprintf("%04i%02i%02i",$year,$mon,$day);

		#Suppression de la redondance # A revoir : Ne pas modifier les donnees de depart !
		$cmd = "sort -u $gff_file > $gff_file.srt ; mv $gff_file.srt $gff_file ; rm -rf $gff_file.srt" ;
		&SystemLaunchLog($cmd);

		#programme de structure secondaire
		my ($prg_ss, $prg_convert);
		if (defined($H_PROGRAM_ACTUAL{'R_FOLD'} ))
		{
			$prg_ss=\$H_PROGRAM_ACTUAL{'R_FOLD'}; # programme de calcul de la structure par default
			$prg_convert=\$H_PROGRAM_ACTUAL{'R_CONV_CT2B'};
		}
		else
		{
			$prg_ss=\$H_PROGRAM_ACTUAL{'R_FOLD_ALT'};
			$prg_convert=\$H_PROGRAM_ACTUAL{'R_CONV_B2CT'};
		} 

		# ouverture du fichier gff
		my @a_t ;
		open (TGFF, $gff_file) || die "ERROR, can't open >$gff_file<\n";
		my $gff_data = new GFF::GeneFeatureSet();
		$gff_data->version(2);
		$gff_data->read(\*TGFF);
		close TGFF;

		# pour blast : on fusionne les overlaps
		if ($prog eq "SCAN_3") 
		{
			open (OUTGFF, ">$gff_file.2" );
			#my @a_t = $gff_data->eachGeneFeature();
			my $gff_unik = new GFF::GeneFeatureSet();
			$gff_unik=$gff_data->self_overlap();
			$gff_unik->dump(\*OUTGFF);
			close OUTGFF;
			@a_t =$gff_unik->eachGeneFeature();
		}
		else
		{
			@a_t =$gff_data->eachGeneFeature();
		}

		# loop over the input stream
		foreach my $ft (@a_t)
		{	
                        next if (not defined ($ft));
			next if ( (not defined ($ft->start())) || $ft->start() eq "<nul>");
            		#print  "$gff_file: ".$ft->start().",". $ft->end().",". $ft->strand()." \n";
			# passe au suivant si blast est trop petit
			next if (($prog eq "SCAN_3") && ( abs ($ft->start()-$ft->end()) <  $length_threshold_blast) );

		# recupere le dernier id d'arn de la sequence (on incrementera que si l'arn est nouveau)
			$rna_id=sprintf("%s%i", $id_seq."_r", $$r_sequence->GetNextRnaId());
			
			# path for image : build with rh_param data
			$path=$$self{'__o_param'}->Get('path_new_release')."/".$$self{'__o_param'}->Get('path_db_learn_rna');
			
			# creation d'un arn temporaire pour faire les test de chevauchement/d'existance
			$tmp_rna=New Rna();
			
			$tmp_rna->Initialize($rna_id, $rna_id, $ft->tagValue('Att'), $self->{'__today'}, "Automatic", $path,$prg_ss , $prg_convert);
                        #print "$tmp_rna->Initialize($rna_id, $rna_id, .".$ft->tagValue('Att').", $self->{'__today'}, Automatic, $path,$prg_ss , $prg_convert);".$ft->feature()."\n" ;
                        if ($ft->feature() eq "snoran") {exit;}
			$tmp_rna->ComputeNameDbxref($ft->tagValue('Att'), $ft->feature());
			
			# ref du programme qui a detecte cet arn :
			my $r_prog_detect = \$H_PROGRAM_ACTUAL{$prog} ;
			$tmp_rna->SetPrincipalRange( $ft->start(), $ft->end(), $ft->strand(), $r_sequence, $r_prog_detect, $date_detect);
			
			
			# On verifie que c'est ARn n'existe pas deja

			my ($find, $rna_exist_id) = $self->RnaExists(\$tmp_rna);
			
			#GESTION DU CHANGEMENT DE VERSION DE BAC
			if ( ${$tmp_rna->GetPrincipalRange()->GetSequence()}->GetOldBacAcc( ) ne "" )
			{
				# on est sur une nouvelle version de bac, si detecter par rfam_scan_intern on a le nom de la famille dans laquelle il appartien et surment l'arn equivalent detecte sur le bac precedant.
				if ( $prog eq 'ISCAN') 
				{
					
					#ACC=RFL00001;ID=mir-395;SCORE=140.75;
					my ($id_family)= $ft->tagValue('Att') =~ /ACC=(RFL\d+)/ ;
					my $old_bac=${$tmp_rna->GetPrincipalRange()->GetSequence()}->GetOldBacAcc( );
					
				#on va chercher l'arn de l'ancien bac dans cette famille
				foreach my $r_rna (@{${$self->{'__h_family'}}{$id_family}->GetRaRna()})
				{
					if (${$$r_rna->GetPrincipalRange()->GetSequence()}->GetId() eq $old_bac)
					{
						if ($$r_rna->GetSequence() eq $tmp_rna->GetSequence() )
						{
						
						my $range=$tmp_rna->GetPrincipalRange();
						$$r_rna->SetPrincipalRange($range->GetStart(),$range->GetEnd(),$range->GetStrand(),$range->GetSequence(), $range->GetProgDetect(),$range->GetDateDetect(),$range->GetSeqData(),$range->GetLength());
						delete (${$self->{'__h_rna'}}{ $rna_id });
						$find=1;
						last;
	
						}
					}
				}
				}
			}
			#GESTION DE LOVERLAP
			if ( $nb_of_overlap > -1 && $find == 0 )
			{
				$$R_LOGGER->Trace("Learn\tOVERLAP existant \t $nb_of_overlap\n",$LOGGER_STAMP,"D"); 
				
				if  ($self->{'__overlap'}->ComputeRangeOnOverlaped($seq_name,\$tmp_rna->GetPrincipalRange()) )
				# si l'arn est sur une zone overlape
				{# remplie le hash query de overlap, avec les range qui match avec le range de l'ARN actuel
				
				$$R_LOGGER->Trace("Learn\tOVERLAP\t 1\n",$LOGGER_STAMP,"D"); 
					foreach my $seq_id_overlap (%{$self->{'__overlap'}->{'__h_result'}})
					{# skip si sequence n'a pas etait analyse ou si non existante
						$$R_LOGGER->Trace("Learn\tOVERLAP seq id_ ovelaped\t $seq_id_overlap\n",$LOGGER_STAMP,"D"); 
						next if( not defined(${$self->{'__h_sequence'}}{$seq_id_overlap}) );
						
						next if (not ${$self->{'__h_sequence'}}{$seq_id_overlap}->GetAnalyzed() );
						my $range_overlap_1=${$self->{'__overlap'}->{'__h_result'}}{$seq_id_overlap};
						
						my $range_overlap_2;
						if (
						 defined(${$self->{'__overlap'}->{'__h_match'}}{$seq_name."_".$seq_id_overlap}) )
						{ # le range de seq 2 est e l'indica 0
							
							$range_overlap_2=\${$self->{'__overlap'}->{'__h_match'}}{$seq_name."_".$seq_id_overlap}[0];
						}
						elsif ( defined(${$self->{'__overlap'}->{'__h_match'}}{$seq_id_overlap."_".$seq_name}) )
						{	
							$range_overlap_2=\${$self->{'__overlap'}->{'__h_match'}}{$seq_id_overlap."_".$seq_name}[1];
						}
						
						#calcul des coordonnee de l'arn detecte sur la sequence qui l'overlape
						my ($start, $end, $strand) = $self->TranslateRna($range_overlap_1, $range_overlap_2,$tmp_rna );
						# creation d'un arn fictif pour comparer les coordonees.
						my $fictif_rna = New Rna();
						$$R_LOGGER->Trace("Learn\tOVERLAP Create fictif rna\t $seq_id_overlap\n",$LOGGER_STAMP,"D"); 
						$fictif_rna->Initialize($rna_id, $rna_id,$tmp_rna->GetDesc(), $self->{'__today'}, "", $tmp_rna->GetPath(),$prg_ss , $prg_convert);
						
						${$self->{'__h_sequence'}}{$seq_id_overlap}->SetSequence();
						$fictif_rna->SetPrincipalRange( $start, $end, $strand, \${$self->{'__h_sequence'}}{$seq_id_overlap}, $r_prog_detect, $date_detect);
						#On verifie si l'arn fictif correspond a un arn deja existant.
						($find, $rna_exist_id) = $self->RnaExists(\$fictif_rna, \${$self->{'__h_sequence'}}{$seq_id_overlap});
						
						${$self->{'__h_sequence'}}{$seq_id_overlap}->UnsetSequence();
						# si oui on cree un range secondaire sur la sequence en cour d'analyse e l'arn correspondant
						if ( $find == 1 ) 
						{
						$$r_sequence->AddRna(\${$self->{'__h_rna'}}{$rna_exist_id});
						${$self->{'__h_rna'}}{$rna_exist_id}->AddSecondaryRange( $ft->start(), $ft->end(), $ft->strand(), $r_sequence, \$H_PROGRAM_ACTUAL{$prog});
						last;
						}
						
					}
				}
				
			}
			# si ARN n'existe pas
			if ($find == 0 )
			{
				$$r_sequence->AddRna(\${$self->{'__h_rna'}}{$rna_id});
				$self->AddRna(\$tmp_rna);
				# On incremente id de sequence :
				$$r_sequence->RegisterRna(${$self->{'__h_sequence'}}{$seq_name}->GetNextRnaId());
			}
		}
	
	}
	$$r_sequence->UnsetSequence();
}

=head2 procedure TranslateRna

 Title        : TranslateRna
 Usage        : $MyLearn->TranslateRna($range_overlap_1, $range_overlap_2,$tmp_rna);
 Function     : Translate bound of $tmp_rna from $range_overlap_2 to $range_overlap_1
 		cf in documentation /user_learn/doc/model/chevauchement.odt
 Args         : $range_overlap_1 : range on which we want to have bounds
 		$range_overlap_2 : range on which rna was found
 		$tmp_rna : rna founded.
 Globals      : none

=cut 

sub TranslateRna
{
	my ($self,$range_overlap_1, $range_overlap_2,$tmp_rna) = @_;
	my ($start,$end,$strand);
	if ( $$range_overlap_1->GetStrand() eq $$range_overlap_2->GetStrand() )
	{
		$start =  $$range_overlap_1->GetStart() + ( $tmp_rna->GetPrincipalRange()->GetStart() - $$range_overlap_2->GetStart() );
		$end =  $$range_overlap_1->GetStart() + ( $tmp_rna->GetPrincipalRange()->GetEnd() - $$range_overlap_2->GetStart() );
		$strand = $tmp_rna->GetPrincipalRange()->GetStrand();
	}
	else
	{
		$start =  $$range_overlap_1->GetStart() + ( $$range_overlap_2->GetEnd() - $tmp_rna->GetPrincipalRange()->GetEnd() );
		$end =  $$range_overlap_1->GetStart() + ( $$range_overlap_2->GetEnd() - $tmp_rna->GetPrincipalRange()->GetStart() );
		if ( $tmp_rna->GetPrincipalRange()->GetStrand() eq "-" )
		{
			$strand = "+";
		}
		else
		{
			$strand = "-";
		}
	}

	
	return ($start, $end, $strand);
}


=head2 procedure GetRnaOn

 Title        : GetRnaOn
 Usage        : $MyLearn->GetRnaOn();
 Function     : Get rna on an range of a sequence ( usualy an overlap)
 Returns      : array of rna
 Args         : $seq_id_overlap 
 		$range_overlap  
 Globals      : none

=cut 
sub GetRnaOn
{
	my ($self,$seq_id_overlap,$range_overlap) = @_;
	my $tmp_rna = New Rna();
	my @res;
	$tmp_rna->SetPrincipalRange($$range_overlap->GetStart(), $$range_overlap->GetEnd(), $$range_overlap->GetStrand(), \${$self->{'__h_sequence'}}{$seq_id_overlap});
	foreach my $id ( keys %{$self->{'__h_rna'}} )
	{
		if (${${$self->{'__h_rna'}}{$id}->GetPrincipalRange()->GetSequence()}->GetId() eq $seq_id_overlap)
		{
			if (${$self->{'__h_rna'}}{$id}->GetPrincipalRange()->GetStrand() eq $$range_overlap->GetStrand())
			{
				if (${$self->{'__h_rna'}}{$id}->GetPrincipalRange()->GetStart() > $$range_overlap->GetStart()
				&& ${$self->{'__h_rna'}}{$id}->GetPrincipalRange()->GetEnd() < $$range_overlap->GetEnd() ) # l'arn est inclu dans la zone de Range_overlap
				{
					push (@res, $id);
				}
			}
		}
	}
}


=head2 procedure AddRna

 Title        : AddRna
 Usage        : $MyLearn->AddRna($r_rna);
 Prerequisite : none
 Function     : Add an Rna in the hash of Rna
 Args         : $r_rna : reference on rna
 Globals      : none

=cut 

sub AddRna
{
	my ($self, $r_rna) = @_;
	my $rna_id=$$r_rna->GetId();
	$$R_LOGGER->Trace("Learn\tAddRna\t$rna_id\n",$LOGGER_STAMP,"DE");
	#rna
	${$self->{'__h_rna'}}{$rna_id}=New Rna();
	# initialisation des attributs de rna
	${$self->{'__h_rna'}}{$rna_id}->Initialize($$r_rna->GetId(), $$r_rna->GetName(), $$r_rna->GetDesc(), $$r_rna->GetLastUpd() ,$$r_rna->GetStatus(),$$r_rna->GetPath(),$$r_rna->GetProgForStructure(), $$r_rna->GetStructure()->GetProgramConvert(), $$r_rna->GetFamilyAcc());

	${$self->{'__h_rna'}}{$rna_id}->SetDbxref($$r_rna->GetDbxref());
	# initialisation des attributs de structure / pour le programme fait a partir de rna
	my $struct=$$r_rna->GetStructure()->GetStructure();
	${$self->{'__h_rna'}}{$rna_id}->GetStructure()->SetStructure($struct) if (defined($struct));
	my $free_en= $$r_rna->GetStructure()->GetFreeEnergy();
	${$self->{'__h_rna'}}{$rna_id}->GetStructure()->SetFreeEnergy( $free_en) if (defined ( $free_en));
	my $date_compute= $$r_rna->GetStructure()->GetDateCompute();
	${$self->{'__h_rna'}}{$rna_id}->GetStructure()->SetDateCompute($date_compute) if (defined($date_compute));
	my $ss_file = $$r_rna->GetStructure()->GetSSFile();
	${$self->{'__h_rna'}}{$rna_id}->GetStructure()->SetSSFile($ss_file ) if (defined ($ss_file ));
	my $rnaml_file = $$r_rna->GetStructure()->GetRnamlFile();
	${$self->{'__h_rna'}}{$rna_id}->GetStructure()->SetRnamlFile($rnaml_file ) if (defined ($rnaml_file ));
	my $img_file = $$r_rna->GetStructure()->GetImage();
	${$self->{'__h_rna'}}{$rna_id}->GetStructure()->SetImage($img_file) if (defined ($img_file));
	
	#range
	# initialisation des attributs de range
	my $range=$$r_rna->GetPrincipalRange();
	if (ref ($range))
	{
		${$self->{'__h_rna'}}{$rna_id}->SetPrincipalRange($range->GetStart(),$range->GetEnd(),$range->GetStrand(),$range->GetSequence(), $range->GetProgDetect(),$range->GetDateDetect(),$range->GetSeqData(),$range->GetLength() );
	}
	# range secondaire
	my $nbr=$$r_rna->GetNumberOfSecondaryRange();
	foreach my $i (0..$nbr)
	{
		$range = $$r_rna->GetSecondaryRange($i);
		if (ref ($range))
		{
			${$self->{'__h_rna'}}{$rna_id}->AddSecondaryRange($range->GetStart(),$range->GetEnd(),$range->GetStrand(),$range->GetSequence(), $range->GetProgDetect(),$range->GetDateDetect(),$range->GetSeqData(),$range->GetLength());
		#	print "\nWARNNNN : 			${$self->{'__h_rna'}}{$rna_id}->AddSecondaryRange($range->GetStart(),$range->GetEnd(),$range->GetStrand(),$range->GetSequence(), $range->GetProgDetect(),$range->GetDateDetect(),$range->GetSeqData(),$range->GetLength() )\n";
		}
	}
	# annotation !!
	my $nb_annot=$$r_rna->GetNbAnnotations();
	if ($nb_annot>=0)
	{
		my $i=0;
		while ($i <= $nb_annot)
		{	
			${$self->{'__h_rna'}}{$rna_id}->AddNewAnnotate($$r_rna->GetAnnotate($i)->GetStart(),
			$$r_rna->GetAnnotate($i)->GetEnd(),
			$$r_rna->GetAnnotate($i)->GetTitle(),
			$$r_rna->GetAnnotate($i)->GetLegend(),
			$$r_rna->GetAnnotate($i)->GetLogin(),
			$$r_rna->GetAnnotate($i)->GetTimeStamp());
			$i++;
		}
	}
}



=head2 procedure RnaExists

 Title        : RnaExists
 Usage        : $MyLearn->RnaExists($r_rna);
 Prerequisite : none
 Function     : Parse hash rna and find if rna exist
 Returns      : $id : hash key of the rna which exists
 Args         : $r_rna
 Globals      : none

=cut 

sub RnaExists
{
	my ($self, $r_rna, $r_sequence) = @_;
	my $id;
	my $find=0;
	my $res=undef;
	if ( defined ($r_sequence))
	{
		foreach $id ( @{$$r_sequence->{'__ra_rna'}} )
		{
			
			$find=${$self->{'__h_rna'}}{$$id->GetId()}->IsSame($r_rna);
			if ($find == 1) 
			{ 
				$res=$$id->GetId();
				last; 
			}
		}
	}
	else
	{
		foreach $id ( keys %{$self->{'__h_rna'}} )
		{
			$find=${$self->{'__h_rna'}}{$id}->IsSame($r_rna);
			if ($find == 1) 
			{ 
				$res=$id;
				last; 
			}
	
		}
	}
	# retourne les type de "same" et l'id de l'arn
	return ($find, $res);
}

=head2 procedure IsUpgraded

 Title        : IsUpgraded
 Usage        : $MyLearn->IsUpgraded(seq_name);
 Prerequisite : none
 Function     : Return the old sequence name ( already analysed) corresponding to the new bac
 Globals      : none

=cut 

sub GetOldAccBac
{
	my ($self, $seq_name)=@_;
	my $old_bac="";
	
	foreach my $a_bacs (@{$self->{'__bac_upgraded'}} )
	{
	
	if ($$a_bacs[1] eq $seq_name )
		{
			$old_bac = $$a_bacs[0];
			last;
		
		}	
		elsif ($$a_bacs[0] eq $seq_name)
		{
			$old_bac = $$a_bacs[1];
			last;
		}	
	}
	
	return $old_bac;
}

=head2 procedure ComputeAllRnaStructure

 Title        : ComputeAllRnaStructure
 Usage        : $MyLearn->ComputeAllRnaStructure();
 Prerequisite : none
 Function     : Compute structure for all rna , to be done when LoadRna is finished
 Globals      : none

=cut 

sub ComputeAllRnaStructure
{
	my ($self) = @_;
	$$R_LOGGER->Trace("Learn\tComputeAllRnaStructure\n",$LOGGER_STAMP,"DE");
	my ($id);
	foreach $id ( keys %{$self->{'__h_rna'}} ) 
	{
		${$self->{'__h_rna'}}{$id}->ComputeStructure();
	}
}


=head2 procedure SaveRnasToRnaml

 Title        : SaveRnasToRnaml
 Usage        : $MyLearn->SaveRnasToRnaml();
 Function     : Dump all rna in rnaml
 
=cut 

sub SaveRnasToRnaml
{
	my ($self) = @_;
	my ($id);
	print "Save rna objets \n";
	$$R_LOGGER->Trace("Learn\tSaveRnasToRnaml\n",$LOGGER_STAMP,"DE");
	foreach $id ( keys %{$self->{'__h_rna'}} ) 
	{
		${$self->{'__h_rna'}}{$id}->SaveRnaml($$self{'__o_param'}->Get('path_new_release')."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml'),$$self{'__o_param'}->Get('path_new_release')."/".$$self{'__o_param'}->Get('path_db_learn_rna_tmp'));
	}
	
}

=head2 procedure BuildFamily

 Title        : BuildFamily
 Usage        : $MyLearn->BuildFamily();
 Prerequisite : 
 Function     : For each Rna , test if have a family and add it, else build a new family
 
=cut 

sub BuildFamily
{

	my ($self) = @_;
	print "Build RnaFamily objets \n";
	$$R_LOGGER->Trace("Learn\tBuildFamily\n",$LOGGER_STAMP,"DE");
	
	# Pour tout les ARN on va chercher a quelle famille ils appartiennent 
	foreach my $rna_id ( keys %{$self->{'__h_rna'}} ) 
	{
		next if (defined(${$self->{'__a_loaded_rna'}}{$rna_id})); #si deja analisee ds old release !
		my $id_fam = $self->RnaBelongsFamily($rna_id);	
		$$R_LOGGER->Trace("\tLearn : BuildFamily Rna $rna_id  : Belongs to $id_fam \n",$LOGGER_STAMP,"DE");
		if ( (not defined($id_fam)) || $id_fam eq "" || $id_fam eq "-1") # si rna n'appartien a aucune family
		{	#cree une famille
			$id_fam=sprintf("%s%05i", "RFL", $self->{'__last_family_id'});
			$$R_LOGGER->Trace("\tLearn : BuildFamily New Family :$id_fam \n",$LOGGER_STAMP,"DE");
			${$self->{'__h_family'}}{$id_fam}= New RnaFamily();
			$self->{'__last_family_id'}++;
			#parametre attendu :			
			#$id, $name, $desc, $last_upd, $db_xref, $status, $path, $rfl_dir, $r_prog_struct, $r_prog_align,$r_prog_build, $r_prog_search
			${$self->{'__h_family'}}{$id_fam}->Initialize(
			"$id_fam", 
			"$id_fam", 
			$STATUS_DEFAULT, 
			$self->{'__today'}, 
			"", 
			$STATUS_DEFAULT, 
			$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family'), 
			$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl'),
			\$H_PROGRAM_ACTUAL{'RFL_FOLD'}, 
			\$H_PROGRAM_ACTUAL{'RFL_ALIGN'}, 
			\$H_PROGRAM_ACTUAL{'RFL_BUILD'});
		}
		# dans tout les cas on ajoute l'arn a la famille
		
		if ( ! defined(${$self->{'__h_family'}}{$id_fam}) )
		{
			print "Warning: the family $id_fam does not exist, unable to add the rna $rna_id\n";
			${$self->{'__h_rna'}}{$rna_id}->Print();
		}
		else
		{
			${$self->{'__h_family'}}{$id_fam}->AddRna(\${$self->{'__h_rna'}}{$rna_id});
	
			${$self->{'__h_rna'}}{$rna_id}->SetFamilyAcc($id_fam);
			# complete la bd interne utilise pour la recherche de l'appartenance d'un arn a une famille
		
			$self->{'__h_family'}{$id_fam}->BuildCm($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam'), $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/$id_fam.cm");
		}
		
		$self->ConcatInternDb($rna_id,$id_fam);
		$self->FormatInternDb();
		${$self->{'__h_rna'}}{$rna_id}->ClearTmpDir();
	}

}

=head2 procedure ImproveSSFamilies

 Title        : ImproveSSFamilies
 Usage        : $MyLearn->ImproveSSFamilies();
 Prerequisite : Each family was computed
 Function     : For each family, call the function to improve the secondary structure image.

=cut 

sub ImproveSSFamilies
{
	my ($self) = @_;
	my ($id, $prog);
	foreach $id ( keys %{$self->{'__h_family'}} ) 
	{
		${$self->{'__h_family'}}{$id}->ImproveSS();	
		${$self->{'__h_family'}}{$id}->DumpMss();
	}
	
}

=head2 procedure FormatInternDb

 Title        : FormatInternDb
 Usage        : $MyLearn->FormatInternDb();
 Prerequisite : File of intern db exist
 Function     : Execute format programme on db fasta file.

=cut 

sub FormatInternDb
{
	my ($self) = @_;
	my $fasta_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_fasta_db'};

	#formatage de la banque :
	$H_PROGRAM_ACTUAL{'FORMAT'}->SetInOut($fasta_file,"");
	$H_PROGRAM_ACTUAL{'FORMAT'}->Run();
}



=head2 procedure ConcatInternDb

 Title        : ConcatInternDb
 Usage        : $MyLearn->ConcatInternDb($rna_id, $id_fam);
 Prerequisite : $rna_id corresponding to rna
 Function     : Concat fasta data in fasta file and in thr file
 Args         : $rna_id : id of rna which sequence data will be concat
		$id_fam : family which rna_id belongs
 Globals      : none

=cut 

sub ConcatInternDb
{
	my ($self, $rna_id,$id_fam) = @_;
	
	if ( $self->{'__h_family'}{$id_fam}->GetLength() < $$self{'__o_param'}->Get('param_family_threshold_length') )
	{
	
	# ajout de la sequence fasta dans le fichier intern
	my $fasta_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_fasta_db'};

	my $family_name=$self->{'__h_family'}{$id_fam}->GetName();
	
	chomp ($family_name);
	$family_name=~s/\s$//;
	# header du fasta : >AF096719.1/375-497 RF00001;5S_rRNA; POUR RFAM
	my $seq = Bio::Seq->new(
		-id 	=> ${$self->{'__h_rna'}{$rna_id}->GetPrincipalRange()->GetSequence()}->GetId()."/".$self->{'__h_rna'}{$rna_id}->GetPrincipalRange()->GetStart()."-".$self->{'__h_rna'}{$rna_id}->GetPrincipalRange()->GetEnd(),
		-desc 	=> $self->{'__h_rna'}{$rna_id}->GetFamilyAcc().";".$family_name.";" , 
		-seq 	=> $self->{'__h_rna'}{$rna_id}->GetPrincipalRange()->GetSeqData());
	my $out = Bio::SeqIO->new( -file => ">>$fasta_file", '-format' => 'Fasta' );
	$out->write_seq($seq);
	$out->DESTROY();
	
	# construction du modele dans le repertoire intern !
	my $thr_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_thr'};
	open (THR , ">>$thr_file") or die "Impossible d'ouvrir thr : >$thr_file<\n";
	my $len=$self->{'__h_family'}{$id_fam}->GetLength();
	# arrondi a la dizaine supperieure
	$len+=10;
	$len=~s/.$/0/;
	#print THR "$id_fam\t".$id_fam."\t5\t$len\tglobal\n";
	print THR "$id_fam\t".$family_name."\t5\t$len\tglobal\n";
	close (THR);
	}
}


=head2 procedure RnaBelongsFamily

 Title        : RnaBelongsFamily
 Usage        : $MyLearn->RnaBelongsFamily();
 Prerequisite : Rfl db is updated and formated
 Function     : Execute ISCAN (on rfl db) to find the family of rna
 Args         : $rna_id
 Globals      : none

=cut 

sub RnaBelongsFamily
{
	my ($self, $rna_id) = @_;
	my ($best_family, $best_score)=("", 0);

	if ($self->{'__last_family_id'} >1) # si on a contruis une famille!
	{
		my $file=${$self->{'__h_rna'}}{$rna_id}->DumpFasta();
		$H_PROGRAM_ACTUAL{'ISCAN'}->SetInOut($file,$file.".rfs");
		$H_PROGRAM_ACTUAL{'ISCAN'}->Run('gff');
	
		#################
		my $gff_file=$H_PROGRAM_ACTUAL{'ISCAN'}->GetFileOut();
		if (! -e $gff_file)
		{
			return ($best_family);
		}
		my $cmd = "sort -u $gff_file > $gff_file.srt ; mv $gff_file.srt $gff_file ; rm -rf $gff_file.srt";
		system $cmd;
 		open (TGFF, $gff_file) || return ($best_family); # si pas d'ouverture alors pas de resultat dc on sort
		my $gff_data = new GFF::GeneFeatureSet();
		$gff_data->version(2);
		$gff_data->read(\*TGFF);
		close (TGFF);
	
 		my @a_t = $gff_data->eachGeneFeature();
		# loop over the input stream
		foreach my $ft  (@a_t) 
		{	# ex of tag value: "ACC=RF00001;ID=5S_rRNA;SCORE=64.14;"

			my $att = $ft->tagValue('Att');
			
			my ($score) = $att =~ /SCORE=([^\;\,]+)[;,]/;
			if ($score>$best_score)
			{
				$best_score=$score;
				($best_family) = $att =~ /ACC=([^\;\,]+)[,;]/;
			}
			
		}
	}
	#clear tmp directory
	
	return ($best_family);
}


=head2 procedure SaveFamiliesToRnaml

 Title        : SaveFamiliesToRnaml
 Usage        : $MyLearn->SaveFamiliesToRnaml();
 Prerequisite : Multalin file have been created
 Function     : Save all family in rnaml

=cut 

sub SaveFamiliesToRnaml
{
	my ($self) = @_;
	my ($id);
	print "Save RNA family objets \n";
	$$R_LOGGER->Trace("Learn\tSaveFamiliesToRnaml\n",$LOGGER_STAMP,"DE");
	foreach $id ( keys %{$self->{'__h_family'}} ) 
	{
		${$self->{'__h_family'}}{$id}->SaveRnaml($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_xml'),$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_tmp'));
	}
	
}

=head2 procedure DumpStatus

 Title        : DumpStatus
 Usage        : $MyLearn->DumpStatus($file);
 Prerequisite : All treatment done
 Function     : Save data information, number on analyzed sequence, of rna ...
 Args         : $file : file to write data of db
  		$old_file :  old status file (data will be concat)

=cut 

sub DumpStatus
{
	my ($self, $file, $old_file) = @_;
	my %h_status;
	my $len=0;
	if ( -e $old_file )
	{
		open (STAT, "$old_file");
		while (my $line = <STAT>)
		{
			chomp($line);
			if ($line =~ /SEQ|BP/)
			{
				my @a_tmp=split ( ' ', $line);
				$h_status{$a_tmp[0]}=$a_tmp[1];
			}
			if ($line =~ /ORG/)
			{
				my @a_tmp=split ( /[\s]+/, $line,2);
				$h_status{'ORG'}=$a_tmp[1];
			
			}
		}
		close STAT;
	}

	foreach my $id  ( keys  %{$self->{'__h_sequence'}} ) 
	{
		$h_status{'BP'}+=${$self->{'__h_sequence'}}{$id}->GetLength() if ( not defined( ${$self->{'__a_loaded_rna'}}{ $id }) ); # si ne fait pas partie de l'ancienne release car le champs n'est pas renseigner.
	}
	my $cmd="find  ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_source_sequence')." -type d ";
	my @res=`$cmd`;
	$h_status{'SEQ'}=$#res;
	#$h_status{'SEQ'}--; #compte de repertoire courrant.
	$h_status{'RNA'}=scalar(keys(%{$self->{'__h_rna'}}));
	$h_status{'FAMILY'}=scalar(keys(%{$self->{'__h_family'}}));
	$h_status{'ORG'}=$h_status{'ORG'}.";".$$self{'__o_param'}->Get('species');
	open (STAT, ">$file") or die "Can't open $file";
	print STAT "SEQ\t". $h_status{'SEQ'} ."\n";
	print STAT "BP\t".$h_status{'BP'} ."\n";	
	print STAT "RNA\t". $h_status{'RNA'}."\n";
	print STAT "FAMILY\t". $h_status{'FAMILY'} ."\n";
	print STAT "ORG\t". $h_status{'ORG'};
	print STAT "\nPROG\n";

	foreach my $id ( keys %H_PROGRAM_ACTUAL ) 
	{
		print STAT $H_PROGRAM_ACTUAL{$id}->GetName()." v".$H_PROGRAM_ACTUAL{$id}->GetVersion()."\n";
	}

	close STAT;
}

=head2 procedure CreateProgramsFromRelease

 Title        : CreateProgramsFromRelease
 Usage        : $MyLearn->CreateProgramsFromRelease($dir);
 Prerequisite : xml data file exist for rna and families
 Function     : From xml files load programs virtual
 Args         : $dir

=cut 

sub CreateProgramsFromRelease
{
	my ($self, $xml_dir) = @_;

	my @lst_prg = ();
	if ( -d $xml_dir )
	{
		my $cmd = "find $xml_dir -name \"*$Var::E_XML\" -exec ".$$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_analysis')." \'{}\' \\;  | sort -u";
		#print $cmd."\n";
		@lst_prg=`$cmd`;
	}
	foreach my $prg (@lst_prg)
	{
		chomp ($prg);		
		next if ($prg eq "" || $prg=~/<?xml version=/) ;
		# ex : mfold-3.2 ou mfold-
		$$R_LOGGER->Trace("Learn\tCreateProgramsFromRelease\n$prg\n",$LOGGER_STAMP,"DE");

		$prg =~ s/\-$/\-0/;
		my @a_T=split ("-" , $prg );
		my $types= $H_PROG_TYPE{$a_T[0]};
		my @a_typ=split(' ', $types);
		$H_PROGRAM_PSEUDO{$prg}= New Program($a_T[0]," " ," ", $a_typ[0] , $a_T[1] ,$R_LOGGER, $LOGGER_STAMP );
		$H_PROGRAM_PSEUDO{$prg}->SetVirtual(1);
		
	}
}
=head2 procedure CreateSequencesFromRelease

 Title        : CreateSequencesFromRelease
 Usage        : $MyLearn->CreateSequencesFromRelease($dir);
 Prerequisite : xml data file exist for rna and families
 Function     : From xml files load sequences
 Args         : $dir

=cut 

sub CreateSequencesFromRelease
{
	my ($self, $dir_rna) = @_;

	my @lst_sequence = ();
	if ( -d $dir_rna )
	{
		my $cmd = "find $dir_rna -name \"*$Var::E_XML\" -exec ".$$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_sequences')." \'{}\' \\;  | sort -u";
		$$R_LOGGER->Trace("Learn\tCreateSequencesFromRelease\n$cmd\n",$LOGGER_STAMP,"DE");
		#print $cmd."\n";
		@lst_sequence=`$cmd`;
	}
	foreach my $seq (@lst_sequence)
	{
		chomp ($seq);		
		next if ($seq eq "" || $seq=~/<?xml version=/) ;
		# $seq seems AccSequence:Species
		my @a_T=split (":" , $seq );
		my $id=$a_T[0];
		$$R_LOGGER->Trace("Learn\tLoadRelease\tLoadSequence: $id\n",$LOGGER_STAMP,"D");
		if (not defined(${$self->{'__h_sequence'}}{$id}))
		{
		$$R_LOGGER->Trace("Learn\tCreateSequencesFromRelease\n$id\n",$LOGGER_STAMP,"DE");

		#ex : $self->AddSequence("/www/LeARN/data/rel_test1/source/sequences/AC15/AC15","AC15", "Medicago truncatula");
		$self->AddSequence(undef,$id, $a_T[1]);
		${$self->{'__h_sequence'}}{$id}->SetAnalyzed("1");
		
		}
	}

}
=head2 procedure CreateRnasFromRelease

 Title        : CreateRnasFromRelease
 Usage        : $MyLearn->CreateRnasFromRelease($dir);
 Prerequisite : xml data file exist for rna
 Function     : From xml files load rnas
 Args         : $dir

=cut 

sub CreateRnasFromRelease
{
	my ($self, $dir_rna, $new_dir_rna) = @_;
	my @lst_file_rna=();
	if ( -d $dir_rna )
	{
		my $cmd = "find $dir_rna -name \"*$Var::E_XML\" ";
		$$R_LOGGER->Trace("Learn\tCreateRnasFromRelease\n$cmd\n","*","DE");
		@lst_file_rna=`$cmd`;
	}
	my $rh_seq=\%{$self->{'__h_sequence'}};
	my $rh_prog = \%H_PROGRAM_PSEUDO;
	foreach my $file (@lst_file_rna) # 1 fichier == 1rna
	{
		chomp $file;
		my $rna = New Rna();
		
		$rna->LoadFromRnaml("$file", $new_dir_rna,$rh_seq );
		$self->AddRna(\$rna);
		my $rna_id=$rna->GetId();
		$$R_LOGGER->Trace("Learn\tCreateRnasFromRelease\n$rna_id\n",$LOGGER_STAMP,"DE");

		${$self->{'__a_loaded_rna'}}{$rna_id}=1;

		my $r_seq = $rna->GetPrincipalRange()->GetSequence();
		my ($number)= $rna_id=~/([0-9]*)$/;
		if ($number >= $$r_seq->GetNextRnaId())
		{
			$$r_seq->RegisterRna($rna_id);
		}
	}	
}



=head2 procedure CreateFamiliesFromRelease

 Title        : CreateFamiliesFromRelease
 Usage        : $MyLearn->CreateFamiliesFromRelease($dir);
 Prerequisite : xml data file exist for families
 Function     : From xml files load families
 Args         : $dir

=cut 

sub CreateFamiliesFromRelease
{
	my ($self, $dir_families, $new_dir_family) = @_;
	my $rh_seq=\%{$self->{'__h_sequence'}};
	my $rh_prog = \%H_PROGRAM_PSEUDO;

	my @lst_file_family=();
	if ( -d $dir_families )
	{
		my $cmd = "find $dir_families -name \"*$Var::E_XML\" ";
		print "CreateFamiliesFromRelease\n";
		$$R_LOGGER->Trace("Learn\tCreateFamiliesFromRelease\n$cmd\n","*","DE");
		@lst_file_family=`$cmd`;
	}
	foreach my $file (@lst_file_family) # 1 fichier == 1 famille
	{
		chomp $file;
		#xslt qui recupere l'id de la famille (pr ne pas se baser sur le nom de fichier)
		my $cmd = $$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_familyid')." $file  | grep RFL";
		$$R_LOGGER->Trace("Learn\tCreateFamiliesFromRelease\n$cmd\n","*","DE");
		#la commande retourne : 
		#<?xml version="1.0"?>
		#RFL00004
		my @a_tab_fam=`$cmd`;
		chomp(@a_tab_fam);
		#on recupere la seconde ligne (ou dernier elt)
		my $id_fam=$a_tab_fam[0];
		print "Loading: $id_fam\n";
		next if (not defined ($id_fam));
		
		chomp($id_fam);
		${$self->{'__h_family'}}{$id_fam}= New RnaFamily();
	
		$cmd = $$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_RnasOfFamily')." $file  | sort -u";
		$$R_LOGGER->Trace("Learn\tCreateFamiliesFromRelease\n$cmd\n","*","DE");
		#ex de retour :
		# <?xml version="1.0"?>
		# AC140023_r1
		# AC140023_r2
		my @a_tab_id_rna=`$cmd`;
		foreach my $rna (@a_tab_id_rna)
		{
			chomp ($rna);		
			next if ($rna eq "" || $rna=~/<?xml version=/) ;
			if (defined (${$self->{'__h_rna'}}{$rna}) )
			{
				${$self->{'__h_family'}}{$id_fam}->AddRna(\${$self->{'__h_rna'}}{$rna});
			}
			else
			{	
				$$R_LOGGER->Trace("Learn\tLoadRelease\tWARN Rna $rna not Loaded in Family : $id_fam\n",$LOGGER_STAMP,"DE");
			}
		}
		if (${$self->{'__h_family'}}{$id_fam}->NumberOfRna() == 0)
		{
			delete ${$self->{'__h_family'}}{$id_fam};
			$$R_LOGGER->Trace("Learn\tCreateFamilyFromRelease\n Warn Delete $id_fam, contains no rna !\n",$LOGGER_STAMP,"DE");
		}
		else
		{
			${$self->{'__h_family'}}{$id_fam}->LoadFromRnaml($file, $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl'), $new_dir_family,\$H_PROGRAM_ACTUAL{'RFL_BUILD'}, \$H_PROGRAM_ACTUAL{'RFL_SEARCH'} );
		}
	}	

	$self->{'__last_family_id'}+=$self->GetMaxIdOfFamily();


}

=head2 procedure LoadRelease

 Title        : LoadRelease
 Usage        : $MyLearn->LoadRelease($dir);
 Prerequisite : xml data file exist for rna and families
 Function     : From xml files load programs, rnas, families
 Args         : $dir

=cut 
sub LoadRelease
{
	my ($self) = @_;
	$$R_LOGGER->Trace("Learn\tLoadRelease\t".$self->{'__old_release'}."\n",$LOGGER_STAMP,"DE");

	my $dir_family=$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_xml');
	my $dir_rna=$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml');
	my $new_dir_rna=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna');
	my $new_dir_family=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family');

	#### selection des programmes et creation des non existants.
	$self->CreateProgramsFromRelease($dir_rna);
	$self->CreateProgramsFromRelease($dir_family);

	#### selection des sequences et creation des non existants.
	$self->CreateSequencesFromRelease($dir_rna);
	
	### traitement de RNA par fichier xml 
	$self->CreateRnasFromRelease($dir_rna,$new_dir_rna);

	### traitement des Family par fichier xml 
	$self->CreateFamiliesFromRelease($dir_family,$new_dir_family);
}

=head2 procedure LoadReleaseFromListOfFamily

 Title        : LoadReleaseFromListOfFamily
 Usage        : $MyLearn->LoadReleaseFromListOfFamily(@families);
 Prerequisite : xml data file exist for rna and families
 Function     : From xml files load programs, rnas, families
 Args         : $dir
 		$rna_file: file of rna wich was modified (CGI : when view family updated)

=cut 
sub LoadReleaseFromListOfFamily
{
	my ($self, $rna_file, @families) = (@_);

# on va creer une release temporaire avec les fichier qui nous interressent pour faire un chargement classique ...
	$$R_LOGGER->Trace("Learn\tLoadReleaseFromListOfFamily\t".$self->{'__old_release'}."\n",$LOGGER_STAMP,"DE");
	
	my $tmp_release=$self->{'__current_release'}."/tmp";
	my $cmd;
	if ( -d $tmp_release ) 
	{
		$cmd="rm -rf $tmp_release";
		&SystemLaunchLog($cmd);
	}
	my $tmp_family=$tmp_release."/".$$self{'__o_param'}->Get('path_db_learn_family_xml');
	my $tmp_rna=$tmp_release."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml');
	$cmd="mkdir -p $tmp_family";
	&SystemLaunchLog($cmd);
	$cmd="mkdir -p $tmp_rna";
	&SystemLaunchLog($cmd);

	my $dir_family=$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_xml');
	my $dir_rna=$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml');
	my $new_dir_rna=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna');
	my $new_dir_family=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family');
	my $new_xml_family=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family_xml');
	my $new_xml_rna=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml');
	
	foreach my $family (@families)
	{
		next if ($family eq "" || -e $family);
				
		# on verif que le fichier n'est pas dans la current release (a partir du formulaire lors de la re execution du cgi)
		
		if ( -e  "$new_xml_family/$family$Var::E_XML" )
		{
			$cmd = "cp $new_xml_family/$family$Var::E_XML $tmp_family";
		}
		else # on prend celui de la old release
		{
			$cmd = "cp $dir_family/$family$Var::E_XML $tmp_family";
		}
		&SystemLaunchLog($cmd);
		
		
		if (  ! -d  $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl')."/$family" )
		{
			$cmd="cp -r ".$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_db_rfl')."/$family ".$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl');
		&SystemLaunchLog($cmd);
		}

		my @a_tab_id_rna = ();
		if ( -d  $tmp_family )
		{
			$cmd = "find $tmp_family -name \"*$Var::E_XML\" -exec ".$$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_RnasOfFamily')." \'{}\' \\;  | sort -u";

			# old:		$cmd = $$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_RnasOfFamily')." $tmp_family/*  | sort -u";
			&SystemLaunchLog($cmd);
		
			#ex de retour :
			# <?xml version="1.0"?>
			# AC140023_r1
			# AC140023_r2
			@a_tab_id_rna=`$cmd`;
		}
		foreach my $rna (@a_tab_id_rna)
		{
			chomp ($rna);		
			next if ($rna eq "" || $rna=~/<?xml version=/) ;
			&SystemLog("FOR : $rna \n");
			if ( -e  "$new_xml_rna/$rna$Var::E_XML" )
			{
				$cmd = "cp $new_xml_rna/$rna$Var::E_XML $tmp_rna";
			}
			else # on prend celui de la old release
			{
				$cmd = "cp $dir_rna/$rna$Var::E_XML $tmp_rna";
			}
			&SystemLaunchLog($cmd);
			&SystemLog("2");
		}
		if (defined($rna_file) && -e $rna_file)
		{
			$cmd = "cp $rna_file $tmp_rna";
			&SystemLaunchLog($cmd);
			&SystemLog("3");
		}
	}
	my $save_old_release=$self->{'__old_release'};
	$self->{'__old_release'}=$tmp_release;
	$self->LoadRelease();
	$self->{'__old_release'}=$save_old_release;
	
	# on cherche le plus grand accession des familles
	my @lst_fam=();
	my @lst_fam_new=();
	if ( -d $dir_family )
	{
		$cmd = "find $dir_family -name \"RFL$Var::E_XML\" ";
		@lst_fam=`$cmd`;
	}
	if ( -d $new_xml_family )
	{
		$cmd = "find $new_xml_family -name \"RFL$Var::E_XML\" ";
		@lst_fam_new=`$cmd`;
	}
	@lst_fam= sort ( @lst_fam, @lst_fam_new );
	my $fam_xml=$lst_fam[$#lst_fam];
	chomp ($fam_xml);
	($self->{'__last_family_id'})=$fam_xml=~/RFL(\d*)/i;
	$self->{'__last_family_id'}++;
	system "rm -rf $tmp_release";
}

=head2 procedure LoadReleaseFromListOfRna

 Title        : LoadReleaseFromListOfRna
 Usage        : $MyLearn->LoadReleaseFromListOfRna(@rna);
 Prerequisite : xml data file exist for rna and families
 Function     : From xml files load programs, rnas, families
 Args         : $dir

=cut 
sub LoadReleaseFromListOfRna
{
	my ($self, @rna) = (@_);
	$$R_LOGGER->Trace("Learn\tLoadReleaseFromListOfRna\t".$self->{'__old_release'}."\n",$LOGGER_STAMP,"DE");

	# on construit la liste des famille correspondante:
	my @families;
	my $dir_rna=$self->{'__old_release'}."/".$$self{'__o_param'}->Get('path_db_learn_rna_xml');
	foreach my $rna (@rna)
	{

		my $cmd = $$self{'__o_param'}->Get('xsltproc_prg')." ".$$self{'__o_param'}->Get('xsl_FamilyOfRna')." $dir_rna/$rna$Var::E_XML";
		
		my @a_tab_id_fam=`$cmd`;
		foreach my $line (@a_tab_id_fam)
		{
				chomp ($line);		
			next if ($line eq "" || $line=~/<?xml version=/) ;
			push (@families, $line);
		}
	}
	$self->LoadReleaseFromListOfFamily(@families);
}


=head2 procedure GetMaxIdOfFamily

 Title        : GetMaxIdOfFamily
 Usage        : $MyLearn->GetMaxIdOfFamily();
 Prerequisite : at least one family exists
 Function     : Get the max id of family to known the next id
 Returns      : a number max 
 Args         : none
 Globals      : none

=cut 

sub GetMaxIdOfFamily
{
	my ($self)=@_;
	my $max=0;
	my $number;
	foreach my $id ( keys %{$self->{'__h_family'}} ) 
	{
		($number)= $id =~/([0-9]+)$/;
		if (defined($number) && ($number > $max))
		{
			$max=$number;
		}
	}
	
	return $max;
}


=head2 procedure GetRefProgram

 Title        : GetRefProgram
 Usage        : $MyLearn->GetRefProgram();
 Prerequisite : Ref programs is complete with real programs and virtual (from rnamls)
 Function     : From a hash of programs and a name and version of program, find the reference of the programs
 Returns      : Return reference of a programs
 Args         : none
 Globals      : none

=cut 

sub GetRefProgram
{
	my ($value,$type)=(@_);
	# $rh_analysis, $analysis_ids : provenant UtilRnaml !
	# sinon rh_analysis est undef
	if (not defined ($type) )
	{ 
		$type="";
	}
	my $rh_programs;
	my $res=undef;
	$$R_LOGGER->Trace("GetRefProgram:  ".$value." ".$type."\n",$LOGGER_STAMP,"DE");

	if ($type eq "" || $type =~ /pseudo|virtual/)
	{
		$rh_programs= \%H_PROGRAM_PSEUDO;
		if (defined $H_PROGRAM_PSEUDO{$value})
		{ 
			$$R_LOGGER->Trace("GetRefProgram:  ".$H_PROGRAM_PSEUDO{$value}->GetName()." ".$H_PROGRAM_PSEUDO{$value}->GetType()." ".$H_PROGRAM_PSEUDO{$value}->IsVirtual()."\n",$LOGGER_STAMP,"DE");
			$res= \$H_PROGRAM_PSEUDO{$value};
		}
	}
	else
	{
		$rh_programs= \%H_PROGRAM_ACTUAL;
		if (defined ($H_PROGRAM_ACTUAL{$value}))
		{ 
			$$R_LOGGER->Trace("GetRefProgram:  ".$H_PROGRAM_ACTUAL{$value}->GetName()." ".$H_PROGRAM_ACTUAL{$value}->GetType()." ".$H_PROGRAM_ACTUAL{$value}->IsVirtual()."\n",$LOGGER_STAMP,"DE");
			$res= \$H_PROGRAM_ACTUAL{$value};
		}
		else
		{
			# on va cherche dans les virtual le nom du prog :
			my $prog_name=$H_PROGRAM_PSEUDO{$value};
			# pour ce nom on va parcourrir la liste des type pour trouve le programme pouvant satisfaire
			my @a_typ=split(' ', $H_PROG_TYPE{$prog_name});
			foreach my $type ( @a_typ )
			{
				if (defined($H_PROGRAM_ACTUAL{$type}))
				{
					$res=\$H_PROGRAM_ACTUAL{$type};
					last;
				}
			}
		}
	}

	return $res;

}


=head2 procedure DetectFamilyToMerge

 Title        : DetectFamilyToMerge
 Usage        : $MyLearn->DetectFamilyToMerge();
 Prerequisite : All what loaded (rna, family)
 Function     : For each family (ordre decroissant) search if families can be merge.
 Returns      : none
 Args         : none
 Globals      : none

=cut 
sub DetectFamilyToMerge
{
	my ($self, $prg_name) = (@_);
	my ($id);
	
	$$R_LOGGER->Trace("Learn\tDetectFamilyToMerge\n",$LOGGER_STAMP,"DE");
	#format db de rfl !!!
	# supprimer le for_blast si AnalyseMergingVRfam
	my $rfl_fasta_file;
	if ($prg_name eq 'rfam')
	{
		$rfl_fasta_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_fasta_db'};
	}
	elsif ($prg_name eq 'blast')
	{
		$rfl_fasta_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_fasta_db_for_blast'};
	}
	$H_PROGRAM_ACTUAL{'FORMAT'}->SetInOut($rfl_fasta_file,"");
	$H_PROGRAM_ACTUAL{'FORMAT'}->Run();
	
	$H_PROGRAM_ACTUAL{'SCAN_3'}->SetParam($$self{'__o_param'}->Get('SCAN_3_param_alt')." -d $rfl_fasta_file ");

	foreach $id ( sort {$b cmp $a} keys %{$self->{'__h_family'}} ) 
	{	
		next if ( not defined(${$self->{'__h_family'}}{$id}));
		$$R_LOGGER->Trace("EXT: Analyse Family $id ".${$self->{'__h_family'}}{$id}->GetName()."\n",$LOGGER_STAMP,"D");
		my $fam ;

		if ($prg_name eq 'rfam')
		{
			$fam=${$self->{'__h_family'}}{$id}->AnalyseMerging(\$H_PROGRAM_ACTUAL{'ISCAN'},$prg_name);
		}
		elsif ($prg_name eq 'blast')
		{
			$fam=${$self->{'__h_family'}}{$id}->AnalyseMerging(\$H_PROGRAM_ACTUAL{'SCAN_3'},$prg_name);		
		}
		next if ($fam eq "");

		$$R_LOGGER->Trace("EXT: Learn\tDetectFamilyToMerge\tTrying to merge family $id ".${$self->{'__h_family'}}{$id}->GetName()." and $fam ".${$self->{'__h_family'}}{$fam}->GetName()."\n",$LOGGER_STAMP,"D");
		#print "Learn\tDetectFamilyToMerge\tFamily $id could be merge with $fam\n";
		# creation d'une famille virtuelle
		my $virtual_family=New RnaFamily();
		$virtual_family->Initialize("virtual", "virtual", "virtual", undef, "virtual", "virtual", $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_learn_family'), $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl'), \$H_PROGRAM_ACTUAL{'RFL_FOLD'}, \$H_PROGRAM_ACTUAL{'RFL_ALIGN'},\$H_PROGRAM_ACTUAL{'RFL_BUILD'}, \$H_PROGRAM_ACTUAL{'RFL_SEARCH'}); 
		next if ( not defined(${$self->{'__h_family'}}{$fam}));
		# Teste si les familles sont fusionnables
		if ($virtual_family->MergeTwoFamilies(\${$self->{'__h_family'}}{$id},\${$self->{'__h_family'}}{$fam}))
		{ 
			# effective merge !
			$$R_LOGGER->Trace("EXT: Learn\tDetectFamilyToMerge\tMerging family $id and $fam\n",$LOGGER_STAMP,"D");
			${$self->{'__h_family'}}{$fam}->MergeWithFamily(\${$self->{'__h_family'}}{$id}, $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam'));
			${$self->{'__h_family'}}{$fam}->BuildCm($self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfam'), $self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/$fam.cm");
			delete ${$self->{'__h_family'}}{$id};
			$self->DumpRflDb();
			# supprimer  (une des famille) ou modifier dans le thr, Rfam.fasta (intern)	, dans le Rfl.fasta, les fichiers xml img et le repertoire de la famille dans le inter_rfl
		}
	}
	
}


=head2 procedure DumpRflDb

 Title        : DumpRflDb
 Usage        : $MyLearn->DumpRflDb();
 Prerequisite : All was loaded (rna, family)
 Function     : Create rfl fasta file, rfam.fasta and thr file.
 Returns      : none
 Args         : none
 Globals      : none

=cut 

sub DumpRflDb
{
	my ($self)=(@_);
	my $fasta_rfl_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_fasta_db'};

	my $fasta_rfam_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_fasta_db_for_blast'};

	my $thr_file=$self->{'__current_release'}."/".$$self{'__o_param'}->Get('path_db_rfl_intern')."/".$self->{'__intern_thr'};

	system "rm -f $fasta_rfl_file $fasta_rfam_file $thr_file";
	foreach my $id ( keys %{$self->{'__h_rna'}} ) 
	{
		next if ((not  defined(${$self->{'__h_rna'}}{$id})) || (not defined (${$self->{'__h_family'}}{${$self->{'__h_rna'}}{$id}->GetFamilyAcc()} ))  );
		my $id_fam=${$self->{'__h_rna'}}{$id}->GetFamilyAcc();
		$self->ConcatInternDb($id ,$id_fam);
	}
	$self->FormatInternDb();
}



=head2 procedure GetProgram

 Title        : GetProgram
 Usage        : $MyLearn->GetProgram($name_program);
 Prerequisite : none
 Function     : find the reference of a program with the name in parameter
 Returns      : a reference on the program 
 Args         : $name_program : a program name
 Globals      : none

=cut 
sub GetProgram
{
my ($self,$prog) = @_;
return (\$H_PROGRAM_ACTUAL{$prog});
}

########################
####### PRINT ##########
########################
=head2 procedure PrintSequences

 Title        : PrintSequences
 Usage        : $MyLearn->PrintSequences();
 Prerequisite : none
 Function     : print hr_sequence,
 Returns      : none
 Args         : none
 Globals      : none

=cut 
sub PrintSequences
{
	my ($self) = @_;
	my ($id, $seq);
	foreach $id ( keys %{$self->{'__h_sequence'}} ) 
	{
		${$self->{'__h_sequence'}}{$id}->Print();
	}
	
}


=head2 procedure DumpAnnotationBySequence

 Title        : DumpAnnotationBySequence
 Usage        : $MyLearn->DumpAnnotationBySequence();
 Prerequisite : none
 Function     : For each sequence create a file with all rna annotation of this sequence
 Returns      : none
 Args         : none
 Globals      : none

=cut 
sub DumpAnnotationBySequence
{
	my ($self) = @_;
	my ($id, $seq);
	foreach $id ( keys %{$self->{'__h_sequence'}} ) 
	{
		${$self->{'__h_sequence'}}{$id}->DumpAnnotation();
	}
	
}

=head2 procedure PrintRnas

 Title        : PrintRnas
 Usage        : $MyLearn->PrintRnas();
 Function     : print h_rna,

=cut 
sub PrintRnas
{	
	my ($self) = @_;
	my ($id);
	foreach $id ( keys %{$self->{'__h_rna'}} ) 
	{
		${$self->{'__h_rna'}}{$id}->Print();
	}
	
}

=head2 procedure PrintPrograms

 Title        : PrintPrograms
 Usage        : $MyLearn->PrintPrograms();
 Function     : print hr_program,

=cut 

sub PrintPrograms
{
	my ($self) = @_;
	my ($id, $prog);
	print "number of prg:". scalar(%H_PROGRAM_ACTUAL)."\n";
	foreach $id ( keys %H_PROGRAM_ACTUAL ) 
	{
		print "id : >$id<\n";
		#$H_PROGRAM_ACTUAL{$id}->SetInOut("","");
		print "PRG: $id : ".$H_PROGRAM_ACTUAL{$id}->GetName()." v:".$H_PROGRAM_ACTUAL{$id}->GetVersion()."\n";
		print "\tcmd: ". $H_PROGRAM_ACTUAL{$id}->GetPrg() ."\n";
	}
	
}

=head2 procedure PrintFamilies

 Title        : PrintFamilies
 Usage        : $MyLearn->PrintFamilies();
 Function     : print hr_family,

=cut 

sub PrintFamilies
{
	my ($self) = @_;
	my ($id, $prog);
	foreach $id ( keys %{$self->{'__h_family'}} ) 
	{
		${$self->{'__h_family'}}{$id}->Print();	
	}
	
}




=head2 procedure SystemLaunchLog

 Title        : SystemLaunchLog
 Usage        : $MyLearn->SystemLaunchLog($cmd);
 Prerequisite : none
 Function     : Launch cmd on system , and log it
 Returns      : none
 Args         : $cmd : command
 Globals      : none

=cut 

sub SystemLaunchLog
{	
	my ($cmd) = @_;
	$$R_LOGGER->Trace("Learn\tCMD: $cmd\n",$LOGGER_STAMP,"DE");
	system $cmd;
}


sub SystemLog
{	
	my ($cmd) = @_;
	$$R_LOGGER->Trace("Learn\tCMD: $cmd\n",$LOGGER_STAMP,"DE");
	
}

=head2 procedure GetRna

 Title        : GetRna
 Usage        : $MyLearn->GetRna($id);
 Function     : Find rna identified by $id
 Returns      : an object rna
 Args         : $id : id of rna
 
=cut 

sub GetRna
{
	my ($self, $id) = @_;
	return (${$self->{'__h_rna'}}{$id});
}


=head2 procedure GetFamily

 Title        : GetFamily
 Usage        : $MyLearn->GetFamily($id);
 Function     : Find family identified by $id
 Returns      : an object family
 Args         : $id : id of family
 
=cut 

sub GetFamily
{
	my ($self, $id) = @_;
	return (${$self->{'__h_family'}}{$id});
}
1;

 

