#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
use CGI;
use CGI::Carp qw/fatalsToBrowser/;
use Var ;

=head2 procedure CloseStd

 Title        : CloseStd
 Prerequisite : 
 Function     : Close std flux
 Return       : Copy of STDOUT and STDERR
 Args         :
 
=cut
sub CloseStd
{
   open my $oldout, ">&STDOUT" or die "Can't dup STDOUT: $!";
   open my $olderr, ">&STDERR" or die "Can't dup STDERR: $!";

   close (STDOUT);
   close (STDERR);
	return ($oldout,$olderr);
}


=head2 procedure OpenStd

 Title        : OpenStd
 Prerequisite : 
 Function     : Open std flux
 Args         : $oldout : stdout
                $olderr : stderr
 
=cut

sub OpenStd
{
	my ($oldout, $olderr)=@_;
  	open STDOUT, ">&", $oldout or die "Can't dup \$oldout: $!";
  	open STDERR, ">&", $olderr or die "Can't dup \$olderr: $!";
 }


=head2 procedure MakeCfg

 Title        : MakeCfg
 Prerequisite : 
 Function     : Make a cfg with good param for a LeARN application
 Args         : $current_release for LeARN application ( a tree file will be create )
                $old_release for getting data for computing
 
=cut

sub MakeCfg
{
	my ($cfg_template_file,$current_release, $old_release, $mask, $mask_old_release,$name)=@_;
	my $cfg_file="$current_release/learn$name.cfg";
	open (F, $cfg_template_file);
	open (OUT,">$cfg_file");
	while ( my $line = <F>)
	{
		
		$line=~s/$mask/$current_release/;
		$line=~s/$mask_old_release/$old_release/;
		print OUT $line;
	}
	close(OUT);	
	close(F);
	return ($cfg_file);

}


=head2 procedure LoadRnaObject

 Title        : LoadRnaObject
 Prerequisite : $rna_id exist and correspond to a rna of learn
 Function     : Load object from rnaml, create cfg file for Learn.
                If clic >= 0 do not create tree directory of LeARN.
                Check if xml data are in new release and load from this data (if modification have been done  via cgi but not save in user workspace)
                If a list of family is defined load from this list and all rna of families
                Else Load from rna.
 Args         : $rna_id            : id of rna to load
                $cfg_template      : cfg template file
                $new_release       : path of new release (where data will be save) usualy sessionspace
                $old_release       : path of old release usualy workspace
                $mask              : mask of the current release in cfg template file
                $mask_old_release  : mask of the old release in cfg template file
                $clic              : number of clic on button in cfg (use to know if tree dir exist)
                @family_id         : list of family to load
 
=cut

sub LoadRnaObject
{

	my ($rna_id,$cfg_template, $new_release ,$old_release ,  $mask, $mask_old_release, $clic , $name, @family_id) =(@_);
	print "<br> LoadRnaObject PARAMETERS : $rna_id,$cfg_template, $new_release ,$old_release ,  $mask, $mask_old_release, $clic , $family_id<br>";
	my $cfgFile=&MakeCfg($cfg_template,$new_release, $old_release, $mask, $mask_old_release, $name );
	print "CFG File : $cfgFile<br>";

	my $rh_param = New ParamParser('GETOPTLONG');
	$rh_param->Set('cfg',$cfgFile);
	$rh_param->AssertFileExists('cfg');
	$rh_param->Update($rh_param->Get('cfg'),'A');
	$rh_param->SetUnlessDefined('LOGGER_LEVEL_EXEC',$LOGGER_LEVEL_EXEC);
	#$rh_param->Set('path_old_release',$old_release);
	#$rh_param->Set('path_new_release',$new_release);
	#$rh_param->Set('log_file',"$new_release/Learn.log");
	#$rh_param->Set('species',"Undef");
	# cre le logger
	my $logger = New Logger ($rh_param->Get('log'),$LOGGER_LEVEL_EXEC,'NOCOMPRESSION');
	$logger->Trace("Begin Learn :\n",$logger->CalcStamp,"DE");
	my $cmd;
	# cre un learn
	my $application= New Learn($rh_param, \$logger);
	 #print "CLIC : $clic<br>";
	if ($clic>=0) # si on a cliqu sur un button de la page (c'est que l'on a dja crer l'arborescence
	{
		$rh_param->Set('createtree',0);
#		$old_release=$new_release;
	}
	else
	{
# 		$cmd="rm -rf ".$application->{'__current_release'}."/".$rh_param->Get('path_db');
# 		system $cmd;
		$rh_param->Set('createtree',1) ;
	}
	
	$rh_param->Set('db_opt',"none");
	$application->Initialize($rh_param->Get('createtree'), $rh_param->Get('db_opt')); # crée l'arborescence
	
	$application->CreatePrograms(); # blastn / rfam_scan / trnaScan-SE / rnafold / mfold

	if (defined (@family_id) && $#family_id > -1 )
	{
		#my @fam=($family_id);
		$application->LoadReleaseFromListOfFamily( $application->{'__current_release'}."/".$rh_param->Get('path_db_learn_rna_tmp')."/".$rna_id."/".$rna_id.$Var::E_XML, @family_id);


	}
	else
	{
		my $dir_xml_rna_old=$application->{'__old_release'}."/".$rh_param->Get('path_db_learn_rna_xml');
		my $dir_xml_rna_current=$application->{'__current_release'}."/".$rh_param->Get('path_db_learn_rna_xml');
		my $dir_tmp_rna=$application->{'__current_release'}."/".$rh_param->Get('path_db_learn_rna_tmp');
		
		
		
		if (! -d "$dir_tmp_rna/$rna_id")
		{
			mkdir ("$dir_tmp_rna/$rna_id");
		}
		# copie du fichier xml dans un repertoire isol pour applique les methode LoadXFormRealease 
		# avec release = au dir contenant un seul fichier !!!
		if ( !-e "$dir_xml_rna_current/$rna_id$Var::E_XML")
		{
			$cmd="cp -a $dir_xml_rna_old/$rna_id$Var::E_XML $dir_tmp_rna/$rna_id/";
			system $cmd;
		}
		else
		{
			$cmd="cp -a $dir_xml_rna_current/$rna_id$Var::E_XML $dir_tmp_rna/$rna_id/";
			system $cmd;
		}
		$application->CreateProgramsFromRelease("$dir_tmp_rna/$rna_id");
		$application->CreateSequencesFromRelease("$dir_tmp_rna/$rna_id");
		$application->CreateRnasFromRelease("$dir_tmp_rna/$rna_id", $application->{'__current_release'}."/".$rh_param->Get('path_db_learn_rna'));

	}
	
	
	return ($application);
}


=head2 procedure DiplayResult

 Title        : DiplayResult
 Prerequisite : none
 Function     : From an xhtml file , print each line
 Args         : $result : xhtml file
 				$numbering : boolean to indicate to numbering each line
=cut

sub DisplayResult
{
	my ($result, $numbering)=@_;
	open (F, $result);
	my $i = 1;
	my $to_print;
	while ( my $line = <F>)
	{
		#$contenu_fichier =~ s/<\?xml version="1\.0"\?>/ / ;
		if (defined $numbering && $line=~/^\s*<tr/)
		{
			$to_print= "$line<td>$i</td>";
			$i++;
		}
		else
		{
			$to_print=$line;
		}
		print "$to_print";
	}
	
	close(F);
}


1;
