#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: January 12, 2006
#

package Overlap;

=pod

=head1 NAME

Overlap - It's an object which contain overlap information on sequences (BACS)

=head1 SYNOPSIS


	
	
=head1 DESCRIPTION


=cut

use strict;
use warnings;

use Range;
use Sequence;

BEGIN
{
    our $VERSION = "0.1";
}


=head2 function New

 Title        : New
 Usage        : New Overlap($file);
 Prerequisite : none
 Function     : Create an object overlap
 Returns      : An object
 Args         : $file : tabular file wich contain overlap informations
				eg: seq1	start1		end1	strand1		seq2	 start2 	end2 	strand2
 Globals      : none

=cut 

sub New
{
    my ($class, $file) = @_;

    my $self = {
                __file  	=> $file,
		__h_match	=> undef, #reference d'objet range qui pointe sur un tableau de reference de range.
		__seq_query	=> "",  # seq d'analyse
		__r_range_query => '@', # range (d'arn) en cour d'analyse
		__a_seq_query_ranges =>undef, #tableau des ranges overlappant de la sequence query en cour d'analyse
		__h_result	=> undef #hash des ranges ovelped on other seq :  string squence -> 1 range
                };
    bless($self, $class);
    if ( -e  $file )
    {
    $self->Init();
    }
	return ($self);

}


=head2 procedure Init

 Title        : Init
 Usage        : $overlop->Init();
 Prerequisite : attribut __file is set
 Function     : initialize hash of match
 Returns      : none
 Args         : none
 Globals      : none

=cut 

sub Init
{
    my ($self) = @_;
	open (OVER , $self->{'__file'}) or die "Impossible douvrir: >".$self->{'__file'}."<\n";
	
	while (my $line=<OVER>)
	{
		print "$line\n";
		my @a_tmp=split(' ', $line); # AC15	15662	17523	-	AC16	552		2413	+
		next if ($#a_tmp < 7);
		print "new range: $a_tmp[1],$a_tmp[2],$a_tmp[3]\n";
		my $id_match = $a_tmp[0]."_".$a_tmp[4];
		${$self->{'__h_match'}}{$id_match}= [(	New Range()	 ,	New Range()	)] ;
		${$self->{'__h_match'}}{$id_match}[0]->Initialize($a_tmp[1],$a_tmp[2],$a_tmp[3]);
		${$self->{'__h_match'}}{$id_match}[1]->Initialize($a_tmp[5],$a_tmp[6],$a_tmp[7]);

	}
	close OVER;
	$self->Print();

}

sub Print
{
	my ($self) = (@_);
	print "Print\n";
	foreach my $seq ( keys %{$self->{'__h_match'}} )
	{
		print "Match sequences :  $seq \n";
		${$self->{'__h_match'}}{$seq}[0]->LightPrint();
		${$self->{'__h_match'}}{$seq}[1]->LightPrint();
	}
}

=head2 fuction ComputeRanges

 Title        : ComputeRanges
 Usage        : $seq->ComputeRanges($seq_id);
 Prerequisite : none
 Function     : Fill the array of ranges with range which overlap $seq_id
 Returns      : Number of overlaped range
 Args         : $range : an Range object
 Globals      : none

=cut 

sub ComputeRanges
{
	my ($self, $seq_id) = (@_);	
	$self->{'__seq_query'}=$seq_id;
	my $index=-1;
	@{$self->{'__a_seq_query_ranges'} }=();
	foreach my $key (sort keys(%{$$self{'__h_match'}}))
    	{
		if ($key =~ /$seq_id/)
		{
			my @a_tmp=split ("_", $key);
			if ($a_tmp[0] =~ /$seq_id/ )
			{
				$index=0;
			}
			elsif ($a_tmp[1] =~ /$seq_id/ )
			{
				$index=1;
			}
			push (@{$self->{'__a_seq_query_ranges'} }, \${$self->{'__h_match'}}{$key}[$index]);
		}
    	}

	return $#{$self->{'__a_seq_query_ranges'} };
}


=head2 fuction SeqIsOverlaped

 Title        : SeqIsOverlaped
 Usage        : $seq->SeqIsOverlaped($seq_id);
 Prerequisite : none
 Function     : Return if seq_id is overlapped $seq_id
 Returns      : Number of overlaped range
 Args         : $range : an Range object
 Globals      : none

=cut 

sub ComputeSeqOverlaped
{
	my ($self, $seq_id) = (@_);
	
	$self->{'__seq_query'}=$seq_id;
	@{$self->{'__a_seq_query_ranges'}}=();
	foreach my $key (sort keys(%{$$self{'__h_match'}}))
	{
		if ($key =~ /$seq_id/)
		{
			push (@{$self->{'__a_seq_query_ranges'}},$key);
		}
	}
	return $#{$self->{'__a_seq_query_ranges'}};
}


=head2 fuction SeqIsOverlaped

 Title        : SeqIsOverlaped
 Usage        : $seq->SeqIsOverlaped($seq_id);
 Prerequisite : none
 Function     : Return if seq_id is overlapped $seq_id
 Returns      : Number of overlaped range
 Args         : $seq_id : an id of sequence
 Globals      : none

=cut 
sub GetOverlaped
{
	my ($self, $seq_id) = (@_);
	my @res;
	my $index=-1;
	return (@$self->{'__a_seq_query_ranges'});
}

=head2 fuction ComputeRangeOnOverlaped

 Title        : ComputeRangeOnOverlaped
 Usage        : $seq->ComputeRangeOnOverlaped($seq_id, $r_range);
 Prerequisite : nom des sequence non inclus les uns dans les autres
 Function     : Je crois que ce n'est pas tres utile.
 Returns      : Number of overlaped range
 Args         : $seq_id : an id of sequence
 Globals      : none

=cut 

sub ComputeRangeOnOverlaped
{
	my ($self,$seq_id, $r_range) = (@_);
	my $res=0;
	my $seq;
	my $index=-1;
	my $other_seq=-1;
	$self->{'__r_range_query'}=$r_range;
	$self->{'__h_result'}=();
	if ($seq_id ne $self->{'__seq_query'}) 
	{
		$self->ComputeSeqOverlaped($seq_id);
	}
	
	foreach my $key (@{$self->{'__a_seq_query_ranges'}})
	{
		$index=-1;
		$other_seq=-1;
		
		if ($key =~ /$seq_id/)
		{
           		my @a_tmp=split ("_", $key);
			if ($a_tmp[0] =~ /$seq_id/ )
			{
				$seq=$a_tmp[0];
				$index=0;
				$other_seq=1;
			}
			elsif ($a_tmp[1] =~ /$seq_id/ )
			{
				$seq=$a_tmp[1];
				$index=1;
				$other_seq=0;
			}
			next if ($index < 0 );
			
			if( ($$r_range->GetStart() >= ${$self->{'__h_match'}}{$key}[$index]->GetStart()  and $$r_range->GetStart() <= ${$self->{'__h_match'}}{$key}[$index]->GetEnd() ) or
		    ( $$r_range->GetEnd()   >= ${$self->{'__h_match'}}{$key}[$index]->GetStart() and $$r_range->GetEnd() <=  ${$self->{'__h_match'}}{$key}[$index]->GetEnd() ) or
		    ( $$r_range->GetStart() <=  ${$self->{'__h_match'}}{$key}[$index]->GetStart() and ${$self->{'__h_match'}}{$key}[$index]->GetEnd()   >= $$r_range->GetEnd() ) )
			{
				print "init h_result: $a_tmp[$other_seq]\n";
				${$self->{'__h_result'}}{$a_tmp[$other_seq]}=\${$self->{'__h_match'}}{$key}[$other_seq];
				$res++;
			}
	    	}
    	}
	return ($res);
}


sub GetHashQuery
{
	my ($self)=(@_);
	return (%{$self->{'__h_result'}});
}

sub GetHashResult
{
	my ($self)=(@_);
	return (%{$self->{'__h_result'}});
}
sub PrintQuery
{
	my ($self,$seq_id, $r_range) = (@_);
	foreach my $key (sort keys(%{$$self{'__h_result'}}))
    {
		print "Seq : $key\n";
       ${${$self->{'__h_result'}}{$key}}->LightPrint();
    }
}


1;

