#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: October 21, 2005
#

package Range;

=pod

=head1 NAME

Sequence- It's an object Range

=head1 SYNOPSIS

	my $prog = New Range();	
	
example: 
	use Range;
	my $my_range = New Range();

=head1 DESCRIPTION


=cut

use strict;
use warnings;
use Bio::Seq;
BEGIN
{
    our $VERSION = "0.2";
}

=head2 function New

 Title        : New
 Usage        : New Range;
 Prerequisite : none
 Function     : Create an object Range
 Returns      : An object Range
 Args         : none
 Globals      : none

=cut 

sub New
{
    my ($class) = @_;

    my $self = {
		__seq_data		=> "",
                __start			=> 0,
                __end			=> 0,
                __strand		=> "",
		__r_sequence	=> undef,
		__length		=> 0,
		__r_prog_detect	=> undef,
		__date_detect	=> ""
                };

    bless($self, $class);
	return ($self);

}


=head2 procedure Initialize

 Title        : Initialize
 Usage        : $seq->Initialize( $start, $end, $strand, $r_sequence,$r_prog_detect,$date_detect);
 Prerequisite : none
 Function     : Initialize the Range object
 Returns      : none
 Args         : $start
				$end		
				$strand
				$r_sequence :  reference on the sequence on which range is
				$r_prog_detect : reference on the program which detected the range
 Globals      : none

=cut 

sub Initialize
{
	my ($self, $start, $end, $strand, $r_sequence, $r_prog_detect, $date_detect) = @_;
	#print "$start, $end, $strand, $r_sequence, $r_prog_detect, $date_detect\n";
	$self->SetSeqData($$r_sequence->GetSubSequence($start, $end, $strand)) if (defined ($r_sequence));
	$self->SetStart($start);
	$self->SetEnd($end);
	$self->SetStrand($strand);
	$self->SetSequence($r_sequence) if (defined ($r_sequence));
	$self->ComputeLength() ;
	$self->SetProgDetect($r_prog_detect) if (defined ($r_prog_detect));
	$self->SetDateDetect($date_detect) if (defined ($date_detect));

}
=head2 procedure InitAll

 Title        : InitAll
 Usage        : $seq->InitAll($seq_data, $start, $end, $strand, $length, $r_sequence,$r_prog_detect,$date_detect);
 Prerequisite : none
 Function     : Initialize the All field Range object : don t compute data !!!
 Returns      : none
 Args         : $seq_data
				$start
				$end		
				$strand
				$length
				$r_sequence :  reference on the sequence on which range is
				$r_prog_detect : reference on the program which detected the range
 Globals      : none

=cut 

sub InitAll
{				
	my ($self, $start, $end, $strand,$r_sequence,$r_prog_detect, $date_detect, $seq_data ,  $length) = @_;
	$self->SetSeqData( $seq_data);
	$self->SetStart($start);
	$self->SetEnd($end);
	$self->SetStrand($strand);
	$self->SetSequence($r_sequence);
	$self->SetLength($length);
	$self->SetProgDetect($r_prog_detect);
	$self->SetDateDetect($date_detect) if (defined ($date_detect));
}



=head2 fuction Print

 Title        : Print
 Usage        : $seq->Print;
 Prerequisite : none
 Function     : Display the range
 Returns      : none
 Args         : none
 Globals      : none

=cut 

sub Print
{
	my ($self) = @_;
	print "-->Range :\n";
	print "\tseq_data : ".$self->GetSeqData()."\n";
	print "\tstart : ".$self->GetStart()."\n";
	print "\tEnd : ".$self->GetEnd()."\n";	
	print "\tStrand : ".$self->GetStrand()."\n";
	print "\tProgDetect : ".${$self->GetProgDetect()}->GetName()."\n" if (defined($self->GetProgDetect()));
	my $seq =$self->GetSequence();
	print "\tDateDetect : ".$self->GetDateDetect()."\n";
	#$$seq->Print() if ( ref($$seq));
}





=head2 fuction LightPrint

 Title        : LightPrint
 Usage        : $seq->LightPrint;
 Prerequisite : none
 Function     : Display only start end and strand
 Returns      : none
 Args         : none
 Globals      : none

=cut 
sub LightPrint
{
my ($self) = @_;
	print "\tstart : ".$self->GetStart()."\n";
	print "\tEnd : ".$self->GetEnd()."\n";	
	print "\tStrand : ".$self->GetStrand()."\n";

}
=head2 fuction IsSame

 Title        : IsSame
 Usage        : $seq->IsSame($range);
 Prerequisite : none
 Function     : Compare to range
 Returns      : 0 if not same
				1 if same on different sequence
				2 if redondancy on same sequence (overlap) => on jete !
 Args         : $range : an Range object
 Globals      : none

=cut 

sub IsSame
{
	my ($self, $range) = @_;
	my $find=0;
	if ($self->GetSequence() eq $range->GetSequence()) 
	{ 
		if ($self->GetStrand() eq $range->GetStrand()) 
		{ 
			# overlap !
			my $start = $self->GetStart() ;
			my $end = $self->GetEnd() ;
			my $startToCmp=$range->GetStart();
			my $endToCmp = $range->GetEnd();
	 		if( ( $start >= $startToCmp and $start <= $endToCmp ) or
			    ( $end   >= $startToCmp and $end  <=  $endToCmp ) or
			    ( $start <= $startToCmp and $end   >= $endToCmp ) )
				{$find=1;} #Sequence overlap.
			# A logger !!
		}
	}
	return ($find);
}



=head2 fuction GetGffFeature

 Title        : GetGffFeature
 Usage        : $seq->GetGffFeature($range);
 Function     : Compare to range
 Returns      : GffFeature
 Args         : $name : name of the feature.
 Globals      : none

=cut 

sub GetGffFeature
{
	my ($self, $name) = @_;
	# <seqname> <source> <feature> <start> <end> <score> <strand> <frame> [attributes] [comments]
	my $line = "$name\tLeARN\tncRNA\t".$self->{'__start'}."\t".$self->{'__end'}."\t.\t".$self->GetStrand()."\t.";
	return ($line);
}
#########GETTEUR ET ACCESSEUR ########"

=item GetLength

	Get length of range
=cut 

sub GetLength
{
	my ($self) = @_;
	return ($self->{'__length'});
}

=item SetLength

	Set length of range
	
=cut 

sub SetLength
{
	my ($self, $length) = @_;
	$self->{'__length'}=$length;
}

=item ComputeLength

	Compute length of range with start and end
	
=cut 

sub ComputeLength
{
	my ($self) = @_;
	$self->{'__length'}=(abs($self->{'__start'}-$self->{'__end'})+1);
}

=item GetSeqData

	Return sequence data
	
=cut 

sub GetSeqData
{
	my ($self) = @_;
	return $self->{'__seq_data'};
}

=item SetSeqData

	Set sequence data
	
=cut 

sub SetSeqData
{
	my ($self, $seq_data) = @_;
	#$seq_data=~ tr/atgcT/AUGCU/;
	$self->{'__seq_data'}=$seq_data;
	
}

=item ComputeSeqData

	Compute sequence data with sequence , start and end
	
=cut 

sub ComputeSeqData
{
	my ($self) = @_;
	$self->SetSeqData(${$self->GetSequence()}->GetSubSequence($self->{'__start'},$self->{'__end'},$self->{'__strand'}));
}


=item GetStart

	Get start
	
=cut 


sub GetStart
{
	my ($self) = @_;

	return $self->{'__start'};
}

=item SetStart

	Set start
	
=cut 


sub SetStart
{
	my ($self, $start) = @_;

	$self->{'__start'}=$start;
}

=item GetEnd

	Get end
	
=cut 

sub GetEnd
{
	my ($self) = @_;

	return $self->{'__end'};
}

=item SetEnd

	Set end
	
=cut 

sub SetEnd
{
	my ($self, $end) = @_;

	$self->{'__end'}=$end;
}


=item GetStrand

	Get strand
	
=cut 

sub GetStrand
{
	my ($self) = @_;

	return $self->{'__strand'};
}

=item SetStrand

	Set strand
	
=cut 

sub SetStrand
{
	my ($self, $strand) = @_;

	$self->{'__strand'}=$strand;
}

=item GetSequence

	Get sequence
	
=cut 

sub GetSequence
{
	my ($self) = @_;

	return $self->{'__r_sequence'};
}

=item SetSequence

	Set sequence
	
=cut 

sub SetSequence
{
	my ($self, $r_sequence) = @_;

	$self->{'__r_sequence'}=$r_sequence;
}


=item GetProgDetect

	Get reference on program (used to detect rna range)
	
=cut 

sub GetProgDetect
{
	my ($self) = @_;
	return $self->{'__r_prog_detect'};
}

=item SetProgDetect

	Set reference on program (used to detect rna range)
	
=cut 

sub SetProgDetect
{
	my ($self, $prog) = @_;
	$self->{'__r_prog_detect'}=$prog;
}

=item GetDateDetect

	Get date of detection
	
=cut 

sub GetDateDetect
{
	my ($self) = @_;
	return $self->{'__date_detect'};
}

=item SetDateDetect

	Set date of detection
	
=cut 

sub SetDateDetect
{
	my ($self, $date) = @_;
	$self->{'__date_detect'}=$date;
}

1;

