#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
# 
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: October 24, 2005
#

#TODO : Stockholm a faire propre : ajouter description, nom ...
# ATTENTION : si on veux utiliser avec un autre contexte : SetContextDir !
# mais il faut aussi changer le path puisque l'image est stocke en fonction du path (niveau de la db LeARN/family ...)
# LIGNE 384 : contien le nom est programme en fixe !!!!
# C'est a l'utilisateur de l'objet de verifier que les donnes sont cohrentes : avec GetComputed!

package RnaFamily;

=pod

=head1 NAME

Rna- It's an object RnaFamily

=head1 SYNOPSIS

	my $prog = New RnaFamily();	
	
example: 
	use RnaFamily;
	my $my_family = New RnaFamily();

=head1 DESCRIPTION


=cut

use strict;
use warnings;
use Cwd;

use Var;
#use vars qw($Var::MAX_LG $Var::DIR_IMG $Var::DIR_TMP $Var::CMD_REFORMAT $Var::REFORMAT_OUT_STO $Var::REFORMAT_OUT_CLU $Var::REFORMAT_OUT_A2M $Var::E_STO $Var::E_ALN $Var::E_MUL $Var::E_FA $Var::E_CM $Var::E_MSS $Var::E_IMG );

BEGIN
{
    our $VERSION = "0.1";

}

#Pour stockholm :
my $ID_SEQ_CONS = "RF_cons";
my $ID_SEQ_SS = "RF_ss";
# nomenclature stockholm : 
my $STO_CONS = "#=GC SS_cons";
my $STO_SS = "#=GC RF";

my $RATE_DECREASE_FREE_ENERGY=0.8;

=head2 function New

 Title        : New
 Usage        : New RnaFamily;
 Prerequisite : none
 Function     : Create an object RnaFamily
 Returns      : An object
 Args         : none
 Globals      : none

=cut 

sub New
{
    my ($class) = @_;

    my $self = {
                id    			=> "",
                __name 			=> "",
                __desc  		=> "",
                __last_upd		=> "",
		__length		=> 0, # compute from structure
		__consensus		=> "", # compute from structure				
                __db_xref		=> "",				
		__stucture		=> New SecondaryStructure(), # compute
		__ra_rna		=> [], # index from 0 .. n
		__status		=> "",
		__path			=> "", # utilis pour stocker l'image db/learn/family/img et le ss dans db/learn/family/tmp/*.alifold
		__computed		=> 0, # if Compute is done and no modification in rna data was made =1
		__rfl_dir		=> "",
		__clustal_file		=> "",
		__stockholm_file	=> "",
		__fa_file		=> "",
		__mul_file		=> "",
		__cm_file		=> "",
		__mss_file		=> "",
		__r_prog_align 		=>'@',
		__r_prog_build  	=>'@',
		__sondage 		=>{},
				
             };
	bless($self, $class);
	$$Learn::R_LOGGER->Trace("RnaFamily\tNew\t\n",$Learn::LOGGER_STAMP,"DE");
	
    return ($self);
}


=head2 procedure Initialize

 Title        : Initialize
 Usage        : $MyRnaFamily->Initialize();
 Prerequisite : none
 Function     : Initialize the RnaFamily object
 Returns      : none
 Args         : $id: id of family
 				$name: Name of family
				$desc: description of family
				$last_upd: last date of update
				$db_xref: references of other database
				$status: status of the family (validate or in study)
				$path : directory of image
				$rfl_dir : directory to store intermediaire file : clustal, stockolm, cm ...
				$r_prog_struct: reference on a program to compute the secondary structure
				$r_prog_align : reference on a program to align all rna
				$r_prog_build : reference on a program to build covariance model
				$r_prog_search: reference on a program to known if a rna match with  the cm
 Globals      : none

=cut 

sub Initialize
{
    my ($self,$id, $name, $desc, $last_upd, $db_xref, $status, $path, $rfl_dir, $r_prog_struct, $r_prog_align,$r_prog_build) = @_;
	$$Learn::R_LOGGER->Trace("RnaFamily\tInitialize\t$id, $name, $desc, $last_upd, $db_xref, $status, $path, $rfl_dir\n",$Learn::LOGGER_STAMP,"DE");
	$self->SetId($id);
	$name=$id if (not defined($name) || $name =="");
	$self->SetName($name);
	$self->SetDesc($desc);
	$self->SetLastUpd($last_upd);
	$self->SetDbxref($db_xref);
	$self->SetStatus($status);
	$self->SetPath($path);
	$self->SetProgAlign($r_prog_align);	
	$self->SetProgBuild($r_prog_build);
	$self->{'__stucture'}->SetProgram($r_prog_struct);
	$self->{'__stucture'}->SetImage("$path/$Var::DIR_IMG/$id$Var::E_IMG");
	#rna family legumineuse : rfl
	$self->SetContextDir($rfl_dir);

}

=head2 procedure Print

 Title        : Print
 Usage        : $family->Print();
 Prerequisite : none
 Function     : Print a family
 Args         : none
 Globals      : none

=cut 

sub Print
{
	my ($self)=@_;
	print "\n\nRNAFamily :\n";
	print "id : ".$self->GetId()."\n";
	print "Name : ".$self->GetName()."\n";
	print "Desc : ".$self->GetDesc()."\n";
	print "LastUpd : ".$self->GetLastUpd()."\n";
	print "Longueur : ".$self->{'__length'} ."\n";
	print "Consensus : ".$self->{'__consensus'} ."\n";
	print "db_xref : ".$self->{'__db_xref'} ."\n";
	$self->{'__stucture'}->Print();
	print "Path : ".$self->GetPath()."\n";
	 
	print 	"Computed : ".$self->{'__computed'} ."\n";
	print 	"rfl_dir : ".$self->{'__rfl_dir'} ."\n";
	print 	"clustal_file : ".$self->{'__clustal_file'} ."\n";
	print 	"stockholm_file : ".$self->{'__stockholm_file'} ."\n";
	print 	"fa_file : ".$self->{'__fa_file'} ."\n";
	print 	"mul_file : ".$self->{'__mul_file'} ."\n";
	print 	"cm_file : ".$self->{'__cm_file'} ."\n";
	print 	"prog_align : ".${$self->{'__r_prog_align'}}->GetName() ."\n" if (defined($self->{'__r_prog_align'}));
	
#	print 	"prog_build : ".${$self->{'__r_prog_build'}}->GetName() ."\n" if (defined($self->{'__r_prog_build'}));
}
 


=head2 procedure SaveRnaml

 Title        : SaveRnaml
 Usage        : $family->SaveRnaml($xml_dir,$tmp_dir);
 Prerequisite : none
 Function     : Write in directory $xml_dir a Rnaml file
 Args         : $xml_dir : directory where rnaml file will be create
				$tmp_dir : tempory directory
 Globals      : none

=cut 

sub SaveRnaml
{
	my ($self,$xml_dir,$tmp_dir)=@_;

	if ($self->GetComputed() eq "No")
	{$self->ComputeStructure();}
	$$Learn::R_LOGGER->Trace("RnaFamily\tSaveRnaml\t".$self->GetId()."\n",$Learn::LOGGER_STAMP,"DE");
	UtilRnaml::SaveFamily(\$self,$xml_dir,$tmp_dir, \${$Learn::R_LOGGER}, $Learn::LOGGER_STAMP );
}

=head2 procedure LoadFromRnaml

 Title        : LoadFromRnaml
 Usage        : $family->LoadFromRnaml($file);
 Prerequisite : none
 Function     : Initialize a family with rnaml file
 Args         : $file : rnaml file
				$rh_prog : reference on a hash which contain programs
 Globals      : none

=cut 
sub LoadFromRnaml
{
	my ($self,$xml_file, $rfl_dir, $dir_family, $r_prog_build, $r_prog_search) = (@_);
	&UtilRnaml::LoadFamily(\$self,$xml_file );
	#cp des fichiers des familles, inutile, ms maj des path
	$self->SetContextDir($rfl_dir);
	$self->SetComputed('No');
	$self->SetPath($dir_family);
	my $old_path_img = $self->{'__stucture'}->GetImage();
	if (defined ($old_path_img) && -e $old_path_img)
	{
		#ex : /www/LeARN/data/rel_0.1/db/learn/rna/img/AC148607_r1.jpg
		my @a_path=split('/',$old_path_img);
		$self->{'__stucture'}->SetImage("$dir_family/$Var::DIR_IMG/".$a_path[$#a_path]);
		my $cmd="cp -a $old_path_img ".$self->{'__stucture'}->GetImage();
		system $cmd;
	}
	$self->SetProgBuild($r_prog_build);
}




=head2 procedure DumpFasta

 Title        : DumpFasta
 Usage        : $family->DumpFasta();
 Prerequisite : none
 Function     : Write fasta file (if family contain only one sequence, it s duplicated)
 Args         : none
 Globals      : none

=cut 

sub DumpFasta
{	
	my ($self, $uniq)=@_;

	my $cmd;
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpFasta\t".$self->GetId()."\n",$Learn::LOGGER_STAMP,"DE");

	if (not -d $self->GetRflDir()."/".$self->GetId())
	{
		$cmd="mkdir -p ".$self->GetRflDir()."/".$self->GetId();
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpFasta\tCMD:".$cmd."\n",$Learn::LOGGER_STAMP,"D");
		system $cmd;
	}

	my ($seq, $head);
	# concatene les sequences au format fasta
	open SEQ , ">".$self->{'__fa_file'} or die "Can't write >".$self->{'__fa_file'}."\n";
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpFasta\t".$self->{'__fa_file'}."\n",$Learn::LOGGER_STAMP,"D");
	for (my $i=0; $i<=$#{$self->{'__ra_rna'}}; $i++)
	{	
		$head=${$self->{'__ra_rna'}[$i]}->GetId();
		$seq=${$self->{'__ra_rna'}[$i]}->GetSequence();
		print SEQ ">$head\n";
		print SEQ "$seq\n";
	}

	if ($#{$self->{'__ra_rna'}} == 0 && (!defined($uniq) || !$uniq)) # si on a une seule squence dans la famille on la duplique pour pouvoir construire un modele
	{
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpFasta\tFamily with only one sequence> Duplicate in fasta\n",$Learn::LOGGER_STAMP,"DE");
		print SEQ ">$head#2\n";
		print SEQ "$seq\n";
	}

	close SEQ;


}


=head2 procedure DumpClustal

 Title        : DumpClustal
 Usage        : $family->DumpClustal();
 Prerequisite : fasta file exists
 Function     : Write clstal file, and create also mul file.
 Args         : none
 Globals      : none

=cut 
sub DumpClustal
{	
	my ($self, $file) = @_;
	my ($seq, $head);
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\t".$self->GetId()."\n",$Learn::LOGGER_STAMP,"DE");

	#test des prrequis
	if (! defined($self->{'__r_prog_align'}) )
	{
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tERROR: DumpClustal > Program to align not defined !\n",$Learn::LOGGER_STAMP,"DE");
		die ("ERROR: DumpClustal > Program to align not defined !\n");
	}
	if (${$self->{'__r_prog_align'}}->IsVirtual())
	{
		$self->{'__r_prog_align'}=&Learn::GetRefProgram(${$self->{'__r_prog_align'}}->GetType(), "actual");
	}
	$self->DumpFasta(0);

#print STDERR "c1\n";
	# test des repertoires et cration !
	my $tmp_dir=$self->GetRflDir()."/".$self->GetId()."/$Var::DIR_TMP";

	if (not -d $tmp_dir)
	{	
		my $cde="mkdir -p  $tmp_dir \n ";
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tcmd: $cde\n",$Learn::LOGGER_STAMP,"D");
		# cette commande fait un shell init err
		system $cde;
	}
#print STDERR "c2\n";
	chdir($tmp_dir);

	# fichier temporaire : fasta / aln multiple / clustal
	my $mul_tmp_file=$tmp_dir."/".$self->GetId().$Var::E_MUL;
	my $clu_tmp_file=$tmp_dir."/".$self->GetId().$Var::E_ALN;
	my $fa_tmp_file=$tmp_dir."/".$self->GetId().$Var::E_FA;

	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tTempory files: $mul_tmp_file, $clu_tmp_file, $fa_tmp_file\n",$Learn::LOGGER_STAMP,"D");

	# travaille sur un fichier fasta temporaire
	my $cmd= "cp -a ".$self->{'__fa_file'}." $fa_tmp_file";
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tcmd: $cmd\n",$Learn::LOGGER_STAMP,"D");
	system $cmd;
	$$Learn::R_LOGGER->Trace("##$fa_tmp_file## copi\n",$Learn::LOGGER_STAMP,"D") if (-e $fa_tmp_file);
	$$Learn::R_LOGGER->Trace("##$fa_tmp_file## non copi\n",$Learn::LOGGER_STAMP,"D") if (! -e $fa_tmp_file);
	
	# Alignement multiple : sur les fichiers temporaires
	
	if (${$self->{'__r_prog_align'}}->GetName() =~ /clustal/)
	{
		${$self->{'__r_prog_align'}}->SetInOut($fa_tmp_file, $self->{'__clustal_file'}) ;
		${$self->{'__r_prog_align'}}->Run();
		$cmd="$Var::CMD_REFORMAT $Var::REFORMAT_OUT_A2M ".$self->{'__clustal_file'}." > ".$self->{'__mul_file'} ;
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tcmd: $cmd\n",$Learn::LOGGER_STAMP,"D");
		system $cmd;	
	}
	elsif (${$self->{'__r_prog_align'}}->GetName() eq "multalin")
	{
		${$self->{'__r_prog_align'}}->SetInOut($fa_tmp_file,  $mul_tmp_file) ;
		${$self->{'__r_prog_align'}}->Run();
	
		#Dplace d'alignement multiple hors du repertoire de tmp
		$cmd= "cp -a $mul_tmp_file " .$self->{'__mul_file'}."\n";
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tCMD: $cmd\n",$Learn::LOGGER_STAMP,"D");
		system $cmd;

		# conversion en clustal
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tConvertToClustal: ".$self->{'__clustal_file'}."\n",$Learn::LOGGER_STAMP,"D");
		${$self->{'__r_prog_align'}}->ConvertToClustal($self->{'__clustal_file'});
	}
	$cmd = "cd ".$self->GetRflDir()."\n";
#5	system $cmd;
 
	chdir($self->GetRflDir());
	
	$cmd = "rm -rf $tmp_dir";
	system $cmd;

	if (defined($file))
	{
		$cmd="cp -a ".$self->{'__clustal_file'}." $file";
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpClustal\tCMD: $cmd\n",$Learn::LOGGER_STAMP,"D");
		system $cmd;
	}
}


=head2 procedure DumpStockholm

 Title        : DumpStockholm
 Usage        : $family->DumpStockholm();
 Prerequisite : mul file exists
 Function     : Write Stockholm file.
 Args         : none
 Globals      : none

=cut 

sub DumpStockholm
{
	my ($self, $file) = @_;
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\n",$Learn::LOGGER_STAMP,"DE");

	if ( ( ! -e $self->{'__clustal_file'}) || ($self->GetComputed() eq "No")) 
	{
		$self->DumpClustal();
		#Ajout du consensus et de la ss	si calcul sinon on calcule !
	}

	if ($self->GetComputed() eq "No")
	{$self->ComputeStructure();}

	# test des repertoires et cration !
	my $tmp_dir=$self->GetRflDir()."/".$self->GetId()."/$Var::DIR_TMP";
	if (not -d $tmp_dir)
	{system "mkdir -p $tmp_dir";}

	my $currdir = cwd();
	chdir($tmp_dir);
	
	#recupre le mul ! (sortie de Multalin)
	my $tmp_file=$tmp_dir."/".$self->GetId().$Var::E_MUL;
	my $cmd="$Var::CMD_REFORMAT $Var::REFORMAT_OUT_A2M ".$self->{'__clustal_file'}." > $tmp_file";
	system $cmd;

	# Ajout a l'alignment consensus et alignement a l'alignment multiple
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\tAjout consensus to: $cmd\n",$Learn::LOGGER_STAMP,"D");
	open(MUL,">>".$tmp_file) or die "DumpStockholm >". $tmp_file."<\n";

	my $struct=$self->GetStructure()->GetStructure();
	$struct =~ s/\)/\>/g;
	$struct =~ s/\(/\</g;

	print MUL ">$ID_SEQ_CONS\n";
	print MUL  $self->GetConsensus()."\n";
	print MUL ">$ID_SEQ_SS\n";
	print MUL  $struct."\n";
	close (MUL);

	
	$cmd="$Var::CMD_REFORMAT $Var::REFORMAT_OUT_STO $tmp_file > ".$self->{'__stockholm_file'}."\n";
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\tCMD: $cmd\n",$Learn::LOGGER_STAMP,"D");
	system $cmd;

	
	# reformate le fichier stockholm dans fichier : 
	# A commenter !!!

	my $sto_tmp_file=$tmp_dir."/".$self->GetId().$Var::E_STO;
	open(STO_IN,$self->{'__stockholm_file'}) or die "$0/PutConsensus >". $self->{'__stockholm_file'}."<\n";
	open(STO_OUT,">$sto_tmp_file") or die "$0/PutConsensus >".$self->{'__stockholm_file'}.".tmp"."<\n";
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\tReformate stockholm from :".$self->{'__stockholm_file'}." to $sto_tmp_file \n",$Learn::LOGGER_STAMP,"D");
	while (my $line = <STO_IN>)
	{	
		my @a_F=split(' ',$line);
		if (defined($a_F[0]) && $a_F[0] =~/^RF_/) 
		{
			$a_F[0] =~ s/$ID_SEQ_SS/$STO_SS/;
			$a_F[0] =~ s/$ID_SEQ_CONS/$STO_CONS/;
			$line = sprintf("%-30s%s\n",$a_F[0],$a_F[1]);
		}	
		if ($#a_F == 1 && not($line=~/^\#/)) # si on a split en 2 car si suprrieur il faut garder toute l'information
		{
			$line = sprintf("%-30s%s\n",$a_F[0],$a_F[1]);
		}
		print STO_OUT $line;
	}
	close (STO_OUT);
	close (STO_IN);

	$cmd= " cp -a $sto_tmp_file " .  $self->{'__stockholm_file'}."\n";
		
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\tCMD: $cmd\n",$Learn::LOGGER_STAMP,"D");
	system $cmd;
	if (defined($file))
	{
		$cmd="cp -a ".$self->{'__stockholm_file'}." $file";
		$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\tCMD: $cmd\n",$Learn::LOGGER_STAMP,"D");

		system $cmd;
	}

	$cmd = "cd ".$self->GetRflDir()."\n";
#	system $cmd;
	
	chdir($currdir); 

	$cmd = "rm -rf $tmp_dir";
	system $cmd;

}


=head2 procedure DumpMss

 Title        : DumpMss
 Usage        : $family->DumpMss();
 Prerequisite : stockolm file exists
 Function     : Write Mss file.
 Args         : none
 Globals      : none

=cut 

sub DumpMss
{
	my ($self, $file) = @_;
	$$Learn::R_LOGGER->Trace("RnaFamily\tDumpStockholm\n",$Learn::LOGGER_STAMP,"DE");

	if ( ( ! -e $self->{'__stockholm_file'}) || ($self->GetComputed() eq "No")) 
	{
		$self->DumpStockholm();
		#Ajout du consensus et de la ss	si calcul sinon on calcule !
	}

	open (STO, $self->{'__stockholm_file'});
	my %h_data;
	my $cons="";
	while (my $line=<STO>)
	{
		my @a_T= split (" ", $line);
		next if ($#a_T < 1);
		if ($line =~/$STO_CONS/)
		{
			$cons.=$a_T[2];
		}
		elsif ($line!~/\#/ )
		{
			$h_data{$a_T[0]}.=$a_T[1];
		}
	}
	system "rm -f ".$self->{'__mss_file'} if (-e $self->{'__mss_file'});


	$cons=~s/>(>+)>/&Rep($1,'>')/ge;
	$cons=~s/<(<+)</&Rep($1,'<')/ge;
	$self->CutLine($cons, "ST\n");

	# moins de 256 caracteres !
	foreach my $key (keys(%h_data))
	{
		$self->CutLine($h_data{$key}, "AS $key\t".length($h_data{$key})."\n");
	}
	if (defined ($file) )
	{
		system "mv ".$self->{'__mss_file'} . " $file";
	}

}

=head2 function CutLine

 Title        : CutLine
 Usage        : $obj->CutLine($line, $header)
 Prerequisite : none
 Function     : with a line cut it in line of 50 char and print hearder following by lines.
 Returns      : none
 Args         : line
				header
 Globals      : none

=cut 

sub CutLine
{
	my ($self,$line, $head)=@_;
	my $i=0;
	open (MSS, ">>".$self->{'__mss_file'});
	print MSS $head;
	while ($i <=length($line))
	{
		print MSS $i+1 if (($i % 50) == 0 || $i==0);
		print MSS "\t".substr($line,$i, 50);
		$i+=50;
		print MSS "\n";
	}
	print MSS "\n";
	close MSS;
}

=head2 function Rep

 Title        : Rep
 Usage        : Rep($str,$char)
 Function     : repeat $char as long as length of $str
 Returns      : a string
 Args         : $str : string 
 		$char:  caracters ( < | > | . )

=cut 

sub Rep
{
	my ($str,$char)= @_;
	my $new = '-' x length($1);
	return "$char$new$char";
}

=head2 procedure BuildCm

 Title        : BuildCm
 Usage        : $family->BuildCm();
 Prerequisite : stockholm file exists
 Function     : Create a covariance model
 Args         : $rfam_dir : directory of rfam (in case we have only one sequence wich correspond to a rfam model.
 		$file : to define if you want to save in a specific path/file
 Globals      : none

=cut 

sub BuildCm
{
	my ($self,$rfam_dir, $file) = (@_);
	$$Learn::R_LOGGER->Trace("RnaFamily\tBuildCm\t".$self->GetId()."\n",$Learn::LOGGER_STAMP,"DE");
	#recupre le mul ! (sortie de Multalin)
	my $tmp_dir=$self->GetRflDir()."/".$self->GetId()."/$Var::DIR_TMP";
	system "mkdir -p $tmp_dir";
	my $currdir = cwd();
	chdir($tmp_dir);	
	my $copy=0;


	if ( (! -e $self->{'__stockholm_file'}) || ($self->GetComputed() eq "No") ) 
	{
		$self->DumpStockholm();
	}

	if (${$self->{'__r_prog_build'}}->IsVirtual())
	{
		$self->{'__r_prog_build'}=&Learn::GetRefProgram(${$self->{'__r_prog_build'}}->GetType(), "actual");
	}

	${$self->{'__r_prog_build'}}->SetInOut($self->{'__stockholm_file'},  $self->{'__cm_file'}) ;
	${$self->{'__r_prog_build'}}->Run();

 	if ($#{$self->{'__ra_rna'}} ==0 && defined ($rfam_dir) ) # si on a une seule squence dans la famille on copie le modle Rfam s'il family dtecter par Rfam
	{
		my $rna_dbxref=${${$self->{'__ra_rna'}}[0]}->GetDbxref();
		if ( $rna_dbxref =~ /Rfam/ )
		{	
			my @tmp= split ("_", $rna_dbxref); # ex : 	Rfam_RF00029
			if (-d $rfam_dir)
			{
				my $cmd="cp $rfam_dir/".$tmp[1].".cm ". $self->{'__cm_file'};
				#print $cmd."\n";
				$$Learn::R_LOGGER->Trace("RnaFamily\tBuildCm\t $cmd\n",	$Learn::LOGGER_STAMP,"DE");
				system $cmd;
				$copy=1;
			}
		}
	}
	my $cmd = "cd ".$self->GetRflDir()."\n";
#	system $cmd;
	
	chdir($currdir);
	$cmd = "rm -rf $tmp_dir";
	system $cmd;
	if (defined($file))
	{
		$cmd="cp -a ".$self->{'__cm_file'}." $file";
		$$Learn::R_LOGGER->Trace("RnaFamily\tBuildCm\tCMD: $cmd\n",$Learn::LOGGER_STAMP,"D");
		system $cmd;
	}
	
}



=head2 function ImproveSS

 Title        : ImproveSS
 Usage        : $family->ImproveSS();
 Prerequisite : BuildCm done, and ComputeStructure also
 Function     : Get from clustal
 Globals      : none

=cut 



sub ImproveSS
{
	my ($self ) = @_ ;
	$$Learn::R_LOGGER->Trace("RnaFamily\tImproveSS\t".$self->GetId()."\n",$Learn::LOGGER_STAMP,"DE");
	#return() if ( $self->NumberOfRna() <= 1);
	return() if ( $self->GetLength()>$Var::MAX_LG);

	my @cons = split ( '', $self->GetConsensus() );
	my @struc = split( '' , $self->GetStructure()->GetStructure());
	my $tmp_dir_rfl=$self->GetRflDir()."/".$self->GetId()."/$Var::DIR_TMP";
	my %h_aln;
	if (not -d $tmp_dir_rfl)
	{system "mkdir -p $tmp_dir_rfl";}
	my $currdir = cwd();
	chdir($tmp_dir_rfl);
	my $tmp_dir_learn=$self->GetPath()."/$Var::DIR_TMP"."/".$self->GetId();
	if (not -d $tmp_dir_learn)
	{system "mkdir -p $tmp_dir_learn";}
	
	#recupre le mul ! (sortie de Multalin)
	my $tmp_file_mul=$tmp_dir_rfl."/".$self->GetId().$Var::E_MUL;
	my $tmp_file_clu=$tmp_dir_rfl."/".$self->GetId().$Var::E_ALN;
	my $cmd="$Var::CMD_REFORMAT $Var::REFORMAT_OUT_A2M ".$self->{'__clustal_file'}." > $tmp_file_mul";
	#print "CMD:	$cmd";
	system $cmd;
	
	my $in = Bio::SeqIO->new( -file => $tmp_file_mul, '-format' => 'fasta'  );
	my $seq = Bio::Seq->new();

	while( $seq = $in->next_seq())
	{	
		push ( @{$h_aln{$seq->id()}},split ('',$seq->seq()) );
	}
	$in->DESTROY();

	for (my $i=$#cons; $i >=0; $i--)
	{
		if ($cons[$i] eq '_')
		{
			foreach my $key( keys  (%h_aln) )
			{
				$h_aln{$key}[$i]='';
			}
		}
	}

	open SEQ , ">$tmp_file_mul" or die "Can't write >$tmp_file_mul<\n";
	#print "open >$tmp_file_mul<\n";
	$$Learn::R_LOGGER->Trace("RnaFamily\tImproveSS\t".$tmp_file_mul."\n",$Learn::LOGGER_STAMP,"D");
	foreach my $key( keys  (%h_aln) )
	{
		my ($seq)=join('',@{$h_aln{$key}});
		print SEQ ">$key\n";
		print SEQ "$seq\n";
	}
	my $save_file=$self->{'__clustal_file'};

	$cmd="$Var::CMD_REFORMAT $Var::REFORMAT_OUT_CLU $tmp_file_mul  | perl -ne \'{split;if ( \$#_==1 ) { \$_=sprintf(\"%-30s%s\\n\",\$_[0],\$_[1]); }; print; }\' > $tmp_file_clu " ;

	#print "CMD:	$cmd";
	system $cmd;
	
	$self->{'__clustal_file'}=$tmp_file_clu;
	$self->SetComputed("Yes") ;
	$self->ComputeStructure() ;
	$self->{'__clustal_file'}=$save_file;
	my @new_struc = split( '' , $self->GetStructure()->GetStructure());
	my $i_new=0;	
	$self->SetConsensus(join('',@cons));
	my @computed_struc;
	my $i_comp=0;

	for (my $i=0; $i <= $#cons ; $i++)
	{
		if ($cons[$i] eq '_')
		{
			$computed_struc[$i_comp]='.';
		}
		else
		{
			$computed_struc[$i_comp]=$new_struc[$i_new];
			$i_new++;
		}
		$i_comp++;
	
	}
	$self->GetStructure()->SetStructure(join('',@computed_struc));
	chdir($currdir);
}



=head2 procedure SetContextDir

 Title        : SetContextDir
 Usage        : $family->SetContextDir($dir);
 Prerequisite : none
 Function     : Change all path of temporary file ( as mul, clustal, fa...);
 Args         : none
 Globals      : none

=cut
sub SetContextDir # il faut auqssi changer le pass s'il lon veux pouvoir acceder a l'image!
{
	my ($self, $rfl_dir) = @_;
	my $id=$self->GetId();
	$$Learn::R_LOGGER->Trace("RnaFamily\tSetContextDir\t$id \tDir:$rfl_dir\n",$Learn::LOGGER_STAMP,"DE");
	if ( not -d $rfl_dir)
	{
		system "mkdir -p $rfl_dir";
	}
	$self->{'__rfl_dir'}=$rfl_dir;
	$self->{'__mul_file'}="$rfl_dir/$id/$id$Var::E_MUL";
	$self->{'__clustal_file'}="$rfl_dir/$id/$id$Var::E_ALN";
	$self->{'__stockholm_file'}="$rfl_dir/$id/$id$Var::E_STO";
	$self->{'__fa_file'}="$rfl_dir/$id/$id$Var::E_FA";
	$self->{'__cm_file'}="$rfl_dir/$id/$id$Var::E_CM";
	$self->{'__mss_file'}="$rfl_dir/$id/$id$Var::E_MSS";

	if ( not -d "$rfl_dir/$id")
	{
		system "mkdir -p $rfl_dir/$id";
	}

}




=head2 function AnalyseMerging

 Title        : AnalyseMerging
 Usage        : $family->AnalyseMerging($r_prog, $format);
 Prerequisite : none
 Function     : Launch program ($r_prog rfam or blast) with all member of a family against rfl db (all member of all families) and identify the best match with an other family
 Return	      : the best family id.
 Args         : $r_prog = program of analyse
		$format = format of the program
 Globals      : none

=cut 

sub AnalyseMerging
{
	my ($self, $r_prog, $format ) = @_ ;

	$self->DumpFasta(1);

	my $res_file=$self->{'__fa_file'}.".am";
	$$r_prog->SetInOut($self->{'__fa_file'},$res_file);
	$$r_prog->Run();	
	# si nouvelle famille
	
	my %h_res;

	open (F, $res_file) or die "impossible d'ouvrir  >$res_file< \n";
	my $line;
	my $current_fam=$self->GetId();
	my ($id_fam);
	while ($line = <F>)
	{	
		chomp ($line);
		my @a_line=split (/[\s]+/, $line) ;
		if ($format eq "rfam" )
		{
			#ex : AC093544_r41    Rfam    domain  1       115     .       +       .       ACC=RFL00022; ID=RFL00022; SCORE=134.21;
			next if ($#a_line < 8 ) ; 
			($id_fam)= $a_line[8] =~ /ACC=(RFL\d*)/ ;

		}
		elsif ($format eq "blast" )
		{
			#ex : AC146705_r1     AC149305.19/95092-95163;RFL00003;RFL00003;      100.00  11      0       0       14      24      14      24      0.48    22.3
			next if ($#a_line < 3 ) ; 
	
			my @a_RF=split (';', $a_line[1]);#AC149305.19/95092-95163;RFL00003;RFL00003;
			next if ($#a_RF < 1);
			$id_fam=$a_RF[1];

		}
		#print STDERR "SEQ: $a_line[0] ; FAM : $current_fam; Match :  $id_fam\n";
		if ( ($id_fam =~ /RFL/) && ($id_fam !~ /$current_fam/) && ($line !~ /\#2/) )
		{		
			$h_res{$id_fam}++;
		}

	}
	close (F);

	my $best_fam="";
	my $best_score=0;
	foreach my $key (keys ( %h_res))
	{
		if ( ($h_res{$key} > $best_score) && ( $key lt  $current_fam ) )
		{
			$best_fam=$key ;
			$best_score=$h_res{$key};
		}
		$$Learn::R_LOGGER->Trace("EXT: $current_fam  match $key :  $h_res{$key}\n",$Learn::LOGGER_STAMP,"D");
	}

	return $best_fam ;
}



=head2 function ClearTmpDir

 Title        : ClearTmpDir
 Usage        : $family->ClearTmpDir();
 Prerequisite : none
 Function     : delete the tempory dir of compute of family
 Return		  : none
 Args         : none
 Globals      : none

=cut 

sub ClearTmpDir
{
	my ($self)=@_;
	my $tmp_dir = $self->GetPath()."/$Var::DIR_TMP/".$self->GetId();
	my $tmp_rfl_dir = $self->GetRflDir()."/".$self->GetId()."/$Var::DIR_TMP";
	system "rm -rf $tmp_dir $tmp_rfl_dir/*";
}




=head2 function MergeTwoFamilies

 Title        : MergeTwoFamilies
 Usage        : $family->MergeTwoFamilies($r_fam1, $r_fam2);
 Prerequisite : none
 Function     : Merge two families (rna members) and compute free energy
 Return		  : 1 if free energy is < 0
				0 else
 Args         : $r_fam1 = ref on family
				$r_fam2 = ref on family
 Globals      : none

=cut 

sub MergeTwoFamilies
{
	my ($self, $r_fam1, $r_fam2 )= (@_);
	my $res = 0;
	foreach my $ref ($r_fam1, $r_fam2  )
	{
		foreach my $r_rna ( @{$$ref->GetRaRna()})
		{
			$self->AddRna($r_rna);
		}
	}
	
	$self->ComputeStructure();
	my $self_free_en=$self->GetStructure()->GetFreeEnergy();
	my $fam1_free_en=$$r_fam1->GetStructure()->GetFreeEnergy();
	my $fam2_free_en=$$r_fam2->GetStructure()->GetFreeEnergy();
	$$Learn::R_LOGGER->Trace("EXT: FREE EN virtual : $self_free_en ; fam 1 : $fam1_free_en ; fam 2 $fam2_free_en\n",$Learn::LOGGER_STAMP,"D");
	my ($max) = ( $fam1_free_en>$fam2_free_en) ? $fam1_free_en : $fam2_free_en;
	$max=$max*$RATE_DECREASE_FREE_ENERGY;
	if ( $self_free_en <= $max)# &&  $self_free_en < $fam1_free_en && $self_free_en < $fam2_free_en)
	{
		$res = 1;
	}
	return $res;
}



=head2 function MergeWithFamily

 Title        : MergeWithFamily
 Usage        : $family->MergeWithFamily($r_fam);
 Prerequisite : none
 Function     : Merge family (in parameter) with self.
 Return		  : none
 Args         : none
 Globals      : none

=cut 

sub MergeWithFamily
{
	my ($self, $r_fam, $rfam_dir) =@_;
	print "MergeWithFamily\n";
	foreach my $r_rna  (@{$$r_fam->GetRaRna()})
	{
		$self->AddRna($r_rna);
		$$r_rna->SetFamilyAcc($self->GetId());
	}
	$self->ComputeStructure();
	$self->DumpStockholm();
	$self->BuildCm($rfam_dir);
}



#########ACCESSEUR ########


=head2 function GetId

 Title        : GetId
 Usage        : $family->GetId();
 Function     : Accessor
 Returns      : Id

=cut 

sub GetId
{
	my ($self) = @_;
	return $self->{'id'};
}

=head2 procedure SetId

 Title        : SetId
 Usage        : $family->SetId($id);
 Function     : Initialize attribute id with parameter

=cut 

sub SetId
{
	my ($self, $id) = @_;

	$self->{'id'}=$id;
}

=head2 function GetName

 Title        : GetName
 Usage        : $family->GetName();
 Function     : Accessor
 Returns      : name attribute

=cut 

sub GetName
{
	my ($self) = @_;

	return $self->{'__name'};
}

=head2 procedure SetName

 Title        : SetName
 Usage        : $family->SetName($name);
 Function     : Initialize attribute name with parameter

=cut 

sub SetName
{
	my ($self, $name) = @_;

	$self->{'__name'}=$name;
}

=head2 function GetLength

 Title        : GetLength
 Usage        : $family->GetLength();
 Function     : Accessor
 Returns      : length attribute

=cut 

sub GetLength
{
	my ($self) = @_;
	return $self->{'__length'};
}

=head2 procedure SetLength

 Title        : SetLength
 Usage        : $family->SetLength($length);
 Function     : Initialize attribute length with parameter

=cut

sub SetLength
{
	my ($self, $length) = @_;
	$self->{'__length'}=$length;
}

=head2 function GetDesc

 Title        : GetDesc
 Usage        : $family->GetDesc();
 Function     : Accessor
 Returns      : description attribute

=cut 

sub GetDesc
{
	my ($self) = @_;

	return $self->{'__desc'};
}

=head2 procedure SetDesc

 Title        : SetDesc
 Usage        : $family->SetDesc($desc);
 Function     : Initialize attribute description with parameter

=cut

sub SetDesc
{
	my ($self, $desc) = @_;

	$self->{'__desc'}=$desc;
}

=head2 function GetLastUpd

 Title        : GetLastUpd
 Usage        : $family->GetLastUpd();
 Function     : Accessor
 Returns      : last update  attribute

=cut 

sub GetLastUpd
{
	my ($self) = @_;

	return $self->{'__last_udp'};
}

=head2 procedure SetLastUpd

 Title        : SetLastUpd
 Usage        : $family->SetLastUpd($last_udp);
 Function     : Initialize attribute last_update with parameter

=cut

sub SetLastUpd
{
	my ($self, $last_udp) = @_;

	$self->{'__last_udp'}=$last_udp;
}

=head2 function GetStatus

 Title        : GetStatus
 Usage        : $family->GetStatus();
 Function     : Accessor
 Returns      : status  attribute

=cut 

sub GetStatus
{
	my ($self) = @_;

	return $self->{'__status'};
}

=head2 procedure SetStatus

 Title        : SetStatus
 Usage        : $family->SetStatus($status);
 Function     : Initialize attribute status with parameter

=cut

sub SetStatus
{
	my ($self, $status) = @_;

	$self->{'__status'}=$status;
}

=head2 function GetDbxref

 Title        : GetDbxref
 Usage        : $family->GetDbxref();
 Function     : Accessor
 Returns      : dbxref  attribute

=cut 

sub GetDbxref
{
	my ($self) = @_;

	return $self->{'__db_xref'};
}

=head2 procedure SetDbxref

 Title        : SetDbxref
 Usage        : $family->SetDbxref($db_xref);
 Function     : Initialize attribute dbxref with parameter
 Attr         : $db_xref : dbxref value
                $compute : flag to indicates if dbxref have to be computed from sondage

=cut

sub SetDbxref
{
	my ($self, $db_xref, $compute) = @_;
	
	if ((not defined($db_xref) || $compute) ) # on travaille sur le sondage
	{	
		if ( $self->GetStatus() !~ /Annotated|experimetal/i )
		{
		my $max1=0;
		my $dbx1="";
		my $name_rna1="";
		my $max2=0;
		my $dbx2="";
		my $name_rna2="";
		foreach my $key ( keys %{$self->{'__sondage'}} ) 
		{
			if (${$self->{'__sondage'}}{$key}[0] > $max1)
			{
				$max1=${$self->{'__sondage'}}{$key}[0];
				$dbx1=$key;
				$name_rna1=${$self->{'__sondage'}}{$key}[1];
			}
			elsif (${$self->{'__sondage'}}{$key} > $max2)
			{
				$max2=${$self->{'__sondage'}}{$key}[0];
				$dbx2=$key;
				$name_rna2=${$self->{'__sondage'}}{$key}[1];
			}
		}
			$db_xref=$dbx1;	
			$self->SetName($name_rna1);
		}
	}
	$self->{'__db_xref'}=$db_xref;	
}

=head2 function GetStructure

 Title        : GetStructure
 Usage        : $family->GetStructure();
 Function     : Accessor
 Returns      : object structure 

=cut 

sub GetStructure
{
	my ($self) = @_;
	return $self->{'__stucture'};
}

=head2 procedure SetStructure

 Title        : SetStructure
 Usage        : $family->SetStructure($struct);
 Function     : Initialize attribute structure with parameter
 Attr         : $struct : object structure

=cut

sub SetStructure
{
	my ($self, $struct) = @_;

	$self->{'__stucture'}=$struct;
}

=head2 procedure ComputeStructure

 Title        : ComputeStructure
 Usage        : $family->ComputeStructure();
 Function     : Dump clustal file is not existing
                Get the actual program if program for computing is virtual
                Compute the structure
 
=cut

sub ComputeStructure
{
	my ($self) = @_;
	# Le fichier d"alignment et le repertoire temporaire pour crer les fichiers
	$$Learn::R_LOGGER->Trace("RnaFamily\tComputeStructure\t".$self->GetId()."\n",$Learn::LOGGER_STAMP,"DE");
	if ( (!( -e $self->{'__clustal_file'})) || ($self->GetComputed() eq "No"))
	{
		$self->DumpClustal();
	}

	if (${$self->GetProgStructure()}->IsVirtual())
	{
		$self->SetProgStructure( &Learn::GetRefProgram( ${$self->GetProgStructure()}->GetType(), "actual") );
	}

	my ($len, $cons) = $self->{'__stucture'}->Compute($self->{'__clustal_file'}, $self->GetPath(), $self->GetId() );
	$self->{'__length'}=$len;
	$self->SetConsensus($cons);

	$self->SetDbxref() if ( $self->GetStatus() !~ /Annotated|experimetal/i );
	$self->SetComputed('Yes');
}

=head2 procedure ComputePlotEval

 Title        : ComputePlotEval
 Prerequis    : Length of consensus and the structure have to be the same
 Usage        : $family->ComputePlotEval();
 Prerequis    : the parenthsis structure of structure object and consensus must be set
 Function     : Plot and eval free energy according to the parenthesis structure
 
=cut

sub ComputePlotEval
{
	my ($self,$consensus)=(@_);
	my $tmp_dir=$self->GetPath()."/tmp/".$self->GetId();
	mkdir $tmp_dir;
	my $tmp_file="$tmp_dir/".$self->GetId().".tmp";
	$consensus =~ s/[-_]/N/g;
	open ( SEQ , ">$tmp_file") or die "unable to create >$tmp_file<\n";
	print SEQ ">".$self->GetId()."\n";
	print SEQ $consensus."\n";
	print SEQ $self->GetStructure()->GetStructure()."\n";
	close (SEQ);
	
	my $r_prog_plot_eval=&Learn::GetRefProgram('R_PLOT_EVAL', "actual");
	$self->{'__stucture'}->SetProgram($r_prog_plot_eval);
	my ($len, $cons) = $self->{'__stucture'}->Compute($tmp_file, $self->GetPath(), $self->GetId() );
	
}

=head2 procedure GetConsensus

 Title        : GetConsensus
 Usage        : $family->GetConsensus();
 Function     : accessor
 Return       : consensus of the alignement.
 
=cut

sub GetConsensus
{
	my ($self) = @_;
	return $self->{'__consensus'};
}

=head2 procedure SetConsensus

 Title        : SetConsensus
 Prerequis    : Length of consensus have to be the same as the alignemnt
 Usage        : $family->GetConsensus($cons);
 Function     : Set the consensus of the alignement.
 Attr         : $cons string of the consensus
 
=cut

sub SetConsensus
{
	my ($self, $cons) = @_;
	$self->{'__consensus'}=$cons;
}

=head2 procedure HaveRna

 Title        : HaveRna
 Usage        : $family->HaveRna($id_rna);
 Function     : find if family have the rna as a member 
 Att          : $id_rna : id of an rna
 Return       : 0 : if id_rna is not a member of the family
                1 : if id_rna is a member of the family
 
=cut

sub HaveRna
{
	my ($self, $id_rna) = (@_);
	my $res=0;
	foreach my $ro_rna ( @{$self->{'__ra_rna'}} )
	{
		if ( $$ro_rna->GetId() eq  $id_rna )
		{
			$res=1;
			last;
		}
	}
	return $res;
}

=head2 procedure GetRna

 Title        : GetRna
 Usage        : $family->GetRna($index);
 Prerequis    : $index must be lower than the number of rna in the family
 Function     : return an rna wich is at the $index of the rna array of the family
 Att          : $index : numeric index
 Return       : reference on an rna
 
=cut


sub GetRna
{
	my ($self, $index) = @_;
	return $self->{'__ra_rna'}[$index];
}

=head2 procedure AddRna

 Title        : AddRna
 Usage        : $family->AddRna($r_rna);
 Prerequis    : rna must already no be in the family
 Function     : Add the rna to the family, complete the sondage hash to compute the dbxref
 Att          : $r_rna : reference on an array
 Return       : none
 
=cut

sub AddRna
{
	my ($self, $r_rna) = @_;
	push (@{$self->{'__ra_rna'}}, $r_rna);

	#Ajout du dbxref de l'Arn dans celui de la famille
	my $dbxref_rna=$$r_rna->GetDbxref();
	my $key=$dbxref_rna;
	my $name=$$r_rna->GetName();
	if ($dbxref_rna =~ /tRNA/)
	{
		$key = "tRNA";
		$name= "tRNA";
	}
		
	if (${$self->{'__sondage'}}{$key})
	{
		${$self->{'__sondage'}}{$key}[0]++;
	}
	elsif ( $name !~ /RFL|Learn/ )
	{
		${$self->{'__sondage'}}{$key}=([1,$name]);
	}
	$self->SetComputed('No');
}

=head2 procedure DeleteRna

 Title        : DeleteRna
 Usage        : $family->DeleteRna($index);
 Prerequis    : $index must be lower than the number of rna in the family
 Function     : Delete the rna at the index $index in the array of rna of the family
 Att          : $index : numeric index
 Return       : none
 
=cut

sub DeleteRna
{
	my ($self, $index) = @_;
	splice (@{$self->{'__ra_rna'}}, $index,1);
	$self->SetComputed('No');
}


=head2 procedure DeleteRna

 Title        : DeleteRna
 Usage        : $family->DeleteRna($id_rna);
 Prerequis    : $id_rna must be in the family
 Function     : Delete the rna with the id $id_rna in the family
 Att          : $id_rna : id of an rna
 Return       : none
 
=cut

sub DeleteRnaFromId
{
	my ($self, $id_rna) = @_;
	my $index=$self->IndexRnaId($id_rna);
	if ($index >-1)
	{
		${$self->{'__ra_rna'}[$index]}->SetFamilyAcc('');
		splice (@{$self->{'__ra_rna'}},$index ,1);
		$self->SetComputed('No');
	}
}

=head2 procedure IndexRnaId

 Title        : IndexRnaId
 Usage        : $family->IndexRnaId($id_rna);
 Prerequis    : $id_rna must be in the family
 Function     : return the index of the rna $id_rna in the array of rna
 Att          : $id_rna : id of an rna
 Return       : index of the rna
 
=cut

sub IndexRnaId
{
	my ($self, $id_rna) = (@_);
	my $res=-1;
	foreach my $ro_rna ( @{$self->{'__ra_rna'}} )
	{
		$res++;
		if ( $$ro_rna->GetId() eq  $id_rna )
		{
			last;
		}
		
	}
	return $res;
}


=head2 procedure NumberOfRna

 Title        : NumberOfRna
 Usage        : $family->NumberOfRna();
 Prerequis    : none
 Function     : return the inumber of rna
 Return       : number of rna
 
=cut

sub NumberOfRna
{
	my ($self) = @_;
	return $#{$self->{'__ra_rna'}}+1;
}

=head2 procedure GetRaRna

 Title        : GetRaRna
 Usage        : $family->GetRaRna();
 Function     : accessor on the array of rna's reference
 Return       : array of rna
 
=cut

sub GetRaRna
{
	my ($self) = @_;
	return $self->{'__ra_rna'};
}

=head2 procedure SetRaRna

 Title        : SetRaRna
 Usage        : $family->SetRaRna();
 Function     : Set the array of rna's reference
  
=cut

sub SetRaRna
{
	my ($self, $ra_rna) = @_;
	$self->{'__ra_rna'}=$ra_rna;
	$self->SetComputed('No');
}

=head2 procedure GetPath

 Title        : GetPath
 Usage        : $family->GetPath();
 Function     : Get path of the family
 Return       : path
 
=cut

sub GetPath
{
	my ($self) = @_;

	return $self->{'__path'};
}

=head2 procedure SetPath

 Title        : SetPath
 Usage        : $family->SetPath($path);
 Function     : set path of the family
 
=cut

sub SetPath
{
	my ($self, $path) = @_;

	$self->{'__path'}=$path;
}

=head2 procedure GetRflDir

 Title        : GetRflDir
 Usage        : $family->GetRflDir();
 Function     : Get path of rfl dir 
 Return       : path
 
=cut

sub GetRflDir
{
	my ($self) = @_;

	return $self->{'__rfl_dir'};
}

=head2 procedure SetRflDir

 Title        : SetRflDir
 Usage        : $family->SetRflDir($path);
 Function     : set path of rfl dir
 
=cut

sub SetRflDir
{
	my ($self, $path) = @_;

	$self->{'__rfl_dir'}=$path;
}



=head2 procedure GetComputed

 Title        : GetComputed
 Usage        : $family->GetComputed();
 Function     : Get the flag computed ( eq [No|Yes] )
 Return       : attrubute computed
 
=cut

sub GetComputed
{
	my ($self) = @_;

	return $self->{'__computed'};
}


=head2 procedure SetComputed

 Title        : SetComputed
 Usage        : $family->SetComputed($computed);
 Prerequis    : $computed ==[No|Yes]
 Function     : set attribute computed
 
=cut

sub SetComputed
{
	my ($self, $computed) = @_;

	$self->{'__computed'}=$computed;
}

=head2 procedure GetProgStructure

 Title        : GetProgStructure
 Usage        : $family->GetProgStructure();
 Function     : Get the reference on the program used to compute the structure
 Return       : reference on a object program
 
=cut

sub GetProgStructure
{
	my ($self) = @_;
	return $self->{'__stucture'}->GetProgram();
}

=head2 procedure SetProgStructure

 Title        : SetProgStructure
 Usage        : $family->SetProgStructure($prog);
 Function     : set the program used for the structure
 Att          : $prog : reference on a program.
=cut

sub SetProgStructure
{
	my ($self, $prog) = @_;
	$self->{'__stucture'}->SetProgram($prog);
}


=head2 procedure GetProgAlign

 Title        : GetProgAlign
 Usage        : $family->GetProgAlign();
 Function     : Get the reference on the program used to align rna sequences
 Return       : reference on a object program
 
=cut

sub GetProgAlign
{
	my ($self) = @_;
	return $self->{'__r_prog_align'};
}

=head2 procedure SetProgAlign

 Title        : SetProgAlign
 Usage        : $family->SetProgAlign($prog);
 Function     : set the program used for the alignemnt
 Att          : $prog : reference on a program.
=cut

sub SetProgAlign
{
	my ($self, $prog) = @_;
	$self->{'__r_prog_align'}=$prog;
}


=head2 procedure GetProgBuild

 Title        : GetProgBuild
 Usage        : $family->GetProgBuild();
 Function     : Get the reference on the program used to build the covariance model
 Return       : reference on a object program
 
=cut

sub GetProgBuild
{
	my ($self) = @_;
	return $self->{'__r_prog_build'};
}

=head2 procedure SetProgBuild

 Title        : SetProgBuild
 Usage        : $family->SetProgBuild($prog);
 Function     : set the program used to build the covariance model
 Att          : $prog : reference on a program.
 
=cut

sub SetProgBuild
{
	my ($self, $prog) = @_;
	$self->{'__r_prog_build'}=$prog;
}

=head2 procedure GetFile

 Title        : GetFile
 Usage        : $family->GetFile();
 Function     : Get the a file acording to a format
 Return       : path of the file.
 
=cut

sub GetFile
{	my ($self, $name) = @_;
	if ($name eq "multalin")
	{
		return $self->{'__mul_file'};
	}
}
1;

