#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: October 19, 2005
#

package SecondaryStructure;

=pod

=head1 NAME

SecondaryStructure - It's an object which represent the secondary structure of a RNA or a Family

=head1 SYNOPSIS

	my $prog = New SecondaryStructure();	
	
example: 
	use SecondaryStructure;
    my $ss= New SecondaryStructure("MyRna", "TTTTTTACGTAACTTGCGTTTTTT");
	$ss->Print();	

	$ss->Compute("ACGGGGATTTTGC");	

	$ss->Print();	

=head1 DESCRIPTION


=cut

use strict;
use warnings;
use Cwd;



BEGIN
{
    our $VERSION = "0.1";
	
}

use Var;
#use vars qw($Var::E_RNAML $Var::E_MFOLD_CT $Var::E_PS $Var::DIR_TMP $Var::Var::DIR_SS);



=head2 function New

 Title        : New
 Usage        : New SecondaryStructure($sequence, $image, $r_program);
 Prerequisite : none
 Function     : Compute from the sequence the value of all attribute and initialyze it.
 Returns      : none
 Args         : $sequence : sequence data
 				$image : file where image will be create
				$r_program :  program used to compute data (free_energy, secondary structure and image)
 Globals      : none

=cut 

sub New
{
	my ($class, $sequence, $image,$r_program, $date_compute) = @_;
	
    my $self = {
                __structure     => "",
                __free_energy 	=> "",
                __image 		=> "",
		__r_program   	=> '@',
		__ss_file		=>"", # only use by rna
		__rnaml_file	=>"", # normaly generated by mfold
		__date_compute	=>"",
		__r_program_convert=>'@'	
                };

    	bless($self, $class);
	if (defined $image) {$self->SetImage($image);}
	if (defined $r_program) {$self->SetProgram($r_program);}
	if (defined $sequence) {$self->Compute($sequence);}
	if (defined $date_compute) {$self->SetDateCompute($date_compute);}
    	return ($self);
}

=head2 function Initialize

 Title        : Initialize
 Usage        : Initialize($structure, $free_energy, $image );
 Prerequisite : none
 Function     : Create an object SecondaryStructure
 Returns      : An object
 Args         : $structure : string with the structure in parenthesis format
  				$free_energy : value of free energy
 				$image : path of the image
 Globals      : none

=cut 

sub Initialize
{
   	my ($self, $structure, $free_energy, $r_program) = @_;
	$self->SetStructure($structure);
	$self->SetFreeEnergy($free_energy);
	if (defined $r_program) { $self->SetProgram($r_program); }
}

=head2 function Compute

 Title        : Compute
 Usage        : Compute($sequence);
 Prerequisite : none
 Function     : Compute from the sequence the value of all attribute.
 Returns      : none
 Args         : $sequence : sequence data if it's for a Rna or file alignment if it's for a family
 Globals      : none

=cut 

sub Compute 
{	

	# Si on apelle compute avec le porgramme RNAalifold, $sequence contient le fichier d'entre
	# sinon on a simplement la sequence

	my ($self, $sequence_file, $db_dir, $molecule_id) = @_;
	my ($len, $cons, $struc, $free_en);
	my $res=0;

	my $currdir = cwd();
	my $tmp_dir="$db_dir/$Var::DIR_TMP/$molecule_id";
	my $ss_dir="$db_dir/$Var::DIR_SS";
	my (@a_t,$f_tmp, $mv_ss_file);
	my $r_prog= $self->GetProgram();
	# on supprime les ancien fichier de structure (pour le CGI lors du recalcul d'un arn ne pas garder les anciens fichiers)
	if (0)
	{
		if (-e $self->{'__rnaml_file'} )
		{
			my $cmd = "rm ". $self->{'__rnaml_file'};
			system $cmd
		}
		
		if (-e $self->{'__ss_file'} )
		{
			my $cmd = "rm ". $self->{'__ss_file'};
			system $cmd
		}
	}
	# intialisation date de calcul de la structure
	# Get the a ll the values for current time
    my ($sec, $min, $hour, $mday, $mon, $year) = gmtime();
   	$year = (1900 + $year);    # We'll have to modify this before 2100
    $mon++;
   	$self->{'__date_compute'} = sprintf("%04i%02i%02i", $year, $mon, $mday);

	# il faut gerer les resultat si exec en meme temp => conflic des fichiers .ps
	if (not -d $tmp_dir)
	{
		system "mkdir -p $tmp_dir ";
	}
	else
	{
			
	}
	chdir("$tmp_dir");

	# delace le fichier car la plus part des prg cree de nombreux fichiers tmp dans le meme repertoire que le fichier d'entree 

	system "cp $sequence_file $tmp_dir/$molecule_id";
	if ( (not (defined ($self->{'__rnaml_file'}))) || ($self->{'__rnaml_file'} eq ""))
	{	
		$self->{'__rnaml_file'}="$ss_dir/$molecule_id.rnaml";
	}

	if ( (not (defined ($self->{'__ss_file'}))) || ($self->{'__ss_file'} eq ""))
	{	
		$self->{'__ss_file'}="$ss_dir/$molecule_id.ct";
	}

	if (defined $sequence_file)
	{	
		if ($$r_prog->GetName()=~/rnafold|mfold|rnaploteval/)
		{	
			
			if (not (ref($self->GetProgramConvert())) )
			{
				if ($$r_prog->GetName() eq 'mfold')
				{
					$self->SetProgramConvert(Learn::GetRefProgram("R_CONV_CT2B", "actual"))
				}
				else
				{
					$self->SetProgramConvert(Learn::GetRefProgram("R_CONV_B2CT", "actual"))
				}
			}
			if ($$r_prog->GetName() eq 'mfold')
			{	
				system "ln -sf $tmp_dir/$molecule_id $tmp_dir/$molecule_id$Var::E_MFOLD";
				$$r_prog->SetInOut("$tmp_dir/$molecule_id$Var::E_MFOLD","$tmp_dir/$molecule_id.$Var::E_MFOLD$Var::E_TAR");
				
				# comme plusieur fichier en sortie on tar tout pour mettre dans le repository
				$res=$$r_prog->Run("tar");
				$$r_prog->SetFileOut($self->{'__rnaml_file'});

				# deplace le ficher rnaml (et ss) de /tmp ds  /ss
				$mv_ss_file="$tmp_dir/$molecule_id$Var::E_MFOLD_CT"; 
				
				@a_t=`ls -t $tmp_dir/$molecule_id*$Var::E_RNAML 2> /dev/null`;

				if ($#a_t>-1) # si execution ok, on traite les fichier rnaml et .ct
				{
					$f_tmp=$a_t[0];
					chomp($f_tmp);
					my $cmd = "cp $f_tmp ".$self->{'__rnaml_file'};
					#print $cmd."\n" ;
					system $cmd ;
					if (-e $mv_ss_file )
					{
						$cmd = "cp $mv_ss_file ".$self->{'__ss_file'};
						#print $cmd."\n" ;
						system $cmd ;
						# convertion du fichier ct en str parenthes
						my $prog = $self->GetProgramConvert();
						$$prog->SetInOut($mv_ss_file,"$tmp_dir/$molecule_id$Var::E_MFOLD");
						$$prog->Run();
						($struc, $free_en)=$$prog->ParseResult();
						$self->Initialize($struc, $free_en);
					}
				}
				
				
			}
			else
			{	
				$$r_prog->SetInOut("$tmp_dir/$molecule_id","$tmp_dir/$molecule_id$Var::E_FOLD");
				
				$res=$$r_prog->Run();
				
				# recupere les informations du fichier fold et initialise les attributs:
				#print $$r_prog->GetCmd();
				($struc, $free_en)=$$r_prog->ParseResult();
				$self->Initialize($struc, $free_en);
				# convertion du fichier ct en str parenthes
				my $prog = $self->GetProgramConvert();
				if (ref($prog))
				{
					$$prog->SetInOut("$tmp_dir/$molecule_id$Var::E_FOLD", $self->{'__ss_file'});
					$$prog->Run();
				}
				
			}
		}
		elsif ($$r_prog->GetName() eq 'rnaalifold')
		{		
			$$r_prog->SetInOut("$tmp_dir/$molecule_id","$tmp_dir/$molecule_id$Var::E_ALIFOLD"); # fichier de sortie on ajoute l'extention .fold.
			$res=$$r_prog->Run();
			($struc, $free_en,$len, $cons)=$$r_prog->ParseResult();
			$self->Initialize($struc, $free_en);
			
		}
		elsif ($$r_prog->GetName() eq 'rnaplot')
		{
			$$r_prog->SetInOut("$tmp_dir/$molecule_id","");
			$res=$$r_prog->Run();
			$self->SetFreeEnergy(0);
		}
		
		# deplace l'image de /tmp ds /img
		`rm -f $tmp_dir/*.mfold.ps`; # the boxplot, not used in LeARN
		@a_t=`ls -t $tmp_dir/*$Var::E_PS 2> /dev/null`; # recup du dernier nom de fichier .ps
		if ($#a_t >-1)
		{
			$f_tmp=$a_t[0];
			chomp($f_tmp);
			my $cmd = "$Var::CONVERT_CMD $f_tmp ".$self->GetImage();
			#print "CMD CONVERT: ". $cmd."<br>" ;
			system $cmd;
		}
	}
	else # si sequence non speifieon ne fait rien.
	{
		$self->Initialize("","");
		$res=1; # si on est dans ce cas on return 1 eq a une erreur
	}
	chdir($currdir);
	# JER system "rm -rf $tmp_dir";
	# si prg est rnaalifold on retourne les info : longueur et consensus.
	return ($len, $cons) if ($$r_prog->GetName() eq 'rnaalifold');	
	# sinon retourne le resultat de l'execution du prg
	return $res;

}

=head2 procedure Print

 Title        : Print
 Usage        : Print();
 Prerequisite : none
 Function     : pritn attribute
 Returns      : none
 Args         : none
 Globals      : none

=cut 

sub Print
{
	my ($self) = @_;
	print "Struct : ".$self->GetStructure()."\n";
	print "Free : ".$self->GetFreeEnergy()."\n";
	print "image : ".$self->GetImage()."\n";
	print "Date Comput : ".$self->GetDateCompute()."\n";
	print "ss file : ".$self->GetSSFile()."\n";
	print "rnaml file : ".$self->GetRnamlFile()."\n";

}


####### SETTEUR et ACCESSEUR #########


=head2 function GetStructure

 Title        : GetStructure
 Usage        : $struct->GetStructure;
 Prerequisite : none
 Function     : Acces __structure
 Returns      : parenthesis structure


=cut 

sub GetStructure
{
	my ($self) = @_;

	return $self->{'__structure'};
}

=head2 function GetFreeEnergy

 Title        : GetFreeEnergy
 Usage        : $struct->GetFreeEnergy;
 Prerequisite : none
 Function     : Acces __free_energy
 Returns      : free energy
 

=cut 

sub GetFreeEnergy
{
	my ($self) = @_;

	return $self->{'__free_energy'};
}


=head2 function GetImage

 Title        : GetImage
 Usage        : $struct->GetImage;
 Prerequisite : none
 Function     : Acces __image
 Returns      : image path


=cut 

sub GetImage
{
	my ($self) = @_;

	return $self->{'__image'};
}

=head2 procedure SetStructure

 Title        : SetStructure
 Usage        : $struct->SetStructure($structure);
 Prerequisite : none
 Function     : Acces __structure


=cut 

sub SetStructure
{
	my ($self, $structure) = @_;

	$self->{'__structure'}=$structure;
}

=head2 procedure SetFreeEnergy

 Title        : SetFreeEnergy
 Usage        : $struct->SetFreeEnergy($free_energy);
 Prerequisite : none
 Function     : Acces __free_energy

=cut 

sub SetFreeEnergy
{
	my ($self, $free_energy) = @_;

	$self->{'__free_energy'}=$free_energy;
}


=head2 procedure SetImage

 Title        : SetImage
 Usage        : $struct->SetImage($image);
 Prerequisite : none
 Function     : Acces __image

=cut 

sub SetImage
{
	my ($self, $image) = @_;

	$self->{'__image'}=$image;
}


=head2 procedure GetProgram

 Title        : GetProgram
 Usage        : $struct->GetProgram();
 Prerequisite : none
 Function     : Return a reference on an object program

=cut 

sub GetProgram
{
	my ($self) = @_;

	return $self->{'__r_program'};
}

=head2 procedure SetProgram

 Title        : SetProgram
 Usage        : $struct->SetProgram($r_program);
 Prerequisite : none
 Function     : Set the reference of the object program use to compute the structure

=cut 

sub SetProgram
{
	my ($self, $r_program) = @_;

	$self->{'__r_program'}=$r_program;
}

=head2 procedure GetProgram

 Title        : GetProgram
 Usage        : $struct->GetProgram();
 Prerequisite : none
 Function     : Return a reference of object program use to convert parenthesis format <-> ct format

=cut 

sub GetProgramConvert
{
	my ($self) = @_;

	return $self->{'__r_program_convert'};
}

=head2 procedure SetProgramConvert

 Title        : SetProgramConvert
 Usage        : $struct->SetProgramConvert($r_program);
 Prerequisite : none
 Function     : Set the reference of the program use to convert parenthesis format <-> ct format

=cut 

sub SetProgramConvert
{
	my ($self, $r_program) = @_;

	$self->{'__r_program_convert'}=$r_program;
}

=head2 procedure GetDateCompute

 Title        : GetDateCompute
 Usage        : $struct->GetDateCompute();
 Prerequisite : none
 Function     : Return the date of compute

=cut 

sub GetDateCompute
{
	my ($self) = @_;
	return $self->{'__date_compute'};
}

=head2 procedure SetDateCompute

 Title        : SetDateCompute
 Usage        : $struct->SetDateCompute();
 Prerequisite : none
 Function     : Set the date of compute

=cut 

sub SetDateCompute
{
	my ($self, $date) = @_;
	$self->{'__date_compute'}=$date;
}

=head2 procedure GetSSFile

 Title        : GetSSFile
 Usage        : $struct->GetSSFile();
 Prerequisite : none
 Function     : Return the path of ss file ( .ct file )

=cut 

sub GetSSFile
{
	my ($self) = @_;
	return $self->{'__ss_file'};
}

=head2 procedure SetSSFile

 Title        : SetSSFile
 Usage        : $struct->SetSSFile($file);
 Prerequisite : none
 Function     : Set the path of ss file 

=cut 

sub SetSSFile
{
	my ($self, $file) = @_;
	$self->{'__ss_file'}=$file;
}

=head2 procedure SetPathsSS

 Title        : SetPathsSS
 Usage        : $struct->SetPathsSS($path);
 Prerequisite : none
 Function     : Set the path for all ss files .ct and .rnaml (of mfold) 

=cut 

sub SetPathsSS
{
	my ($self, $path) = @_;
	my $ss_file =$self->GetSSFile();
	my $rnaml_file =$self->GetRnamlFile();
	
	my @a_tmp=split ('/',$ss_file);
	# on recupere le nom du dernier fichier
	my $file=$a_tmp[$#a_tmp];
	if (-e $path."/".$file)
	{
		$self->SetSSFile($path."/".$file);
	}
	else
	{
		$self->SetSSFile("");
	}
	
	@a_tmp=split ('/',$rnaml_file);
	# on recupere le nom du dernier fichier
	$file=$a_tmp[$#a_tmp];
	if (-e $path."/".$file)
	{
		$self->SetRnamlFile($path."/".$file);
	}
	else
	{
		$self->SetRnamlFile("");
	}

}

=head2 procedure SetPathImg

 Title        : SetPathImg
 Usage        : $struct->SetPathImg($path);
 Prerequisite : none
 Function     : Set the path for img

=cut 


sub SetPathImg
{
	my ($self, $path) = @_;
	my $img_file =$self->GetImage();
	
	my @a_tmp=split ('/',$img_file);
	# on recupere le nom du dernier fichier
	my $file=$a_tmp[$#a_tmp];
	if (-e $path."/".$file)
	{
		$self->SetImage($path."/".$file);
	}
	else
	{
		$self->SetImage("");
	}
}

=head2 procedure GetRnamlFile

 Title        : GetRnamlFile
 Usage        : $struct->GetRnamlFile();
 Prerequisite : none
 Function     : Return the rnaml file path (of mfold only for the moment.)

=cut 

sub GetRnamlFile
{
	my ($self) = @_;
	return $self->{'__rnaml_file'};
}

=head2 procedure SetRnamlFile

 Title        : SetRnamlFile
 Usage        : $struct->SetRnamlFile($file);
 Prerequisite : none
 Function     : Set the rnaml file path (of mfold only for the moment.)

=cut 

sub SetRnamlFile
{
	my ($self, $file) = @_;
	$self->{'__rnaml_file'}=$file;
}

1;

