#
# $Id$
#

# Copyright INRA

# Celine.Noirot@toulouse.inra.fr
# Thomas.Schiex@toulouse.inra.fr
# Christine.Gaspin@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a computer program whose purpose is to provide a
# web-based system for detecting, clustering and annotating non protein
# coding RNA genes.

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.
 
#
# 	Celine.Noirot@toulouse.inra.fr
#	Last updated: 7 Nov, 2005
#

package UtilRnaml;

=pod

=head1 NAME

UtilRnaml - It's an object which englobes Rnaml and Twig

=head1 SYNOPSIS


	
example: 


	
=head1 DESCRIPTION


=cut
# Package for RNAML :

use rnaml::rnaml;
use rnaml::molecule::molecule;
use rnaml::molecule::sequence::sequence;
use rnaml::molecule::identity::identity;
use rnaml::molecule::identity::taxonomy::taxonomy;
use rnaml::molecule::sequence::numbering_system::numbering_system;
use rnaml::molecule::sequence::numbering_system::numbering_range::numbering_range;
use rnaml::molecule::sequence::seq_annotation::seq_annotation;
use rnaml::molecule::sequence::seq_annotation::segment::segment;
use rnaml::molecule::structure::structure;
use rnaml::molecule::structure::model::model;
use rnaml::molecule::structure::model::model_info::model_info;
use rnaml::molecule::structure::model::model_info::free_energy::free_energy;
use rnaml::analysis::analysis;
use rnaml::analysis::program::program;
use rnaml::molecule::structure::model::str_annotation::str_annotation;
use rnaml::molecule::structure::model::secondary_structure_display::secondary_structure_display;
use rnaml::molecule_class::molecule_class;
use rnaml::molecule_class::alignment::alignment;
use rnaml::molecule_class::alignment::ali_sequence::ali_sequence;
use rnaml::molecule_class::consensus_molecule::consensus_molecule;
use rnaml::general::base_id_3p;
use rnaml::general::base_id_5p;
use rnaml::general::base_id;

use strict;
use warnings;
use XML::XPath;
use XML::XPath::Node::Element;
use rnaml::molecule::sequence::seq_data::seq_data;
use FileHandle;     
use XML::Twig;  
use XML::TreeBuilder;
use Learn;


# TAG des comment de la structure secondaire d'un ARN
my $TAG_PARENTHESIS="PARENTHESIS";
my $TAG_TABULATED="TABULATED";
my $TAG_RNAML="RNAML";

BEGIN
{
    our $VERSION = "0.1";
}

use Var;
#use vars qw($Var::E_XML $Var::DIR_BUG $Var::DIR_RELEASE );

my $rnaml_header='<?xml version="1.0"?><!DOCTYPE rnaml SYSTEM "'.$Var::DIR_RELEASE.'/lib/rnaml/rnaml.dtd">';
my $VALID_XML="xmllint --valid ";

=item SaveFamily

	Write a Family in Rnaml format from a family object.

=cut

sub SaveFamily
{
	my ($ro_family,$xml_dir,$tmp_dir, $R_LOGGER, $LOGGER_STAMP)=@_;

	if (not -e $$ro_family->GetFile("multalin"))
	{
		print "Error : file not found : ". $$ro_family->GetFile("multalin") ."\n";
	}

	### define rnaml structure ###
	my $rnaml = new rnaml::rnaml();
	$rnaml->version("1.1");
	my $molecule_class = new rnaml::molecule_class::molecule_class('molecule-class');
	$rnaml->molecule_class($molecule_class);

	my $identity = new rnaml::molecule::identity::identity();
	$molecule_class->identity($identity);
	
	my $name= new XML::XPath::Node::Element('name');
	$identity->name($name);
	my $numbering_system = new rnaml::molecule::sequence::numbering_system::numbering_system('numbering-system');
	$molecule_class->numbering_system($numbering_system);

	my $numbering_range = new rnaml::molecule::sequence::numbering_system::numbering_range::numbering_range('numbering-range');
	$numbering_system->numbering_range($numbering_range);

	my $alignment = new rnaml::molecule_class::alignment::alignment();
	$molecule_class->alignment($alignment);

	my $ali_sequence = new rnaml::molecule_class::alignment::ali_sequence::ali_sequence('ali-sequence');
	$alignment->ali_sequence($ali_sequence)	;
	
	my $molecule_id = new XML::XPath::Node::Element('molecule-id');
	$ali_sequence->molecule_id($molecule_id);
	my $seq_data = new rnaml::molecule::sequence::seq_data::seq_data('seq-data');
	$ali_sequence->seq_data($seq_data);

	my $consensus_molecule = new rnaml::molecule_class::consensus_molecule::consensus_molecule('consensus-molecule');
	$molecule_class->consensus_molecule($consensus_molecule);
	
	my $alignment_id = new XML::XPath::Node::Element('alignment-id');
	$consensus_molecule->alignment_id($alignment_id);

	my $molecule_cons = new rnaml::molecule::molecule();
	$consensus_molecule->molecule($molecule_cons);	

	my $sequence_cons = new rnaml::molecule::sequence::sequence();
	$molecule_cons->sequence($sequence_cons);		

	my $seq_data_cons = new rnaml::molecule::sequence::seq_data::seq_data('seq-data');
	$sequence_cons->seq_data($seq_data_cons);
	my $structure = new rnaml::molecule::structure::structure();
	$molecule_cons->structure($structure);

	# si la structure a été détectée par rnafold, on creer arborescence: structure/model/model-info/free-energy
	my $model=new rnaml::molecule::structure::model::model();
	$structure->model($model);

	my $model_info=new rnaml::molecule::structure::model::model_info::model_info();
	$model->model_info($model_info);

	my $free_en= new XML::XPath::Node::Element('free-energy');
	$model_info->free_energy($free_en);

	my $str_annotation=new rnaml::molecule::structure::model::str_annotation::str_annotation();
	$model->str_annotation($str_annotation);
	my $secondary_structure_display=new rnaml::molecule::structure::model::secondary_structure_display::secondary_structure_display();
	$alignment->id("A1");
	$numbering_system->id($$ro_family->GetId());
	$model->secondary_structure_display($secondary_structure_display);
	
	my $comment="";
	$comment.="$TAG_PARENTHESIS=".$$ro_family->GetStructure()->GetStructure()." " if ($$ro_family->GetStructure()->GetStructure() ne "") ;
	$comment.="$TAG_TABULATED=".$$ro_family->GetStructure()->GetSSFile()." " if (-e $$ro_family->GetStructure()->GetSSFile() ) ;
	$comment.="$TAG_RNAML=".$$ro_family->GetStructure()->GetRnamlFile()." " if (-e $$ro_family->GetStructure()->GetRnamlFile()) ;
	$str_annotation->comment($comment);
	
	$secondary_structure_display->comment($$ro_family->GetStructure()->GetImage());
	#enregistrement de la structure
	my $rnaml_file="$xml_dir/".$$ro_family->GetId().$Var::E_XML;

	open XML, ">$rnaml_file" or die "impossible d'ecrire : $rnaml_file";
	print XML $rnaml_header."\n" ; # A VOIR si necessaire
	print XML $rnaml;
	close XML;
	# tmp dir est un repertoire gééral dans fichier temp, on ajoute le nom de la famille
	$tmp_dir.=$$ro_family->GetId();
	if (! -d $tmp_dir)
	{
		mkdir ($tmp_dir);
	}
	my $xml_tmp_file="$tmp_dir/".$$ro_family->GetId().$Var::E_XML;
	

	###remplissage du rnaml avec Twig ###
	my $fh_file = new FileHandle ">$xml_tmp_file" or die ("ERROR: create $xml_tmp_file failed \n");
	my($xml_twig)=new XML::Twig( TwigHandlers =>  
	{ 
		'rnaml/molecule-class' => sub 	{ 
					&THSetAtt(@_, 'comment', $$ro_family->GetDesc()); 	
					&THSetDbxref(@_, $$ro_family);
					},
		'rnaml/molecule-class/identity' => sub 
		{	 &THSetAtt(@_, 'comment', $$ro_family->GetStatus()); },
		'rnaml/molecule-class/identity/name'=> sub 
		{	 &THSetElt(@_, $$ro_family->GetName()); },
		'rnaml/molecule-class/alignment/ali-sequence'	=> sub 
		{	 &THSetFamilySeqElts(@_, $ro_family); },	
		'rnaml/molecule-class/alignment' 	=> sub 	
		{
					&THAnalysisToElt(@_, \${$$ro_family->GetProgAlign()}); 
					&THSetAtt(@_, 'comment', $$ro_family->NumberOfRna()); 
		},
		'rnaml/molecule-class/consensus-molecule/alignment-id' => sub 
		{ &THSetAtt(@_, 'ref', "A1"); },
		'rnaml/molecule-class/consensus-molecule/molecule' => sub 
		{ 	&THSetAtt(@_, 'id', $$ro_family->GetId()."_cons"); },
		'rnaml/molecule-class/consensus-molecule/molecule/sequence/seq-data' => sub 
		{ &THSetElt(@_, $$ro_family->GetConsensus()); },
		'rnaml/molecule-class/consensus-molecule/molecule/structure/model/model-info/free-energy' => sub 
		{	 &THSetElt(@_, $$ro_family->GetStructure()->GetFreeEnergy()); },
		'rnaml/molecule-class/consensus-molecule/molecule/structure' => sub 
		{ 	 &THAnalysisToEltStructure(@_, $ro_family); },
		'rnaml/molecule-class/numbering-system/numbering-range'	=> sub 
		{ 	 &THSetFamilyLengthElt(@_, $ro_family); },
		'rnaml'	=> sub 
		{ &THSetRevisionElt(@_, $$ro_family); }
	} );
	$xml_twig->parsefile($rnaml_file);
	$xml_twig->set_pretty_print('record');
	$xml_twig->print($fh_file);
	$xml_twig->purge();
	$fh_file->close();
	my $cmd=$VALID_XML." $xml_tmp_file";

	if (0)
	{
	
	my $res=system($cmd);
	if ($res != 0) # xml file contain errors
	{
		$rnaml_file="$xml_dir/../$Var::DIR_BUG/".$$ro_family->GetId().$Var::E_XML;
		$$Learn::R_LOGGER->Trace("UtiliRnaml\tSaveFamily\t".$$ro_family->GetId()." file not valid saved in: $rnaml_file \n",$Learn::LOGGER_STAMP,"DE");
		
	}
	$$Learn::R_LOGGER->Trace("UtiliRnaml\tSaveFamily\tXmllint resultats:".$res."\tid: ".$$ro_family->GetId()."\n",$Learn::LOGGER_STAMP,"DE");
	}
	system "mv $xml_tmp_file $rnaml_file";

	rmdir ($tmp_dir);
}

=item THAnalysisToElt

	Set an analysis a the section

=cut


sub THAnalysisToElt
{
	my( $twig,  $section, $r_prog, $file)= @_;
	my $analyse = &GetAnAnalyse( $r_prog);
	
	&THSetAtt ( $twig, $section, 'analysis-ids', &THGetAtt($twig,  $analyse, 'id') );
	my (@a_etat, @gmt);
	if (defined ($file))
	{
		@a_etat=stat($file);
		@gmt=gmtime($a_etat[9]);
	}	
	else
	{
		@gmt=gmtime();
	}

	my $date= &GetDateFromGmtime(@gmt);
	$date->paste( last_child=> $analyse );
	$analyse->paste( last_child => $section->parent('rnaml'));	

}

=item THReferenceToElt

	Set an id analysis to the section and create the analysis

=cut

sub THAnalysisToEltStructure
{
	my( $twig,  $section, $ro_family)= @_;
	my $analyse = &GetAnAnalyse( \${$$ro_family->GetProgStructure()});

	&THSetAtt ( $twig, $section, 'analysis-ids', &THGetAtt($twig,  $analyse, 'id') );
	
	my $date;
	my $date_str=$$ro_family->GetStructure()->GetDateCompute();
	if (defined ($date_str ) )
	{
		$date= &GetDateFromString($date_str);
	}
  	elsif (-e $$ro_family->GetStructure()->GetImage())   	# Si le fichier image existe on récupère des infos dessus
	{
		my(@etat); 
		# recup les stat du fichier
		@etat=stat($$ro_family->GetStructure()->GetImage()); 
		# indice 9 => date
		$date= &GetDateFromGmtime(gmtime($etat[9]));

	} 

	$date->paste( last_child=> $analyse ) if (defined ($date ));

	# copie l'analyse (mfold ou rnafold)
	$analyse->paste( last_child => $section->parent('rnaml'));	

	# modifie id de l'analyse : nom du programme-version
	&THSetAtt ( $twig, $analyse, 'id', ${$$ro_family->GetStructure()->GetProgram()}->GetName()."-".${$$ro_family->GetStructure()->GetProgram()}->GetVersion());	

}
=item THSetFamilySeqElts

	Set all seq_data from alignement (multalin file)

=cut

sub THSetFamilySeqElts
{
	my( $twig,  $section, $ro_family)= @_;
	my $multalin_file = $$ro_family->GetFile("multalin");
	my $seq = Bio::Seq -> new();
	if ( !-e $multalin_file)
	{
		$$ro_family->DumpClustal();
	}
	my $in_aln = Bio::SeqIO->new( -file => "$multalin_file", '-format' => 'Fasta' );
	
	# on traite une fois 
	$seq = $in_aln->next_seq();
	&THSetAtt ( $twig, $section->descendants('molecule-id'), 'ref',$seq->id());

	# pour chaque sequence il faut une molecule à la base du rnaml.
	my $molecule= XML::Twig::Elt->new( 'molecule' );
	&THSetAtt ( $twig, $molecule, 'id', $seq->id());
	my $identity= XML::Twig::Elt->new( 'identity' );
	my $name= XML::Twig::Elt->new( 'name' );
	my $taxon= XML::Twig::Elt->new( 'taxonomy' );
	
	my $r_rna=$$ro_family->GetRna($$ro_family->IndexRnaId($seq->id()));
	my $species= XML::Twig::Elt->new( 'species' );
	&THSetElt ( $twig,$species,${$$r_rna->GetPrincipalRange()->GetSequence()}->GetOrganism() );
	&THSetElt ( $twig,$name,$$r_rna->GetName() );
	$species->paste(first_child => $taxon);
	$name->paste(first_child => $identity);
	$taxon->paste(last_child => $identity);	
	$identity->paste(first_child => $molecule);
	$molecule->paste(first_child => $section->parent('rnaml'));
	&THSetElt ( $twig,$section->descendants('seq-data'), $seq->seq());

	while( $seq = $in_aln->next_seq() ) # pour chaque alignement on va créer
	{	 
		if (defined $seq->id() &&  !($seq->id() =~/\#2$/ ) ) # verifie que l'id correspond a une clé des Arn
		{
			my $sequence=$section->copy($section);				

			&THSetAtt ( $twig, $sequence->descendants('molecule-id'), 'ref', $seq->id());
			# pour chaque sequence il faut une molecule à la base du rnaml.
			$molecule= XML::Twig::Elt->new( 'molecule' );
			&THSetAtt ( $twig, $molecule, 'id', $seq->id());
			$identity= XML::Twig::Elt->new( 'identity' );
 			$name= XML::Twig::Elt->new( 'name' );
			$taxon= XML::Twig::Elt->new( 'taxonomy' );
			
			$r_rna=$$ro_family->GetRna($$ro_family->IndexRnaId($seq->id()));
			$species= XML::Twig::Elt->new( 'species' );
			&THSetElt ( $twig,$species,${$$r_rna->GetPrincipalRange()->GetSequence()}->GetOrganism() );
			&THSetElt ( $twig,$name,$$r_rna->GetName() );
			$species->paste(first_child => $taxon);
			$name->paste(first_child => $identity);
			$taxon->paste(last_child => $identity);	
			$identity->paste(first_child => $molecule);
			
			$molecule->paste(first_child => $section->parent('rnaml'));
			&THSetElt ( $twig,$sequence->descendants('seq-data'), $seq->seq());
			$sequence->paste(after => $section);

		}
	}
}


=item THSetFamilyLengthElt

	Set numbering range of family

=cut

sub THSetFamilyLengthElt
{
	my( $twig,  $section, $ro_family)= @_;

	# crée des Elt start et end et les rajoutent aux numbering-range
	my $start= XML::Twig::Elt->new( start => 1 );
	$start->paste( first_child => $section);
	my $length  = XML::Twig::Elt->new( 'length' => $$ro_family->GetLength);
	$length->paste ( last_child => $section);
}

=item SaveRna

	Save in rnaml a rna object

=cut

sub SaveRna
{
	my ($o_rna,$xml_dir,$tmp_dir)=@_;
	
	
	### define rnaml structure ###
	my $rnaml = new rnaml::rnaml();
	$rnaml->version("1.1");

	my $molecule = new rnaml::molecule::molecule();
	$rnaml->molecule($molecule);

	my $identity = new rnaml::molecule::identity::identity();
	$molecule->identity($identity);

	my $sequence= new rnaml::molecule::sequence::sequence();
	$molecule->sequence($sequence);

	my $name= new XML::XPath::Node::Element('name');
	$identity->name($name);
	
	my $org= new XML::XPath::Node::Element('species');
	my $taxo= new rnaml::molecule::identity::taxonomy::taxonomy();
	$taxo->species($org);
	$identity->taxonomy($taxo);
	
	
	my $numbering_system = new rnaml::molecule::sequence::numbering_system::numbering_system();
	$sequence->numbering_system($numbering_system);
	
	my $numbering_range = new rnaml::molecule::sequence::numbering_system::numbering_range::numbering_range();
	$numbering_system->numbering_range($numbering_range);

	my $seq_data = new rnaml::molecule::sequence::seq_data::seq_data('seq-data');
	$sequence->seq_data($seq_data);
	
	# definition de la struture secondaire
	my $structure = new rnaml::molecule::structure::structure();
	$molecule->structure($structure);
	# si la structure a été détectée par rnafold, on creer arborescence: structure/model/model-info/free-energy
	if (${$o_rna->GetStructure()->GetProgram()}->GetName() ne 'mfold')
	{
		my $model=new rnaml::molecule::structure::model::model();
		$structure->model($model);
		my $model_info=new rnaml::molecule::structure::model::model_info::model_info();
		$model->model_info($model_info);
		my $str_annotation=new rnaml::molecule::structure::model::str_annotation::str_annotation();
		$model->str_annotation($str_annotation);
		my $secondary_structure_display=new rnaml::molecule::structure::model::secondary_structure_display::secondary_structure_display();
		$model->secondary_structure_display($secondary_structure_display);
		
		my $comment="";
		$comment.="$TAG_PARENTHESIS=".$o_rna->GetStructure()->GetStructure()." " if ($o_rna->GetStructure()->GetStructure() ne "") ;
		$comment.="$TAG_TABULATED=".$o_rna->GetStructure()->GetSSFile()." " if (-e $o_rna->GetStructure()->GetSSFile() ) ;
		$comment.="$TAG_RNAML=".$o_rna->GetStructure()->GetRnamlFile()." " if (-e $o_rna->GetStructure()->GetRnamlFile()) ;

		$str_annotation->comment($comment);
		$secondary_structure_display->comment($o_rna->GetStructure()->GetImage());
	}
	# set des attributs :
	$molecule->id($o_rna->GetId());
	$molecule->type("rna");
	
	$molecule->comment($o_rna->GetDesc());
	$sequence->strand($o_rna->GetPrincipalRange()->GetStrand());
	$sequence->length($o_rna->GetPrincipalRange()->GetLength());
	
	# annotation 
	my $nb_annot=$o_rna->GetNbAnnotations();
	if ($nb_annot>=0)
	{
		my $seq_annot= new rnaml::molecule::sequence::seq_annotation::seq_annotation('seq-annotation');
			
		my $segment=new rnaml::molecule::sequence::seq_annotation::segment::segment('segment');
		my $b5p= new rnaml::general::base_id_5p('base-id-5p');
		my $b3p= new rnaml::general::base_id_3p('base-id-3p');
		my $b_id=new rnaml::general::base_id('base-id');
		my $pos=new XML::XPath::Node::Element('position');
		my $seg_name=new XML::XPath::Node::Element('seg-name');
		$b_id->position($pos);
		$b5p->base_id($b_id);
		$b3p->base_id($b_id);
		$segment->base_id_5p($b5p);
		$segment->base_id_3p($b3p);
		$segment->seg_name($seg_name);
		$seq_annot->segment($segment);
		$sequence->seq_annotation($seq_annot);
	}
	
	if (! -d $tmp_dir)
	{
		mkdir ($tmp_dir);
	}
	#enregistrement de la structure
	my $rnaml_file="$xml_dir/".$o_rna->GetId().$Var::E_XML;

	open (XML, ">$rnaml_file") or die "impossible d'ecrire : $rnaml_file";
	print XML $rnaml_header."\n" ;# A VOIR si necessaire
	print XML $rnaml;
	close (XML);
	

	my $xml_tmp_file="$tmp_dir/".$o_rna->GetId().$Var::E_XML;
	my $index=-1;
	
	###remplissage du rnaml avec Twig ###

	my $fh_file = new FileHandle ">$xml_tmp_file" or die ("ERROR: create $xml_tmp_file failed \n");
	my($xml_twig)=new XML::Twig( TwigHandlers =>  { 
		'rnaml/molecule/identity/name' 	=> sub { &THSetElt(@_, $o_rna->GetName()); },
		'rnaml/molecule/identity' 	=> sub { &THSetAtt(@_, 'comment' ,$o_rna->GetStatus()); },
		'rnaml/molecule/identity/taxonomy/species' => sub { &THSetElt(@_, ${$o_rna->GetPrincipalRange()->GetSequence()}->GetOrganism()   ); },
		'rnaml/molecule/sequence'	=> sub { &THSetRnaSequenceElts(@_, $o_rna); },
		'rnaml/molecule/structure'	=> sub { &THSetRnaStructureElt(@_, $o_rna); },
		'rnaml/molecule'		=> sub { &THSetDbxref(@_, $o_rna); },
		'rnaml'				=> sub { &THSetRevisionElt(@_, $o_rna); },
		'rnaml/molecule/sequence/seq-annotation'=> sub {&THSetRnaAnnotationElts(@_, $o_rna);},
							 } );
	$xml_twig->parsefile($rnaml_file);
	$xml_twig->set_pretty_print('record');
	$xml_twig->print($fh_file);
	$xml_twig->purge;
	$fh_file->close;

	my $cmd=$VALID_XML." $xml_tmp_file";
	if (0)
	{
	my $res=system($cmd);
	
	if ($res != 0) # xml file contain errors
	{
		$rnaml_file="$xml_dir/../$Var::DIR_BUG/".$o_rna->GetId().$Var::E_XML;
		$$Learn::R_LOGGER->Trace("UtiliRnaml\tSaveRna\t".$o_rna->GetId()." file not valid saved in: $rnaml_file \n",$Learn::LOGGER_STAMP,"DE");
		
	}
	
	$$Learn::R_LOGGER->Trace("UtiliRnaml\tSaveRna\tXmllint resultats:".$res."\tid: ".$o_rna->GetId()."\n",$Learn::LOGGER_STAMP,"DE");
}
	system "mv $xml_tmp_file $rnaml_file";
	
	#rm ($tmp_dir);
}
#------------------ Complex Twig functions  -------------------
=item THSetRnaSequenceElts

	Set all Numbering Range with primiry range and secondaries ranges

=cut

sub THSetRnaSequenceElts
{
	my( $twig,  $section, $o_rna)= @_;
	#RANGE PRINCIPAL
	#Fixe les valeurs des attributs de sequence
	&THSetAtt ( $twig, $section, 'strand', $o_rna->GetPrincipalRange()->GetStrand());
	&THSetAtt ( $twig,  $section, 'comment', "primary");
	
	#Fixe les valeurs des attributs de numbering-system
	&THSetAtt ( $twig, $section->descendants('numbering-system'), 'id',${$o_rna->GetPrincipalRange()->GetSequence()}->GetId());
	# id de l'analyse : nom du programme-version
	my $analysis_id=${$o_rna->GetPrincipalRange()->GetProgDetect()}->GetName()."-".${$o_rna->GetPrincipalRange()->GetProgDetect()}->GetVersion();
	&THSetAtt ( $twig, $section, 'analysis-ids', $analysis_id );

	# On ne verifie pas si l'analyse existe deja puisque range principal
	my $analysis = &GetAnAnalyse($o_rna->GetPrincipalRange()->GetProgDetect());
	&THSetAtt ( $twig, $analysis, 'id', $analysis_id);
	#ajoute la date a l'analyse
	my $date= &GetDateFromString($o_rna->GetPrincipalRange()->GetDateDetect());
	$date->paste( last_child=> $analysis );
	$analysis->paste( last_child => $section->parent('rnaml'));

	# crée des Elt start et end et les rajoutent aux numbering-range
	my $start= XML::Twig::Elt->new( start => $o_rna->GetPrincipalRange()->GetStart() );
	$start->paste( first_child => $section->descendants('numbering-range'));
	my $end  = XML::Twig::Elt->new( end => $o_rna->GetPrincipalRange()->GetEnd());
	$end->paste ( last_child => $section->descendants('numbering-range'));
	&THSetElt ( $twig,$section->descendants('seq-data'), $o_rna->GetPrincipalRange()->GetSeqData());
	
	#RANGES SECONDAIRES
	for (my $i=0; $i<=$#{$o_rna->{'__a_ranges'}}; $i++)
	{
		my $sequence=$section->copy($section);	
				
		&THSetElt ( $twig,$sequence->descendants('seq-data'), $o_rna->{'__a_ranges'}[$i]->GetSeqData());
		&THSetElt ( $twig,$sequence->descendants('start'), $o_rna->{'__a_ranges'}[$i]->GetStart());
		&THSetElt ( $twig,$sequence->descendants('end'), $o_rna->{'__a_ranges'}[$i]->GetEnd());
		&THSetAtt ( $twig, $sequence, 'strand', $o_rna->{'__a_ranges'}[$i]->GetStrand());
		&THSetAtt ( $twig, $sequence, 'comment', "secondary");
		&THSetAtt ( $twig, $sequence, 'length', $o_rna->{'__a_ranges'}[$i]->GetLength());
		#Fixe en attributs de numbering-system l'id de la sequence mère
		# A revoir car, si sur la meme sequence mere alors, numbering range dans le même numbering system!
		&THSetAtt ( $twig, $sequence->descendants('numbering-system'), 'id',${$o_rna->{'__a_ranges'}[$i]->GetSequence()}->GetId());
		
		$sequence->paste(after => $section);
		# test si analyse existante 
		$analysis_id=${$o_rna->{'__a_ranges'}[$i]->GetProgDetect()}->GetName()."-".${$o_rna->{'__a_ranges'}[$i]->GetProgDetect()}->GetVersion();
		# id de l'analyse : nom du programme-version
		&THSetAtt ( $twig, $sequence, 'analysis-ids', $analysis_id );
	
		my $find=0;
		foreach my $node ($twig->descendants('analysis'))
		{
			if ($node->{'att'}->{'id'} eq $analysis_id) 
			{
				$find=1;
				last;
			}
		}
		
	
		if ($find == 0 ) #si analyse inexistante
		{
			# traite l'analyse :
			$analysis = &GetAnAnalyse($o_rna->{'__a_ranges'}[$i]->GetProgDetect());
			&THSetAtt ( $twig, $analysis, 'id', $analysis_id);
			#ajoute la date a l'analyse
			$date= &GetDateFromString($o_rna->{'__a_ranges'}[$i]->GetDateDetect());
			$date->paste( last_child=> $analysis );
			$analysis->paste( last_child => $sequence->parent('rnaml'));
		}
		# range secondaire n'ont pas d'annotation
		my @seq_annot=$sequence->children('seq-annotation');
		foreach my $value (@seq_annot)
		{
			$value->delete();
		}
	}
}


=item THSetRnaAnnotationElts

	Set all Annotation 

=cut

sub THSetRnaAnnotationElts
{
	my( $twig,  $section, $o_rna)= @_;
	
	my $nb_annot=$o_rna->GetNbAnnotations();
	my $i=0;
	# on garde le modele seq_annotation.
	my $seq_annotation=$section;		
	#if ( &THGetAtt ($twig, $seq_annotation->parent('sequence'), 'comment' ) eq "primary")
	#{
		while ($i <= $nb_annot)
		{		
			&THSetAtt ($twig, $seq_annotation, 'comment', $o_rna->GetAnnotate($i)->GetLogin());
			my @elts=$seq_annotation->get_xpath('./segment');
			my $pos=$elts[0];
			my $txt="TIMESTAMP=".$o_rna->GetAnnotate($i)->GetTimeStamp() if ( defined( $o_rna->GetAnnotate($i)->GetTimeStamp() ) && $o_rna->GetAnnotate($i)->GetTimeStamp() ne "" );
			$txt.=" LEGEND=".$o_rna->GetAnnotate($i)->GetLegend() if ( defined( $o_rna->GetAnnotate($i)->GetLegend() ) && $o_rna->GetAnnotate($i)->GetLegend() ne "" );
			 
			&THSetAtt ($twig, $pos, 'comment',$txt)if (defined($pos));
			
			@elts=$seq_annotation->get_xpath('./segment/base-id-5p/base-id/position');
			$pos=$elts[0];
			&THSetElt ($twig, $pos, $o_rna->GetAnnotate($i)->GetStart())if (defined($pos));
			
			@elts=$seq_annotation->get_xpath('./segment/base-id-3p/base-id/position');
			$pos=$elts[0];
			&THSetElt ($twig, $pos, $o_rna->GetAnnotate($i)->GetEnd())if (defined($pos));
			
			@elts=$seq_annotation->get_xpath('./segment/seg-name');
			$pos=$elts[0];
			
			&THSetElt ($twig, $pos, $o_rna->GetAnnotate($i)->GetTitle()) if (defined($pos));
			if ($i>0) # si on a traiter le premier element
			{
				$seq_annotation->paste( after => $section );
			}
			$seq_annotation=$section->copy($section);	
			$i++;
		}
	

}







=item THSetDbxref

	Set dbxref

=cut

sub THSetDbxref
{
	my( $twig,  $section, $obj)= @_;

	&THSetAtt ( $twig, $section, 'database-ids', $obj->GetDbxref());
	my $dbxref=$obj->GetDbxref();

	my $db_entry = XML::Twig::Elt->new('database-entry');
	&THSetAtt ( $twig, $db_entry, 'id', $dbxref);

	# si on a un famille on peux avoir plusieur dbxref séparé par un espace
	my @a_T=split(" ",$dbxref);
		
	if ($#a_T>-1)
	{
		foreach my $db (@a_T)
		{	
			next if ($db eq "");
			# on resplite pour obtenir nom de banque et id
			my @a_db_id=split (/_/, $db);
			if ($#a_db_id>0)
			{
				my $db_xml = XML::Twig::Elt->new('database'=> $a_db_id[0]);
				$db_xml->paste(last_child=> $db_entry);
				my $entry= XML::Twig::Elt->new('entry'=> $a_db_id[1]);
				$entry->paste(last_child=> $db_entry);
			}
			else
			{
				my $db_xml = XML::Twig::Elt->new('database'=> $db);
				$db_xml->paste(last_child=> $db_entry);
				my $entry= XML::Twig::Elt->new('entry'=> $db);
				$entry->paste(last_child=> $db_entry);
			}

		}
	}
	$db_entry->paste( last_child => $section->parent('rnaml'));
}


=item THAddRnaSSDisplay

	Set secondary structure display element
Fn a supprimer !
=cut

sub THAddRnaSSDisplay
{
	my( $twig,  $section, $o_rna)= @_;
	# il faut faire un test si le neoud existe deja ?!!
	my $ss_display;
	if (defined $section->child(0, 'secondary-structure-display'))
	{
		$ss_display->cut ($section->child(0, 'secondary-structure-display'));
	}
	else
	{
		$ss_display= XML::Twig::Elt->new( 'secondary-structure-display');
	}
	&THSetAtt($twig,$ss_display ,'comment', $o_rna->GetStructure()->GetImage()); 
	$ss_display->paste (last_child => $section);		
}
=item THSetRnaStructureElt

	Set secondary structure

=cut

sub THSetRnaStructureElt
{
	my( $twig,  $section, $o_rna)= @_; 
	# teste quel prg a ete exec pour le calcul de la ss
	# si mfold copie du fichier rnaml sinon get des info de rnafold
	my ( $analysis );
	my $prg_name=${$o_rna->GetStructure()->GetProgram()}->GetName();
	
	
	if ($prg_name eq 'mfold')
	{	
		my $file=$o_rna->GetStructure()->GetRnamlFile();
		my ( $mfold_ss );
		if (-e $file && ! -d $file)
		{
			my $comment="";
			$comment.="$TAG_PARENTHESIS=".$o_rna->GetStructure()->GetStructure()." " if ($o_rna->GetStructure()->GetStructure() ne "") ;
			$comment.="$TAG_TABULATED=".$o_rna->GetStructure()->GetSSFile()." " if (-e $o_rna->GetStructure()->GetSSFile()) ;
			$comment.="$TAG_RNAML=".$o_rna->GetStructure()->GetRnamlFile()." " if (-e $o_rna->GetStructure()->GetRnamlFile()) ;

			my($xml_twig)=new XML::Twig( TwigHandlers =>  
			{ 
				'rnaml/molecule/structure/model/str-annotation' => sub 
				{
				&THSetAtt(@_,'comment',$comment); 
				},
				'rnaml/analysis/program/prog-name' => sub 
				{ &THSetElt(@_, lc(&THGetElt(@_))); 	}, 
				'rnaml/molecule/structure/model' => sub 
				{ $mfold_ss= &CopyElt(@_); },
				'rnaml/analysis' => sub 
				{$analysis= &CopyElt(@_);} 
			} );
			$xml_twig->parsefile($file);
			$xml_twig->purge();
			$mfold_ss->paste( last_child => $section);
			# (not defined $section->descendants('secondary-structure-display'))

			my $ss_display= XML::Twig::Elt->new( 'secondary-structure-display');
			&THSetAtt($twig,$ss_display ,'comment', $o_rna->GetStructure()->GetImage());
			$ss_display->paste(last_child => $section->descendants('model'));
			
		}
		else
		{
			#creation de l'analysis
			$analysis = &GetAnAnalyse($o_rna->GetStructure()->GetProgram());	
		}
	}
	else #if ($prg_name eq 'rnafold')# rnafold 
	{	
		# 1er getStructure : objet structure de o_rna, le 2nd le champ structure de l'objet structure
			
		my $free_en= XML::Twig::Elt->new( 'free-energy' => $o_rna->GetStructure()->GetFreeEnergy());
		# paste la free energy dans le model info.
		$free_en->paste( last_child => $section->descendants('model-info'));		

		#creation de l'analysis
		$analysis = &GetAnAnalyse($o_rna->GetStructure()->GetProgram());
		
	}
	my $date;
	my $date_str= $o_rna->GetStructure()->GetDateCompute();
	if (defined ($date_str) && $date_str ne "")
	{
		$date= &GetDateFromString($date_str);
	} 	
  	elsif (-e $o_rna->GetStructure()->GetImage()) # Si le fichier image existe on récupère des infos dessus
	{
		my(@etat); 
		# recup les stat du fichier
		@etat=stat($o_rna->GetStructure()->GetImage()); 
		# indice 9 => date
		$date= &GetDateFromGmtime(gmtime($etat[9]));
		
	} 
	
	$date->paste( last_child=> $analysis ) if (defined ($date)) ;
	# copie l'analyse (mfold ou rnafold)
	$analysis->paste( last_child => $section->parent('rnaml'));	

	# modifie id de l'analyse : nom du programme-version
	&THSetAtt ( $twig, $analysis, 'id', ${$o_rna->GetStructure()->GetProgram()}->GetName()."-".${$o_rna->GetStructure()->GetProgram()}->GetVersion());	

	# Ajout ed la famille dans le comment de structure
	&THSetAtt ( $twig, $section, 'comment', $o_rna->GetFamilyAcc());

	# ajoute l'attribut id de analyse à la structure : nom du programme-version
	&THSetAtt ( $twig, $section, 'analysis-ids',${$o_rna->GetStructure()->GetProgram()}->GetName()."-".${$o_rna->GetStructure()->GetProgram()}->GetVersion());
}

=item GetDateFromGmtime

	Return a elemnt date (xml) from a Gmtime date
	
=cut

sub GetDateFromGmtime
{
	my ($sec, $min, $hour, $day, $mon, $year)=(@_);
	# format de la date : gmtime : $sec, $min, $hour, $mday, $mon, $year
	my $xdate = XML::Twig::Elt->new('date');
	if (not (defined ($day) && defined ($mon) && defined ($year)))
	{
		($sec, $min, $hour, $day, $mon, $year)=gmtime();
	}
	$mon++;
	$year+=1900;
	#Date de l'analyse today !
	# On convertit avec gmtime la valeur de mtime.
	my $xday = XML::Twig::Elt->new('day' => sprintf("%02i", $day) );
	$xday->paste(last_child=> $xdate);
	my $xmon = XML::Twig::Elt->new('month'=> sprintf("%02i", $mon) );
	$xmon->paste(last_child=> $xdate);
	my $xyear= XML::Twig::Elt->new('year'=> sprintf("%04i", $year));
	$xyear->paste(last_child=> $xdate);
	return $xdate;
}

=item GetDateFromString

	Return a elemnt date (xml) from a string date AAAAMMDD
	
=cut

sub GetDateFromString
{
	my ($date)=@_;
	my $xdate = XML::Twig::Elt->new('date');
	if (defined($date) && $date ne "")
	{
		my $xday = substr($date, 6,2);	my $xmon = substr($date, 4,2 ); my $xyear = substr($date, 0,4);
		
		$xday = XML::Twig::Elt->new('day' => $xday);
		$xday->paste(last_child=> $xdate);
		$xmon = XML::Twig::Elt->new('month'=> $xmon);
		$xmon->paste(last_child=> $xdate);
		$xyear= XML::Twig::Elt->new('year'=> $xyear);
		$xyear->paste(last_child=> $xdate);
	}
	return $xdate;
}

=item GetAnAnalyse

	Return an analyse elemnt date (xml) from a program object.
	
=cut

sub GetAnAnalyse
{
	# Créer un element analyse en fonction d'une reference de programme 
	my ($r_prog)=@_;
	# format de la date : gmtime : $sec, $min, $hour, $mday, $mon, $year

	my $prg_name=XML::Twig::Elt->new( 'prog-name' => ${$r_prog}->GetName() );
	my $prg_version=XML::Twig::Elt->new( 'prog-version' => ${$r_prog}->GetVersion() );
	my $prg= new XML::Twig::Elt->new( 'program' => $prg_name);	
	$prg_version->paste(last_child => $prg);
	my $analysis = new  XML::Twig::Elt->new( 'analysis' => $prg);
	&THSetAtt(undef, $analysis, 'id' , ${$r_prog}->GetName()."-".${$r_prog}->GetVersion() );
	return $analysis;
}

=item CopyElt

	Copy an element
	
=cut

sub CopyElt
{
	my( $ss_twig,  $ss_section)= @_;
	return $ss_section->copy();

}

=item THSetRevisionElt

	Set a revision date in xml from a Rna or a Family
	
=cut

# generic fonction for family and rna
sub THSetRevisionElt
{
	my( $twig,  $section, $object)= @_;
	my $date=$object->GetLastUpd();
	
	my ($xday, $xmon, $xyear);
	$xday = substr($date, 6,2);	$xmon = substr($date, 4,2 ); $xyear = substr($date, 0,4);

	if ( $section->descendants('revision') )
	{
		if (defined($date))
		{ 	
			my $curent_date = $section->descendants('revision')->{'year'}->text().$section->descendants('revision')->{'month'}->text().$section->descendants('revision')->{'day'}->text();
			# on met a jour la date de revsion (ou ajoute  une revision ??? )
			if (int($curent_date) < int($date)) 
			{
					
				$section->descendants('revision')->{'year'}->text($xyear);
				$section->descendants('revision')->{'month'}->text($xmon);
				$section->descendants('revision')->{'day'}->text($xday);
			}
		}
	}
	else
	{
		my $rev = XML::Twig::Elt->new('revision');
		
		my $xdate;
		if (defined($date))
		{
			$xdate= &GetDateFromString($date);
		}
		else
		{
			my @gmt=gmtime();
			$xdate= &GetDateFromGmtime(@gmt);
		}
		$xdate->paste(first_child =>$rev );
		$rev->paste( last_child => $section);
	}
	

}





=item LoadFamily

	Set attributs of a family from an xml file.
	
=cut


sub LoadFamily
{
	my ($ro_family,$xml_file)=(@_);
	my ( $day, $mon, $year , $prog_struct_id, $struct, $dbxref);
	my($xml_twig)=new XML::Twig( TwigHandlers =>  
	{ 
		'rnaml/molecule-class'		=> sub 	
		{											$dbxref = &THGetAtt(@_, 'database-ids');
		$$ro_family->SetDesc(&THGetAtt(@_, 'comment'));
		},
		'rnaml/molecule-class/identity' 		=> sub 	
		{	$$ro_family->SetStatus(&THGetAtt(@_, 'comment')); },
		'rnaml/molecule-class/identity/name' 	=> sub 	
		{	$$ro_family->SetName(&THGetElt(@_)); },
		'rnaml/molecule-class/numbering-system'	=> sub 
		{ 	$$ro_family->SetId(&THGetAtt(@_, 'id')); },
		'rnaml/molecule-class/numbering-system/numbering-range/length'=> sub 	
		{ $$ro_family->SetLength(&THGetElt(@_));},
		'rnaml/molecule-class/alignment' => sub 	
		{
			my $prog_align= &THGetAtt(@_, 'analysis-ids');
			$$ro_family->SetProgAlign(&Learn::GetRefProgram($prog_align, "pseudo") );
		},
		'rnaml/molecule-class/consensus-molecule/molecule/sequence/seq-data'	=> sub 
		{ $$ro_family->SetConsensus(&THGetElt(@_));},
		'rnaml/molecule-class/consensus-molecule/molecule/structure'=> sub 
		{ 
		$prog_struct_id=&THGetAtt(@_, 'analysis-ids') ;
		$$ro_family->SetProgStructure(&Learn::GetRefProgram( $prog_struct_id, "pseudo" ));
		},	
		'rnaml/molecule-class/consensus-molecule/molecule/structure/model/model-info/free-energy'=> sub 
		{ $$ro_family->GetStructure()->SetFreeEnergy(&THGetElt(@_)); },
		'rnaml/molecule-class/consensus-molecule/molecule/structure/model/secondary-structure-display'=> sub 	
		{ $$ro_family->GetStructure()->SetImage(&THGetAtt(@_, 'comment')); },
		'rnaml/molecule-class/consensus-molecule/molecule/structure/model/str-annotation'=> sub
		{$struct = &THGetAtt(@_, 'comment');},
		'rnaml/revision/date/day'	=> sub { $day=&THGetElt(@_); },
		'rnaml/revision/date/month'	=> sub { $mon=&THGetElt(@_); },
		'rnaml/revision/date/year'	=> sub { $year=&THGetElt(@_); },
	} );
	$xml_twig->parsefile($xml_file);
	$$ro_family->SetDbxref($dbxref);
	if (defined($struct))
	{
		my @a_struc=split (' ', $struct);
		foreach my $tag (@a_struc)
		{
			my @a_tag=split(/=/,$tag);
			if ($a_tag[0] =~ /$TAG_PARENTHESIS/)
			{
				$$ro_family->GetStructure()->SetStructure($a_tag[1]) ;
			}
			elsif ($a_tag[0] =~ /$TAG_RNAML/)
			{
				$$ro_family->GetStructure()->SetRnamlFile($a_tag[1]);
			}
			elsif ($a_tag[0] =~ /$TAG_TABULATED/)
			{
				$$ro_family->GetStructure()->SetSSFile($a_tag[1]);
			}
		}
	}
	
	my $analyse_node=$xml_twig->elt_id($prog_struct_id);
	if (defined($analyse_node))
	{	
		my $date_node=$analyse_node->last_child_matches('date');
		my @day_a=$date_node->children('day');
		my @month_a = $date_node->children('month');
		my @year_a = $date_node->children('year');
#		print "$year_a$month_a$day_a\n";
		$$ro_family->GetStructure()->SetDateCompute($year_a[0]->text().$month_a[0]->text().$day_a[0]->text());
	}
	$xml_twig->purge;
	$$ro_family->SetLastUpd("$year$mon$day");
}




=item LoadRna

	Set attributs of a rna from an xml file.
	
=cut


sub LoadRna
{
	my ($ro_rna,$xml_file,  $dir_rna_release, $rh_sequences)=(@_);
	my ( $id,$prog_ss, $day, $mon, $year, $family, $free_en, $strut_string,$strut_img);
	my %h_analysis;
	print "LoadFromRnaml $xml_file\n";
	$$Learn::R_LOGGER->Trace("Rna\tLoadFromRnaml\t $xml_file\n",$Learn::LOGGER_STAMP,"DE");

	my($xml_twig_analysis)=new XML::Twig( TwigHandlers =>  
		{ 
		'rnaml'		=> sub 	{&GetAnalysis(@_, \%h_analysis);},
		});
	$xml_twig_analysis->parsefile($xml_file);
	$xml_twig_analysis->purge;
	$xml_twig_analysis->DESTROY();

	my($xml_twig)=new XML::Twig( TwigHandlers =>  
	{ 
	'rnaml/molecule'	=> sub 	{
		$$ro_rna->SetDesc(&THGetAtt(@_, 'comment'));
		$$ro_rna->SetDbxref(&THGetAtt(@_, 'database-ids'));
		$id=&THGetAtt(@_, 'id');	 
					},
	'rnaml/molecule/identity' 	=> sub {$$ro_rna->SetStatus(&THGetAtt(@_, 'comment')); },
	'rnaml/molecule/identity/name' 	=> sub {$$ro_rna->SetName(&THGetElt(@_)); },
	'rnaml/molecule/sequence'	=> sub 	{&THTreatSequences(@_, $ro_rna,\%h_analysis, $rh_sequences);
						&THGetAnnotation(@_, $ro_rna);},
	'rnaml/molecule/structure'	=> sub 	{ 
			$prog_ss=&THGetAtt(@_, 'analysis-ids'); 
			$family=&THGetAtt(@_, 'comment'); 
						},
	'rnaml/molecule/structure/model/model-info/free-energy' => sub { $free_en=&THGetElt(@_);},
	'rnaml/molecule/structure/model/secondary-structure-display'=> sub { $strut_img=&THGetAtt(@_, 'comment'); },
	'rnaml/molecule/structure/model/str-annotation'=> sub 	{$strut_string=&THGetAtt(@_, 'comment');}, # ce noeud est rempli par mfold le comment existe si c'est rnafold qui l'a detecter.PAS FORCEMENT!!!
	'rnaml/revision/date/day'=> sub { $day=&THGetElt(@_); },
	'rnaml/revision/date/month'=> sub { $mon=&THGetElt(@_); },
	'rnaml/revision/date/year'=> sub { $year=&THGetElt(@_); },
	} );
	$xml_twig->parsefile($xml_file);
	$xml_twig->purge;

	$$ro_rna->SetId($id);
	$$ro_rna->SetPath($dir_rna_release);
	$$ro_rna->SetLastUpd("$year$mon$day");
	
	$$ro_rna->SetFamilyAcc($family);
	$$ro_rna->{'__stucture'}->SetFreeEnergy($free_en);
	# le comment de str-annotation correspont soit a la chaine parathésée soit au fichier rnaml de mfold
	if (defined($strut_string))
	{
	
		my @a_struc=split (' ', $strut_string);
		foreach my $tag (@a_struc)
		{
			my @a_tag=split(/=/,$tag);
			if ($a_tag[0] =~ /$TAG_PARENTHESIS/)
			{
				$$ro_rna->{'__stucture'}->SetStructure($a_tag[1]) ;
			}
			elsif ($a_tag[0] =~ /$TAG_RNAML/ && -e $a_tag[1])
			{
				$$ro_rna->{'__stucture'}->SetRnamlFile($a_tag[1]);
			}
			elsif ($a_tag[0] =~ /$TAG_TABULATED/ && -e $a_tag[1])
			{
				$$ro_rna->{'__stucture'}->SetSSFile($a_tag[1]);
			}
		}
	}

	# on stocke l'ancien chemin qui sera modifier ensuite avec le deplacement du fichier
	$$ro_rna->{'__stucture'}->SetImage($strut_img);
	my $r_prg= &Learn::GetRefProgram($prog_ss, "pseudo" );
	
	$$ro_rna->SetProgForStructure( $r_prg ) ;
	my $analyse_tmp= $h_analysis{$prog_ss};
	my @a_analyse= split ( ";", $analyse_tmp); # ex : mFold;3.2;20051123
	$$ro_rna->{'__stucture'}->SetDateCompute($a_analyse[2]) if (defined ($a_analyse[2]));

}


sub printH_ana
{
	my ($rh_analysis)=(@_);
	
	my $id;
	foreach $id ( keys %{$rh_analysis} ) 
	{
		print "$id :  $rh_analysis->{$id}\n";
	}

}

=item THGetAnnotation

	Set an annotation to the rna object from xml file
	
=cut


sub THGetAnnotation
{
	my ( $twig, $section, $ro_rna)=(@_);
	
	my @seq_annot=$section->get_xpath('seq-annotation');
	foreach my $seq_an (@seq_annot)
	{
		my $login = &THGetAtt($twig,$seq_an, 'comment');
		my $legend = &THGetAtt($twig,$seq_an->child('segment'), 'comment');
				
		my @a_tmp=split(' ', $legend,2);
		
		my @a_time=split(/=/, $a_tmp[0]);
		
		my @a_comment=split(/=/, $a_tmp[1]);	
		
		my @elts=$seq_an->get_xpath('segment/base-id-5p/base-id/position');
		my $pos=$elts[0];
		my $start = &THGetElt($twig,$pos);
		
		@elts=$seq_an->get_xpath('segment/base-id-3p/base-id/position');
		$pos=$elts[0];
		my $end = &THGetElt($twig,$pos);
		
		@elts=$seq_an->get_xpath('segment/seg-name');
		$pos=$elts[0];
		my $title = &THGetElt($twig,$pos);
		
		$$ro_rna->AddNewAnnotate($start,$end,$title,$a_comment[1],$login, $a_time[1]);
	}
}

=item GetAnalysis

	Fill in hash with analyse data on rnml file
	
=cut

sub GetAnalysis
{
	my ( $twig, $section,$rh_analysis)=(@_);
	my @nodes=$twig->findnodes('/rnaml/analysis');
	foreach my $node (@nodes)
	{
		my $id = &THGetAtt($twig,$node, 'id');
		my $name = &THGetElt($twig,$node->descendants('prog-name'));
		my $version = &THGetElt($twig,$node->descendants('prog-version'));
		my $date = &THGetElt($twig,$node->descendants('year'))."".&THGetElt($twig,$node->descendants('month'))."".&THGetElt($twig,$node->descendants('day'));
		$$rh_analysis{$id}="$name;$version;$date";
	}
}

=item THTreatSequences

	Fill in rna object with sequence data.
	
=cut

sub THTreatSequences
{
	my ($twig,$section, $ro_rna,$rh_analysis, $rh_sequences)=(@_) ;
	# on parse toutes les séquences, recupère toutes les donnée, fait le lien avec les prg et la sequence mere , puis on enregistre
	my @lst_nodes=$twig->findnodes('/rnaml/molecule/sequence');
	foreach my $node (@lst_nodes)
	{
	
		my $analysis_ids=&THGetAtt($twig,$node, 'analysis-ids');
		my $length=&THGetAtt($twig,$node, 'length');
		my $strand=&THGetAtt($twig,$node, 'strand');
		my $seq_acc=&THGetAtt($twig,$node->descendants('numbering-system'), 'id');
		my $start=&THGetElt($twig,$node->descendants('start'));
		my $end=&THGetElt($twig,$node->descendants('end'));
		my $seq_data=&THGetElt($twig,$node->descendants('seq-data'));
		my $r_seq= \${$rh_sequences}{$seq_acc} if defined ($rh_sequences->{$seq_acc});
		my $analyse_tmp= $rh_analysis->{$analysis_ids};
		my @a_analyse= split ( ";", $analyse_tmp); # ex : mFold;3.2;20051123
		my $r_prg= &Learn::GetRefProgram( $analysis_ids, "pseudo" );
		my $detect_date =  $a_analyse[2];
		if (&THGetAtt($twig,$node, 'comment') eq "primary")
		{
			$$ro_rna->SetPrincipalRange($start, $end, $strand, $r_seq, $r_prg, $detect_date,$seq_data,$length);
		}
		elsif (&THGetAtt($twig,$node, 'comment') eq "secondary")
		{
			$$ro_rna->AddSecondaryRange($start, $end,$strand , $r_seq,$r_prg, $detect_date, $seq_data, $length);
		}
	}

}


#------------------------------------------------ Basic Twig functions  ---------------------------------
 
=head1 BasicTwigfunctions
 
=item THSetAtt

	initialize the content of an attribute

=cut

sub THSetAtt
{
my( $twig, $section, $attribut, $value)= @_;

        $section->{'att'}->{$attribut} = $value;
}

=item THSetElt

	initialize the content of an element

=cut

sub THSetElt
{
my( $twig, $section, $value)= @_;

        $section->set_text($value);
}

=item THGetAtt

	return the content of an attribute

=cut

sub THGetAtt
{
my( $twig, $section, $attribut)= @_;

        my $value = $section->{'att'}->{$attribut};
        return($value);
}

=item THGetElt

	return the content of an element

=cut

sub THGetElt
{
my( $twig, $section)= @_;

        my $value = $section->text();
        return($value);
}

1;

