#
# $Id: WebBuilder.pm 139 2007-05-17 10:19:56Z gouzy $
#

package WebBuilder;

# Emmanuel.Courcelle@toulouse.inra.fr
# Jerome.Gouzy@toulouse.inra.fr

# This software is a perl module whose purpose is to provide a
# perl framework for building web sites

# This software is governed by the CeCILL license under French law and
# abiding by the rules of distribution of free software.  You can  use,
# modify and/ or redistribute the software under the terms of the CeCILL
# license as circulated by CEA, CNRS and INRIA at the following URL
# "http://www.cecill.info".

# As a counterpart to the access to the source code and  rights to copy,
# modify and redistribute granted by the license, users are provided only
# with a limited warranty  and the software's author,  the holder of the
# economic rights,  and the successive licensors  have only  limited
# liability.

# In this respect, the user's attention is drawn to the risks associated
# with loading,  using,  modifying and/or developing or reproducing the
# software by the user in light of its specific status of free software,
# that may mean  that it is complicated to manipulate,  and  that  also
# therefore means  that it is reserved for developers  and  experienced
# professionals having in-depth computer knowledge. Users are therefore
# encouraged to load and test the software's suitability as regards their
# requirements in conditions enabling the security of their systems and/or
# data to be ensured and,  more generally, to use and operate it in the
# same conditions as regards security.

# The fact that you are presently reading this means that you have had
# knowledge of the CeCILL license and that you accept its terms.

=pod

=head1 NAME

WebBuilder - An object which manage CGI based web server design 
	
- structure of the web site is provided by a XML document
- layout is provided by a css file

=head1 SYNOPSIS

by default, only variables with name containing the prefix __wb_ are propagated through the menu options

=head1 DESCRIPTION

07-Oct-2005
add cookies managment

12-Oct-2005
multi css

04-Dec-2005
add session managment

01-Apr-2006
add SetLocalStyleSheet to add document specific style
add SetHeaderTitle to modify the main title

02-Apr-2006
add "behaviour" to the module
add url encoding behaviour (warning: __wb_function and __wb_main_menu must be not encoded)
add GetParameters

Feb-2007
add GetParams
add GetObjectParam
add AddJavascript and the javascript management
add InitSession
add SetFunction
XHTML 1.0 strict specification
minor change to :
	__GetMenuItem
	Realize
	__LoadDescription

March-2007
add relative path management

April-2007
fix bug caused by ForceArray on SetHeaderTitle
ATTENTION <tag>value</tag> are now forbidden into XML Descriptor except for title !
use <tag val="value" if you want to add new ones
TODO try another XML parser than XML::Simple to clarify code.
Add __PrintPatience
Add __HidePatience
	both functions deal with a div indicating that calculations are running

=cut


use warnings;
use strict;

use Carp;
use CGI;
use XML::Simple;
use Data::Dumper;
use ParamParser;

our %H_DEFBEHAVIOUR;
eval { use Convert::UU qw(uudecode uuencode); };
if ($@ eq "")
{
	$H_DEFBEHAVIOUR{'encoded_url'}         = 1;
}

use constant PATIENCE_ID	=>	"wb_patience";


BEGIN
{
	our $VERSION = do {my @r = (q$Rev: 212 $ =~ /\d+/g); $r[0]};
}

=head2 function New

	Title      :	New
	Usage      :	$o_webbuilder = New WebBuilder($o_param_parser);
	Prerequiste:	none
	Function   :	constructor of the object
	Returns    :	a valid WebBuilder object
	Args       :	$o_param_parser,	a valid ParamParser object
	Globals    :	none

=cut

sub New
{
	my ($pkg,$rh_param)=@_;

	my %__h_behaviour = %H_DEFBEHAVIOUR;

	$rh_param->SetUnlessDefined('__wb_main_menu',$rh_param->Get('__wb_function'));

	my $self = {
		__desc_file 	=> $rh_param->Get('__wb_xmlstructure'),
		__cookie    	=> $rh_param->Get('__wb_cookie'),
		__charset		=> "utf-8",
		__main_menu 	=> $rh_param->Get('__wb_main_menu'),
		__function  	=> $rh_param->Get('__wb_function'),
		__session   	=> $rh_param->Get('__wb_session'),
 		__tmp_path		=> "",
		__tmp_url 		=> "",
		__cgi 			=> "",
		__url_stylesheet=> "",
		__url_javascript=> [],
		__local_stylesheet => "",
		__h_behaviour => \%__h_behaviour,
		__rh_param  	=> $rh_param,
		__desc      	=> {}
	};

	bless($self,$pkg);

	$self->__LoadDescription();

	return($self);
}

=head2 procedure Realize

	Title      :	Realize
	Usage      :	$o_webbuilder->Realize();
	Prerequiste:	the page must have been configured through the XML file and dynamics functions
	Procedure  :	methods to print the HTML page.
                   	first, it checks session
                    	then, it writes HTML headers, WB headers (logos and menus)
                	after, it calls the function that print the body part of the HTML file
                	last, it closes all previous opened tags
                		_regardless of what the 'body-part-function' has printed.
	Returns    :	nothing
	Args       :	$o_param_parser,	a valid ParamParser object
	Globals    :	none

=cut

sub Realize
{
	my($self) = @_;

#	l'entete cgi sert a prevenir le serveur _apache_ qu'on affiche une page web
#	on definit tout de meme 2 fois le codage des caracteres !
	my $cgi = new CGI;
	if (defined($$self{'__cookie'}) && $$self{'__cookie'} ne ""  )
	{
		print $cgi->header(-cookie=>$$self{'__cookie'}, -charset=>$self->{__charset});
	}
	else
	{
		print $cgi->header(-charset=>$self->{__charset});
	}

	$self->InitSession()	if( defined($self->{__tmp_path}) && $self->{__tmp_path} ne "" );
	$self->__PrintHeader();
	$self->__PrintSections();
	$self->__PrintPatience();
	print "<div class=\"wb_content\">\n";

	if ( defined($$self{'__function'}) && $$self{'__function'} ne ""  )
	{
		##	retro-compatibility
		##	we have to check if __function is the function's name or a ref to the function
		my $ref_fn = $$self{'__function'};
		my $type_var = ref($ref_fn);

		if($type_var eq "CODE")
		{
			&$ref_fn($self);
		}
		### we have the function name !
		elsif($type_var eq "")
		{
			my $fct = "main::$ref_fn";
			no warnings;
			no strict "refs";
			\&{$fct}($self);
		}
		### unknown type -> error in the setFunction code
		else
		{
			print "$0 - Realize\n!!--$type_var--!!\n";
		}
	}
	print "\n</div>\t<!-- end wb_content -->\n";
	$self->__HidePatience();
	print "\n</body>\n</html>\n\n";

	return;
}


=head2 function GetUrl

	Title      :	GetUrl
	Usage      :	$url = $o_webbuilder->GetUrl($discarded_params);
	Prerequiste:	none
	Function   :	return the url of the script and parameters
                	depending on the behaviour 'encoded_url' the parameters can be encoded or not
	Returns    :	a string,	the url
	Args       :	$discarded_params,	a comma separated list of parameters to be discarded
	Globals    :	none

=cut

sub GetUrl
{
	my($self,$toremove) = @_;

	$toremove =~ s/,/\|/g;
	$toremove .= "|xmlstructure|__wb_url";
	$toremove =~ s/^\|//;
	$toremove =~ s/\|$//;

	my $url = $$self{'__cgi'}."?";
	$url .= $self->GetParameters($toremove,"");
	return $url;
}

=head2 function GetParameters

	Title      :	GetParameters
	Usage      :	$params = $o_webbuilder->GetParameters( $unwanted_param, $wanted_param );
	Prerequiste:	none
	Function   :	methods to get parameters in cgi format _encoded or not depending on the behaviour
                	can be more/less than all the parameters of the url
	Returns    :	a string,	the list of parameters in cgi format
	Args       :	$unwanted_param,a string, a comma separated list of parameters to be discarded
                	$wanted_param,	a string, an url formatted list of parameters to add
	Globals    :	none

=cut

sub GetParameters
{
my ($self,$toremove,$additional_parameters) = @_;

	if ( defined($toremove) )
	{
		$toremove =~ s/,/\|/g;
	}
	$toremove .= "|xmlstructure|__wb_url";
	$toremove =~ s/^\|//;
	$toremove =~ s/\|$//;

	my @a_param_keys = $$self{'__rh_param'}->GetKeys("__wb_");
	my $parameters = "";
	foreach my $key (sort @a_param_keys)
	{
		next if ( $key =~ /$toremove/ );
		$parameters .= "$key=".$$self{'__rh_param'}->Get($key)."&";
	}
	$parameters .= "$additional_parameters" if ( defined($additional_parameters) );
	$parameters =~ s/\&$//;
	if ( $self->GetBehaviour('encoded_url') )
	{
		$parameters = &ParamParser::Encode($parameters);
	}
	return $parameters;
}


=head2 function GetParams

	Title      :	GetParams
	Usage      :	$value = $o_webbuilder->GetParams( $var );
	Prerequiste:	none
	Function   :	accessor for variables of ParamParser object.
	Returns    :	the value of of $var in the ParamParser object
	Args       :	$var,	a string, the name of the variable
	Globals    :	none

=cut

sub GetParams
{
	my ($self, $param_to_get) = @_;

	return $$self{'__rh_param'}->Get($param_to_get);
}

=head2 function GetObjectParam

	Title      :	GetObjectParam
	Usage      :	$o_param_parser = $o_webbuilder->GetObjectParam();
	Prerequiste:	none
	Function   :	accessor for ParamParser object.
	Returns    :	the ParamParser object
	Args       :	none
	Globals    :	none

=cut

sub GetObjectParam
{
	my ($self, $param_to_get) = @_;

	return $$self{'__rh_param'};
}

=head2 function GetCgi

	Title      :	GetCgi
	Usage      :	$cgi_url = $o_webbuilder->GetCgi();
	Prerequiste:	none
	Function   :	accessor for the url of the cgi.
	Returns    :	return the url of the cgi
	Args       :	none
	Globals    :	none

=cut

sub GetCgi
{
my($self) = @_;

	return $$self{'__cgi'};
}

=head2 Procedure ClearSession

	Title      :	ClearSession
	Usage      :	$o_webbuilder->ClearSession();
	Prerequiste:	none
	Procedure  :	(re)initialize the session
	Returns    :	nothing
	Args       :	none
	Globals    :	none

=cut

sub ClearSession
{
	my($self) = @_;

	$$self{'__session'} = "";
	$$self{'__rh_param'}->Set('__wb_session',$$self{'__session'});

	return;
}

=head2 function GetSession

	Title      :	GetSession
	Usage      :	$session_id = $o_webbuilder->GetSession();
	Prerequiste:	none
	Function   :	accessor for the session id
	Returns    :	return the session id
	Args       :	none
	Globals    :	none

=cut

sub GetSession
{
my($self) = @_;

	return $$self{'__session'};
}

=head2 function GetSessionUrl

	Title      :	GetSessionUrl
	Usage      :	$session_url = $o_webbuilder->GetSessionUrl();
	Prerequiste:	none
	Function   :	accessor for the url of the session directory
	Returns    :	return the url of the session directory
	Args       :	none
	Globals    :	none

=cut

sub GetSessionUrl
{
	my($self) = @_;

	if ( $$self{'__tmp_url'} ne "" && $$self{'__session'} ne "" )
	{
		return $$self{'__tmp_url'}.'/'.$$self{'__session'};
	}
	return undef;
}

=head2 function GetSessionPath

	Title      :	GetSessionPath
	Usage      :	$session_path = $o_webbuilder->GetSessionPath();
	Prerequiste:	none
	Function   :	accessor for the unix path (absolute) of the session directory
	Returns    :	return the (absolute) unix path of the session directory
	Args       :	none
	Globals    :	none

=cut

sub GetSessionPath
{
	my($self) = @_;

	# if something is not defined just don't compute anything
	if ( (!defined($$self{'__tmp_path'})) or ($$self{'__tmp_path'} eq "")
		or (!defined($$self{'__session'})) or ($$self{'__session'} eq ""))
	{
		return undef;
	}
	return $$self{'__tmp_path'} . '/' . $$self{'__session'};
}

=head2 function GetForm

	Title      :	GetForm
	Usage      :	$html_form = $o_webbuilder->GetForm();
	Prerequiste:	none
	Function   :	Get the form formated html code corresponding to application and function parameters
	Returns    :	Get the form formated html code corresponding to application and function parameters
	Args       :	none
	Globals    :	none

=cut

sub GetForm
{
my($self,$cgi,$string_inside_the_form,$label,$input_name,$input_size) = @_;

	my $str = "<form method=\"post\" action=\"$cgi\">\n";
	if ( defined($input_name) )
	{
		$str .= "$label<input name=$input_name";
		if ( defined($input_name) ) 
		{
			$str .= " size=$input_size";
		}
		$str .= ">\n";
	}
	if ( $self->GetBehaviour('encoded_url') )
	{
			$str .= "\t\t<input name=\"__wb_url\" type=\"hidden\" value=\"".$self->GetParameters()."\">\n";
	}
	else
	{
		my @a_param_keys = $$self{'__rh_param'}->GetKeys("__wb_");
		foreach my $kkey (sort @a_param_keys)
		{
			$str .= "\t\t<input name=\"$kkey\" type=\"hidden\" value=\"".$$self{'__rh_param'}->Get($kkey)."\">\n";
		}
	}
	$str .= "$string_inside_the_form\n";
	$str .= "</form>\n";
	return $str;
}

=head2 Procedure SetDocumentStyleSheet

	Title      :	SetDocumentStyleSheet
	Usage      :	$o_webbuilder->SetDocumentStyleSheet($style_tag);
	Prerequiste:	none
	Procedure  :	Initialize the internal document stylesheet from the parameter string
	Returns    :	nothing
	Args       :	$style_tag,	a string, an inclusion of css
                	example : $style_tag = '<style type="text/css"> body { background-color:#FFFFCC;} </style>'
	Globals    :	none

=cut

sub SetDocumentStyleSheet
{
	my($self,$string) = @_;

	$$self{'__local_stylesheet'} = $string;

	return;
}

=head2 Procedure AddJavascript

	Title      :	AddJavascript
	Usage      :	$o_webbuilder->AddJavascript($rh_js_file1 [,$rh_js_file2, ...]);
			or $o_webbuilder->AddJavascript( {url=>$url1[, comment=>$comment1]} [, {url=>$url2, comment=>$comment2}]);
	Prerequiste:	none
	Procedure  :	Add one extern javascript file
	Returns    :	nothing
	Args       :	$rh_js_file,	a reference to a h table.
                	this hash must have 2 keys as following
                	( url => "path to the file", comment => "any comment for this file, like copyright")
			Remarque : you can use more than one reference and the comment key is optional.
	Globals    :	none

=cut

sub AddJavascript
{
	my($self, @a_rh_js) = @_;

	my $ra_file_list = $$self{'__url_javascript'};

	push(@$ra_file_list, @a_rh_js);

	return;
}

=head2 Procedure SetFunction

	Title      :	SetFunction
	Usage      :	$o_webbuilder->SetFunction($ref_fn);
	Prerequiste:	none
	Procedure  :	ReInitialize the internal wb_function from the parameter string
	Returns    :	nothing
	Args       :	$ref_fn,	a reference to or the name of the function
                		in fact it can be call like SetFunction(\&FunctionName);
                		or even like SetFunction('FunctionName');
	Globals    :	none

=cut

sub SetFunction
{
	my($self,$string) = @_;

	$$self{'__function'} = $string;

	return;
}

=head2 Procedure SetHeaderTitle

	Title      :	SetHeaderTitle
	Usage      :	$o_webbuilder->SetHeaderTitle($title);
	Prerequiste:	none
	Procedure  :	Modify/overwrite the header/title read from the xml file
	Returns    :	nothing
	Args       :	$title,	a string, the new title
	Globals    :	none

=cut

sub SetHeaderTitle
{
	my($self,$string) = @_;

	$$self{'__desc'}->{header}[0]->{title}[0] = $string;

	return;
}

=head2 Procedure SetBehaviour

	Title      :	SetBehaviour
	Usage      :	$o_webbuilder->SetBehaviour($key);
	Prerequiste:	global Hash $H_DEFBEHAVIOUR must be defined
	Procedure  :	Control the behaviour of the module
                	for now it's just the encoding of the url
                	when set, the url is encoded (default if module Convert:UU available)
	Returns    :	nothing
	Args       :	$key,	a string,	nowaday only 'encoded_url'
	Globals    :	none

=cut

sub SetBehaviour
{
    my ($self, $key) = @_;

    return unless (&WebBuilder::__ValidBehaviour($key));
    $$self{'__h_behaviour'}{$key} = 1;

	return;
}

=head2 function GetBehaviour

	Title      :	GetBehaviour
	Usage      :	$behaviour_is_set = $o_webbuilder->GetBehaviour($key);
	Prerequiste:	global Hash $H_DEFBEHAVIOUR must be defined
	function   :	accessor for the behaviour
                	for now it's just the encoding of the url
	Returns    :	1 if the behaviour whose name is passed by parameter is set, 0 if not set
	Args       :	$key,	a string,	nowaday only 'encoded_url'
	globals    :	none

=cut

sub GetBehaviour
{
    my ($self, $key) = @_;
    return 0 unless (&WebBuilder::__ValidBehaviour($key));
    return $$self{'__h_behaviour'}{$key};
}

=head2 procedure UnsetBehaviour

	Title      :	UnsetBehaviour
	Usage      :	$o_webbuilder->UnsetBehaviour($key);
	Prerequiste:	global Hash $H_DEFBEHAVIOUR must be defined
	Procedure  :	unset the behaviour of the parameter
                	for now it's just the encoding of the url, if unset, the url is not encoded 
	Returns    :	nothing
	Args       :	$key,	a string,	nowaday only 'encoded_url'
	globals    :	none

=cut

sub UnsetBehaviour
{
    my ($self, $key) = @_;

    return unless (&WebBuilder::__ValidBehaviour($key));
    $$self{'__h_behaviour'}{$key} = 0;

	return;
}


=head2 procedure InitSession

	Title      :	InitSession
	Usage      :	$o_web_builder->InitSession();
	Prerequiste:	none
	Procedure  :	initialize the session, unless already initialized
                	Create the session directory, unless already created
	Returns    :	nothing
	Args       :	none
	globals    :	none

=cut

sub InitSession
{
	my($self) = @_;

	# si la session n'existe pas on la cree
	if ( ! defined($$self{'__session'}) || $$self{'__session'} eq "" )
	{
		$$self{'__session'} = "WB".time.$$;
		$$self{'__rh_param'}->Set('__wb_session',$$self{'__session'});
	}

	# on recupere le chemin absolu de session
	my $dirsession = $self->GetSessionPath();
	die "ERROR - session directory is not defined"	if ( !defined($dirsession) );

	# if the session dir does not exist, we create one
	if ( !-e $dirsession )
	{
		mkdir $dirsession or die "ERROR - Cannot make the directory $dirsession";
	}

	# now that we are sure that $dirsession exist, we go in it
	chdir($dirsession);

	return;
}


=head2 procedure __PrintSections

	Title      :	__PrintSections
	Usage      :	$self->__PrintSections();
	Prerequiste:	none
	Procedure  :	print all the second part of the page ie wb_header2�= the menu
	Returns    :	nothing
	Args       :	none
	globals    :	none

=cut

sub __PrintSections
{
	my($self) = @_;
	my $second_level_menu = "";

	my $main_menu = $$self{'__desc'}->{main_menu_item};
	print "<div id=\"wb_header2\">\n";
	print "\t<ul class=\"wb_main\">\n";

	# i sue this base of string to save space
	my $partial_cgi_start = $$self{'__cgi'}."?";
	$partial_cgi_start .= $self->GetParameters("main_menu,function","");

	foreach my $key (sort {  $b cmp $a } keys(%$main_menu))
	{
		my $active_li = "";
		my $active_a = " class=\"wb_main_a_inactive\" ";
		my $url;

		# I complete the cgi url with one more parameter
		my $partial_cgi = $partial_cgi_start;
		$partial_cgi .= "&amp;__wb_main_menu=".$$main_menu{$key}->{function};

		if ( $$main_menu{$key}->{function} eq $$self{'__main_menu'} )
		{
			$active_li = " class=\"wb_main_li_active\" ";
			$active_a = " class=\"wb_main_a_active\" ";
			my $rh_item = $$main_menu{$key}->{sub_menu_item};
			foreach my $item (sort keys(%$rh_item))
			{
				# i add the last parameter of the cgi url
				my $cgi_down = $partial_cgi;
				$cgi_down .= "&amp;__wb_function=".$$rh_item{$item}->{function};
				my $str = $self->__GetMenuItem("menu",$cgi_down,"","",\$$rh_item{$item});
				$second_level_menu .= $str;
			}
		}
		# i add the last parameter of the cgi url
		my $cgi_up = $partial_cgi;
		$cgi_up .= "&amp;__wb_function=".$$main_menu{$key}->{function};
		print $self->__GetMenuItem("main_menu",$cgi_up,$active_li,$active_a,\$$main_menu{$key});
	}
	print "\t</ul>\n</div>\n\n";

	if ( $second_level_menu ne "" )
	{
		print "<div class=\"wb_sub\">\n";
		print "<ul>\n";
		print $second_level_menu;
		print "</ul>\n";
		print "</div>\n\n";
	}

	return ;
}

=head2 function __GetMenuItem

	Title      :	__GetMenuItem
	Usage      :	$s_menu = $self->__GetMenuItem($type,$cgi,$cgi_only,$active_li,$active_a,$rh_item);
	Prerequiste:	none
	Function   :	return a string that contains the html code to print for a single menu item
	Returns    :	a string that contains the html code to print for a single menu item
	Args       :	$type,	a string "menu" for non activated link
                	$cgi,	a string containing the url of the cgi with some parameters
                	$active_li,	a string containing qualification for li tag (" class='...'") for a currently activated link
                	$active_a,		a string containing qualification for a tag (" class='...'") for a currently activated link
                	$rh_item,	hash reference of submenu
	globals    :	none

=cut

sub __GetMenuItem
{
	my ($self,$type,$cgi,$active_li,$active_a,$rh_item) = @_;
	my $str = "";
	my $url;

	$url = ($$rh_item->{function} eq "extern") ? $$rh_item->{url} : $cgi;

	if ( $$rh_item->{function} =~ /^WBForm/)
	{
		my $cgi_to_run = ( defined($$rh_item->{url}) && $$rh_item->{url} ne "" ) ? $$rh_item->{url} : $$self{'__cgi'} ;
		$str ="\t\t<li class=\"wb_li_form\">".$self->GetForm($cgi_to_run,"",$$rh_item->{label},$$rh_item->{input_name},$$rh_item->{input_size})."</li>\n";
	}
	else
	{
		if ( $type eq "menu" )
		{
			$str = "\t<li><a href=\"$url\">$$rh_item->{content}</a><br/></li>\n";
		}
		else
		{
			print "\t\t<li$active_li><a $active_a href=\"$url\">".$$rh_item->{label}."</a></li>\n";
		}
	}
	return $str;
}


=head2 procedure __PrintHeader

	Title      :	__PrintHeader
	Usage      :	$self->____PrintHeader();
	Prerequiste:	none
	Procedure  :	print out all the HTML header and the first part of WB header
	Returns    :	nothing
	Args       :	none
	Globals    :	none

=cut

sub __PrintHeader
{
	my($self) = @_;

print<<EOF
<?xml version="1.0" encoding="$self->{__charset}"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
	<meta http-equiv="Content-Type" content="text/html;charset='$self->{__charset}'" />
EOF
;
	# I use a variable to speed the display
	# _usually one print of a big variable is faster than many prints of small variables -Seb-
	my $external_script = "";
	#css
	my $css   = $$self{'__url_stylesheet'};
	foreach my $key (sort keys(%$css))
	{
		$external_script .= "\t<link href=\"".$$css{$key}->{url}."\" rel=\"stylesheet\" type=\"text/css\" />\n";
	}
	#javascript
	my $ra_javascript = $$self{'__url_javascript'};
	foreach my $rh_js_file (@$ra_javascript)
	{
		$external_script .= "\t<script type=\"text/javascript\" src=\"".$rh_js_file->{url}."\"></script>";
		#integration of commentaries essentially used to report copyright
		my $comment = $rh_js_file->{comment};
		#the comment follows the script tag
		$external_script .= "<!-- $comment -->"	if( defined($comment) );
		$external_script .= "\n";
	}

#	my $javascript = $$self{'__url_javascript'};
#	foreach my $key (sort keys(%$javascript))
#	{
#		$external_script .= "\t<script type=\"text/javascript\" src=\"".$$javascript{$key}->{url}."\"></script>";
#		#integration of commentaries essentially used to report copyright
#		my $comment = $$javascript{$key}->{comment};
#		#the comment follows the script tag
#		$external_script .= "<!-- $comment -->"	if( defined($comment) );
#		$external_script .= "\n";
#	}

	# [0] has been added to fit with the ForceArray option
	my $title = $$self{'__desc'}->{header}[0]->{title}[0];
	my $local_css = $$self{'__local_stylesheet'};

print<<EOF
	$local_css
	$external_script
	<title>$title</title>
</head>
<body>
EOF
;

	print "<div id=\"wb_header1\">\n";
	print "<p id=\"wb_title\">$title</p>\n";
	my $logo = $$self{'__desc'}->{header}[0]->{logo};
	foreach my $key2 (sort keys(%$logo))
	{
		print "\t<a href=\"".$$logo{$key2}->{link}."\"><img src=\"".$$logo{$key2}->{img_src}."\" class=\"wb_logo\" alt=\"".$$logo{$key2}->{alt}."\" height=\"".$$logo{$key2}->{height}."\"  /></a>\n";
	}
	print "</div>\n\n";

	return;
}

=head2 procedure __LoadDescription

	Title      :	__LoadDescription
	Usage      :	$self->__LoadDescription();
	Prerequiste:	none
	Procedure  :	parse XML file and fill object attribute with the read information.
	Returns    :	nothing
	Args       :	none
	Globals    :	none

=cut

sub __LoadDescription
{
my($self) = @_;

	$$self{'__desc'}=XMLin($$self{'__desc_file'}, ValueAttr => ['id'], ForceArray => 1 );

# [0] has been added to fit with the ForceArray option

	if ( defined($$self{'__desc'}->{tmp}[0]->{path}) )
	{	
		$$self{'__tmp_path'} = $$self{'__desc'}->{tmp}[0]->{path};	
	}
	$$self{'__tmp_url'}  = $$self{'__desc'}->{tmp}[0]->{url}  if ( defined($$self{'__desc'}->{tmp}[0]->{url}));
	$$self{'__cgi'} = $$self{'__desc'}->{cgi}[0]->{url};
	$$self{'__url_stylesheet'} = $$self{'__desc'}->{stylesheet}[0]->{css};
	
	# id for javascript are sensless, but it must be used in the XML
	# so we have to change the h_tab into a list of rh_tab
	my $rh_tmp = $$self{'__desc'}->{javascript}[0]->{js};
	# @a_list is a list of reference of hash
	my @a_rh_list = values (%$rh_tmp);
	
	# now we have to add this current list to the list known by the object
	my $ra_files_list = $$self{'__url_javascript'};
	push(@$ra_files_list, @a_rh_list);

	# i have to set absolute path here because it seems to be impossible to call it twice
	# unless it is caused by calling it after chdir ? -Seb
    if ( defined($$self{'__tmp_path'}) and $$self{'__tmp_path'} ne '' and ($$self{'__tmp_path'} !~ /^\// ) )
    {
        use Cwd 'abs_path';
        $$self{'__tmp_path'} = abs_path($$self{'__tmp_path'});
    }

#debug
# 	print Dumper($self);

	return;
}


=head2 function __ValidBehaviour

	Title      :	__ValidBehaviour
	Usage      :	$behaviour_is_valid = &WebBuilder::__ValidBehaviour($behaviour);
	Prerequiste:	global $H_DEFBEHAVIOUR must have been declared, it's a function not $self arg !
	function   :	tell if the $behaviour is known/valid.
                	it is 'class restricted', nothing outside of this class have to use it
	Returns    :	boolean, 1 if $behaviour is known, 0 otherwise
	Args       :	$behaviour,	a string, the behaviour to judge
	globals    :	%H_DEFBEHAVIOUR, a hash of behaviours

=cut

sub __ValidBehaviour
{
    my $key = shift;
    return 1 if (exists $H_DEFBEHAVIOUR{$key});
    return 0;
}

=head2 procedure __Dump

	Title      :	__Dump
	Usage      :	$self->__Dump();
	Prerequiste:	none
	Procedure  :	Print the structure of the layout
	Returns    :	boolean, 1 if $behaviour is known, 0 otherwise
	Args       :	$behaviour,	a string, the behaviour to judge
	Globals    :	%H_DEFBEHAVIOUR, a hash of behaviours

=cut

sub __Dump
{
my($self) = @_;

	print Dumper($self);

	return;
}


=head2 procedure SetCharset

	Title      :	SetCharset
	Usage      :	$self->SetCharset($charset);
	Prerequiste:	none
	Procedure  :	Set the character encoding to the norme $charset, default used by WB is utf-8
	Returns    :	nothing
	Args       :	$charset,	a string representing a norme of coding like utf-8 or iso-8859-1
	Globals    :	none

=cut

sub SetCharset
{
	my $self = shift;
	my ($charset) = @_;

	$self->{__charset} = $charset;
}


=head2 procedure __PrintPatience

	Title      :	__PrintPatience
	Usage      :	$self->__PrintPatience();
	Prerequiste:	should be called only into Realize()
	Procedure  :	print a div containing a message and/or an image
	            	that will be hidden, at the end of Realize
	Returns    :	nothing
	Args       :	none
	Globals    :	none

=cut

sub __PrintPatience
{
	my $self = shift;
	my $id = &PATIENCE_ID;
	$| = 1;	# STDOUT must be unbuffered

	my $msg = $$self{'__desc'}->{patience}[0]->{msg};
	my $img_src = $$self{'__desc'}->{patience}[0]->{img}[0]->{src};
	my $img_alt = $$self{'__desc'}->{patience}[0]->{img}[0]->{alt};

	$msg = '' unless defined($msg);
	$img_src = '' unless defined($img_src);
	$img_alt = '' unless defined($img_alt);

	# no patience message
	return if ($msg . $img_src . $img_alt eq '');

	my $patience = "<div id='$id'>\n";

	if($img_src . $img_alt ne '')
	{	$patience .= "\t<img src='$img_src' alt='$img_alt' />\n";	}

	if($msg ne '')
	{	$patience .= "\t$msg\n";	}

	print "$patience</div>\n";
	return;
}


=head2 procedure __HidePatience

	Title      :	__HidePatience
	Usage      :	$self->__HidePatience();
	Prerequiste:	should be called only into Realize()
	            	javascript should be enabled
	Procedure  :	Hide the "patience" div that contains a message and/or an image
	Returns    :	nothing
	Args       :	none
	Globals    :	none

=cut

sub __HidePatience
{
	my $self = shift;
# 	my () = @_;

	my $id = &PATIENCE_ID;

print <<HTML
<script type="text/javascript">
<!--
var obj = document.getElementById("$id");

obj.style.visibility = "hidden";

//-->
</script>
HTML
;

	return;
}


1;
